﻿/*
 * This source file (Chart.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.UI.WebControls;
using System.Drawing;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// Allows for displaying of results in a flash chart
    /// </summary>
    public partial class Chart : CubedResult
    {
        #region Properties

        /// <summary>
        /// This ensures that when the ID of this Chart Add In gets set, the sub-control chart's ID
        /// also get's set
        /// </summary>
        public override string ID
        {
            get
            {
                return base.ID;
            }
            set
            {
                base.ID = value;
                ChartControl.ID = this.ID + "_ChartControl";
            }
        }

        /// <summary>
        /// Returns the unique identifier of the Add In
        /// </summary>
        public override Guid AddInID
        {
            get { return new Guid("b5560706-b841-4648-a497-9e39f659d599"); }
        }

        /// <summary>
        /// Returns the name of the Add In
        /// </summary>
        public override string Name
        {
            get { return "Chart"; }
        }

        /// <summary>
        /// Returns a description of the Add In
        /// </summary>
        public override string Description
        {
            get { return "Allows for displaying of results in a series of flash charts"; }
        }

        /// <summary>
        /// Returns the current version of the Add In
        /// </summary>
        public override double CurrentVersion
        {
            get { return 1.1; }
        }

        /// <summary>
        /// Returns the name of the user control containing the Add In's markup
        /// </summary>
        public override string UserControlName
        {
            get { return "Chart.ascx"; }
        }

        /// <summary>
        /// Returns the name of the image to display in the search results for this item
        /// </summary>
        public override string SearchLogo
        {
            get { return "chart_preview.png"; }
        }

        /// <summary>
        /// Determines whether this chart should be displayed in dashboard mode (without any
        /// options for change) or standard item mode
        /// </summary>
        public override bool DashboardMode
        {
            get
            {
                if (Session[this.ID + "_ChartDashboardMode"] == null)
                    Session[this.ID + "_ChartDashboardMode"] = false;
                return (bool)Session[this.ID + "_ChartDashboardMode"];
            }
            set
            {
                Session[this.ID + "_ChartDashboardMode"] = value;
            }
        }

        /// <summary>
        /// The results which are displayed in this chart
        /// </summary>
        public override object Results
        {
            get
            {
                if (Session[this.ID + "_ChartResults"] == null)
                    Session[this.ID + "_ChartResults"] = new DataTable();
                return (object)Session[this.ID + "_ChartResults"];
            }
            set
            {
                Session[this.ID + "_ChartResults"] = value;
            }
        }

        /// <summary>
        /// The current display settings for the chart control
        /// </summary>
        public ChartDisplaySettings DisplaySettings
        {
            get
            {
                if (Session[this.ID + "_ChartDisplaySettings"] == null)
                    Session[this.ID + "_ChartDisplaySettings"] = new ChartDisplaySettings();
                return (ChartDisplaySettings)Session[this.ID + "_ChartDisplaySettings"];
            }
            set
            {
                Session[this.ID + "_ChartDisplaySettings"] = value;
            }
        }

        /// <summary>
        /// The identifier of the search this grid is a part of
        /// </summary>
        public override Guid SearchID
        {
            get
            {
                if (Session[this.ID + "_SearchID"] == null)
                    Session[this.ID + "_SearchID"] = Guid.Empty;
                return (Guid)Session[this.ID + "_SearchID"];
            }
            set
            {
                Session[this.ID + "_SearchID"] = value;
            }
        }

        #endregion

        #region Implemented Methods

        /// <summary>
        /// Returns a Base64 encoded version of our current settings
        /// </summary>
        /// <returns>A Base64 encoded string containing the current settings</returns>
        public override string SaveWebDisplaySettings()
        {            
            // First update our settings
            if (SelectedXAxis != null && SelectedYAxis != null &&
                SelectedXAxis.SelectedIndex > -1 && SelectedYAxis.SelectedIndex > -1)
            {
                // Axis members have been selected so choose update our settings
                DisplaySettings.DefaultXAxis = SelectedXAxis.Text;
                DisplaySettings.DefaultYAxis = SelectedYAxis.Text;

                // Then update our chart type
                DisplaySettings.Chart = (ChartType)Enum.Parse(typeof(ChartType),
                    SelectedChart.Text);

                // Then update our decimal places
                int decimalplaces = 2;
                int.TryParse(DecimalPlaces.Text, out decimalplaces);
                DisplaySettings.DecimalPlaces = decimalplaces;
            }
            else
            {
                // No members have been selected to set empty defaults
                DisplaySettings.DefaultXAxis = String.Empty;
                DisplaySettings.DefaultYAxis = String.Empty;
                DisplaySettings.Chart = ChartType.Column3D;
                DisplaySettings.DecimalPlaces = 2;
            }

            // Then return our settings
            return DisplaySettings.ToString();
        }

        /// <summary>
        /// Given the Base64 string containing settings, updates the chart with those settings
        /// </summary>
        /// <param name="settings">A Base64 encoded version of the current settings</param>
        /// <returns>True if updated successfully</returns>
        public override bool LoadWebDisplaySettings(string settings)
        {
            // Then read our settings
            DisplaySettings.FromString(settings);

            // Check if our Axes have been loaded and load them if not
            if (SelectedXAxis.Items.Count < 1 || SelectedYAxis.Items.Count < 1 ||
                SelectedChart.Items.Count < 1)
                PrepareResults();            

            // Then update our Axes if they exist
            if (SelectedXAxis.Items.FindByText(DisplaySettings.DefaultXAxis) != null)
                SelectedXAxis.Text = DisplaySettings.DefaultXAxis;
            if (SelectedYAxis.Items.FindByText(DisplaySettings.DefaultYAxis) != null)
                SelectedYAxis.Text = DisplaySettings.DefaultYAxis;
            if (SelectedChart.Items.FindByText(DisplaySettings.Chart.ToString()) != null)
                SelectedChart.Text = DisplaySettings.Chart.ToString();

            // Then update our decimal places
            DecimalPlaces.Text = DisplaySettings.DecimalPlaces.ToString();

            return true;
        }

        /// <summary>
        /// Returns the current result as a bitmap image ready for inserting into a consolidated document
        /// </summary>
        /// <returns>A bitmap representation of the current chart</returns>
        public override Bitmap GetResultImage()
        {
            return new Bitmap(400, 400);
        }

        #endregion

        #region Events

        /// <summary>
        /// Renders the results to the chart if any results have been specified
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // Then attempt to load our results
            PrepareResults();

            // Then check to see if we should display the pivot and chart options. These should only be displayed
            // when we are using an MDDataSet and we're not in dashboard mode
            if (!DashboardMode && Results is Structures.XMLA.MDDataSet)
            {
                AxisChooserRow.Visible = true;
                ChartChooserRow.Visible = true;
                SpacerRow.Visible = true;
            }
            else if (!DashboardMode)
            {
                AxisChooserRow.Visible = false;
                ChartChooserRow.Visible = true;
                SpacerRow.Visible = true;
            }
            else
            {
                AxisChooserRow.Visible = false;
                ChartChooserRow.Visible = false;
                SpacerRow.Visible = false;
            }

            // Check to see if we have any results and display the no results label if none are 
            // available
            if ((Results is DataSet && ((DataSet)Results).Tables.Count > 0 &&
                ((DataSet)Results).Tables[0].Rows.Count > 0) ||
                (Results is DataTable && ((DataTable)Results).Rows.Count > 0) ||
                (Results is Structures.XMLA.MDDataSet && ((Structures.XMLA.MDDataSet)Results).Axes.Count > 0 &&
                ((Structures.XMLA.MDDataSet)Results).Axes[0].Members.Count > 0))
            {
                ChartControl.Visible = true;
                NoResultsLabel.Visible = false;
            }
            else
            {                
                ChartControl.Visible = false;
                NoResultsLabel.Visible = true;
                AxisChooserRow.Visible = false;
                ChartChooserRow.Visible = false;
                SpacerRow.Visible = false;
            }
        }

        /// <summary>
        ///  When the selected X axis changes, pivot the results
        /// </summary>
        protected void SelectedXAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update our display settings
            string oldx = DisplaySettings.DefaultXAxis;
            DisplaySettings.DefaultXAxis = SelectedXAxis.Text;

            // Then, if the newly selected X axis is the same as the Y axis, change the Y to the 
            // old X-Axis
            if (DisplaySettings.DefaultXAxis.ToUpper() == DisplaySettings.DefaultYAxis.ToUpper())
            {
                DisplaySettings.DefaultYAxis = oldx;
                SelectedYAxis.Text = oldx;
            }
        }

        /// <summary>
        ///  When the selected Y axis changes, pivot the results
        /// </summary>
        protected void SelectedYAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update our display settings
            string oldy = DisplaySettings.DefaultYAxis;
            DisplaySettings.DefaultYAxis = SelectedYAxis.Text;

            // Then, if the newly selected Y axis is the same as the X axis, change the X to the 
            // old Y-Axis
            if (DisplaySettings.DefaultYAxis.ToUpper() == DisplaySettings.DefaultXAxis.ToUpper())
            {
                DisplaySettings.DefaultXAxis = oldy;
                SelectedXAxis.Text = oldy;
            }
        }

        /// <summary>
        /// When the selected chart type changes, update the settings
        /// </summary>
        protected void SelectedChart_SelectedIndexChanged(object sender, EventArgs e)
        {
            DisplaySettings.Chart = (ChartType)Enum.Parse(typeof(ChartType), SelectedChart.Text);
        }

        #endregion

        #region Support Methods

        /// <summary>
        /// This prepares our control based on the given results
        /// </summary>
        private void PrepareResults()
        {
            // First prepare our results data table
            DataTable results = new DataTable();
            bool multidimensional = false;
            if (Results is DataTable)
                results = (DataTable)Results;
            else if (Results is DataSet && ((DataSet)Results).Tables.Count > 0)
                results = ((DataSet)Results).Tables[0];
            else if (Results is Structures.XMLA.MDDataSet)
            {
                results = ((Structures.XMLA.MDDataSet)Results).ToDataSet().Tables[0];
                multidimensional = true;
            }

            // Then load our Axis definitions for selected values
            SelectedXAxis.Items.Clear();
            SelectedYAxis.Items.Clear();
            if (Results is Structures.XMLA.MDDataSet)
            {
                // Since the results are a multi-dimensional data set, add the axes from the data set
                foreach (Structures.XMLA.Axis thisaxis in ((Structures.XMLA.MDDataSet)Results).Axes)
                {
                    SelectedXAxis.Items.Add(thisaxis.Name);
                    SelectedYAxis.Items.Add(thisaxis.Name);
                }
            }
            else
            {
                // As the results are not a multi-dimensional dataset, just add 2 default axes
                SelectedXAxis.Items.Add("Axis 1");
                SelectedYAxis.Items.Add("Axis 1");
                SelectedXAxis.Items.Add("Axis 2");
                SelectedYAxis.Items.Add("Axis 2");
            }
            
            // Then load our Chart Types
            SelectedChart.Items.Clear();
            foreach (string thischart in Enum.GetNames(typeof(ChartType)))
                SelectedChart.Items.Add(new ListItem(thischart, thischart));
         
            // Then, if the selected axes are available, select them
            if (SelectedXAxis.Items.FindByText(DisplaySettings.DefaultXAxis) != null)
                SelectedXAxis.Text = DisplaySettings.DefaultXAxis;
            else if (SelectedXAxis.Items.Count > 0)
            {
                SelectedXAxis.SelectedIndex = 0;
                DisplaySettings.DefaultXAxis = SelectedXAxis.Text;
            }
            if (SelectedYAxis.Items.FindByText(DisplaySettings.DefaultYAxis) != null)
                SelectedYAxis.Text = DisplaySettings.DefaultYAxis;
            else if (SelectedYAxis.Items.Count > 1)
            {
                SelectedYAxis.SelectedIndex = 1;
                DisplaySettings.DefaultYAxis = SelectedYAxis.Text;
            }
            else if (SelectedYAxis.Items.Count > 0)
            {
                SelectedYAxis.SelectedIndex = 0;
                DisplaySettings.DefaultYAxis = SelectedYAxis.Text;
            }

            // Then if the selected chart is available, select it
            if (SelectedChart.Items.FindByText(DisplaySettings.Chart.ToString()) != null)
                SelectedChart.Text = DisplaySettings.Chart.ToString();
            else if (SelectedChart.Items.Count > 0)
                SelectedChart.SelectedIndex = 0;

            // Finally, provided axes have been selected and we have an MDDataSet - ensure the
            // correct axes have been pivoted
            if (SelectedXAxis.SelectedIndex > -1 && SelectedYAxis.SelectedIndex > -1 &&
                Results is Structures.XMLA.MDDataSet)
            {
                results = ((Structures.XMLA.MDDataSet)Results).ToDataSet(SelectedXAxis.Text,
                    SelectedYAxis.Text).Tables[0];
            }

            // Now bind our results
            ChartControl.ChartType = DisplaySettings.Chart;
            ChartControl.DecimalPlaces = DisplaySettings.DecimalPlaces;
            ChartControl.MultiDimensional = multidimensional;
            ChartControl.Datasource = results;            
            ChartControl.DataBind();
        }

        #endregion                                
    }
}