/*
 * This source file (GaugeDisplaySettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.UI.WebControls;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// This class stores the required settings for displaying the Gauge Viewer
    /// </summary>
    [Serializable]
    public class GaugeDisplaySettings
    {
        private Double _maxValue = 0;
        /// <summary>
        /// Gets or sets the maximum value of the Gauge.
        /// </summary>
        public Double MaxValue
        {
            get { return _maxValue; }
            set
            {
                if (_maxValue != value)
                {
                    _maxValue = value;
                }
            }
        }

        private Double _stepValue = 100;
        /// <summary>
        /// Gets or Sets the Step Value of the Gauge.
        /// </summary>
        public Double StepValue
        {
            get { return _stepValue; }
            set
            {
                if (_stepValue != value)
                {
                    _stepValue = value;
                }
            }
        }

        private GaugeValueType _valueType = GaugeValueType.OriginalValue;
        /// <summary>
        /// Gets or sets the Value Type to display on the Gauge.
        /// </summary>
        public GaugeValueType ValueType
        {
            get { return _valueType; }
            set
            {
                if (_valueType != value)
                {
                    _valueType = value;
                }
            }
        }

        private GaugeRangesDisplayType _rangesDisplay = GaugeRangesDisplayType.Normal;
        /// <summary>
        /// Gets or sets the way the Ranges must display.
        /// </summary>
        public GaugeRangesDisplayType RangesDisplay
        {
            get { return _rangesDisplay; }
            set
            {
                if (_rangesDisplay != value)
                {
                    _rangesDisplay = value;
                }
            }
        }

        /// <summary>
        /// This sets the settings based on the passed in Base64 encoded string
        /// </summary>
        /// <param name="pSettings">A base64 encoded string containing settings</param>
        public void FromString(string pSettings)
        {
            string regularstring = UnicodeEncoding.Unicode.GetString(Convert.FromBase64String(pSettings));
            string[] settingsstrings = regularstring.Split(';');
            if (settingsstrings.Length >= 3)
            {
                _maxValue = Double.Parse(settingsstrings[0]);
                _stepValue = Double.Parse(settingsstrings[1]);
                _valueType = (GaugeValueType)Enum.ToObject(typeof(GaugeValueType), Int32.Parse(settingsstrings[2]));
                if (settingsstrings.Length >= 4)
                {
                    _rangesDisplay = (GaugeRangesDisplayType)Enum.ToObject(typeof(GaugeRangesDisplayType), Int32.Parse(settingsstrings[3]));
                }
            }
        }

        /// <summary>
        /// This returns the settings as a Base64 encoded string
        /// </summary>
        /// <returns>A base64 encoded string form of the settings</returns>
        public override String ToString()
        {
            string regularsettings = _maxValue.ToString(System.Globalization.CultureInfo.InvariantCulture) + ";";
            regularsettings += _stepValue.ToString(System.Globalization.CultureInfo.InvariantCulture) + ";";
            regularsettings += ((Int32)_valueType).ToString(System.Globalization.CultureInfo.InvariantCulture) + ";";
            regularsettings += ((Int32)_rangesDisplay).ToString(System.Globalization.CultureInfo.InvariantCulture);
            return Convert.ToBase64String(UnicodeEncoding.Unicode.GetBytes(regularsettings));
        }

        /// <summary>
        /// Initialises a new default instance of the Gauge display settings
        /// </summary>
        public GaugeDisplaySettings()
        {
        }
    }
}