﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Web;
using Bamboo.Cubed.UI.WebControls;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    public static class GaugeRangesHandler
    {
        /// <summary>
        /// Gets a Ranges Array based on the Display Type.
        /// </summary>
        /// <param name="pType">Display type used to create the Ranges Array.</param>
        /// <param name="pMaxValue">Maximum Value of to create the Ranges.</param>
        /// <returns></returns>
        public static ImageGaugeSubControl.Range[] GetRanges(GaugeRangesDisplayType pType, Double pMaxValue)
        {
            switch(pType)
            {
                case GaugeRangesDisplayType.Normal:
                    return GetNormalRanges(pMaxValue).ToArray();
                case GaugeRangesDisplayType.Reversed:
                    return GetReversedRanges(pMaxValue).ToArray();
                default:
                    return GetNormalRanges(pMaxValue).ToArray();
            }
        }

        /// <summary>
        /// Gets the Ranges ordered from Red to Green.
        /// </summary>
        /// <param name="pMaxValue"></param>
        /// <returns></returns>
        private static List<ImageGaugeSubControl.Range> GetReversedRanges(Double pMaxValue)
        {
            float rangeIncrement = (float)(pMaxValue / 4);
            float nextStartValue = 0;
            float nextEndValue = rangeIncrement;

            ImageGaugeSubControl.Range range1 = new ImageGaugeSubControl.Range();
            range1.Color = Color.Red;
            range1.RangeStartValue = nextStartValue;
            range1.RangeEndValue = nextEndValue;

            nextStartValue += rangeIncrement;
            nextEndValue += rangeIncrement;

            ImageGaugeSubControl.Range range2 = new ImageGaugeSubControl.Range();
            range2.Color = Color.Orange;
            range2.RangeStartValue = nextStartValue;
            range2.RangeEndValue = nextEndValue;

            nextStartValue += rangeIncrement;
            nextEndValue += rangeIncrement;

            ImageGaugeSubControl.Range range3 = new ImageGaugeSubControl.Range();
            range3.Color = Color.Yellow;
            range3.RangeStartValue = nextStartValue;
            range3.RangeEndValue = nextEndValue;

            nextStartValue += rangeIncrement;
            nextEndValue += rangeIncrement;

            ImageGaugeSubControl.Range range4 = new ImageGaugeSubControl.Range();
            range4.Color = Color.Green;
            range4.RangeStartValue = nextStartValue;
            range4.RangeEndValue = nextEndValue;

            List<ImageGaugeSubControl.Range> arrRanges = new List<ImageGaugeSubControl.Range>(4);
            arrRanges.Add(range1);
            arrRanges.Add(range2);
            arrRanges.Add(range3);
            arrRanges.Add(range4);

            return arrRanges;
        }

        /// <summary>
        /// Gets the Ranges ordered from Green to Red.
        /// </summary>
        /// <param name="pMaxValue"></param>
        /// <returns></returns>
        private static List<ImageGaugeSubControl.Range> GetNormalRanges(Double pMaxValue)
        {
            float rangeIncrement = (float)(pMaxValue / 4);
            float nextStartValue = 0;
            float nextEndValue = rangeIncrement;

            ImageGaugeSubControl.Range range1 = new ImageGaugeSubControl.Range();
            range1.Color = Color.Green;
            range1.RangeStartValue = nextStartValue;
            range1.RangeEndValue = nextEndValue;

            nextStartValue += rangeIncrement;
            nextEndValue += rangeIncrement;

            ImageGaugeSubControl.Range range2 = new ImageGaugeSubControl.Range();
            range2.Color = Color.Yellow;
            range2.RangeStartValue = nextStartValue;
            range2.RangeEndValue = nextEndValue;

            nextStartValue += rangeIncrement;
            nextEndValue += rangeIncrement;

            ImageGaugeSubControl.Range range3 = new ImageGaugeSubControl.Range();
            range3.Color = Color.Orange;
            range3.RangeStartValue = nextStartValue;
            range3.RangeEndValue = nextEndValue;

            nextStartValue += rangeIncrement;
            nextEndValue += rangeIncrement;

            ImageGaugeSubControl.Range range4 = new ImageGaugeSubControl.Range();
            range4.Color = Color.Red;
            range4.RangeStartValue = nextStartValue;
            range4.RangeEndValue = nextEndValue;

            List<ImageGaugeSubControl.Range> arrRanges = new List<ImageGaugeSubControl.Range>(4);
            arrRanges.Add(range1);
            arrRanges.Add(range2);
            arrRanges.Add(range3);
            arrRanges.Add(range4);

            return arrRanges;
        }
    }
}
