﻿/*
 * This source file (Grid.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using System.Globalization;
using System.Reflection;
using System.IO;
using System.Web.UI.MobileControls;
using System.Collections.Generic;
using Microsoft.Live.ServerControls.VE;
using System.Drawing;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// Allows for displaying of results in a map
    /// </summary>
    public partial class Map : CubedResult
    {
        #region Properties

        /// <summary>
        /// Returns the unique identifier of the Add In
        /// </summary>
        public override Guid AddInID
        {
            get { return new Guid("5672dd45-69a5-4e4e-a15e-7ad8c4670857"); }
        }

        /// <summary>
        /// Returns the name of the Add In
        /// </summary>
        public override string Name
        {
            get { return "Map"; }
        }

        /// <summary>
        /// Returns a description of the Add In
        /// </summary>
        public override string Description
        {
            get { return "Allows for displaying of results in a world map"; }
        }

        /// <summary>
        /// Returns the current version of the Add In
        /// </summary>
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        /// <summary>
        /// Returns the name of the user control containing the Add In's markup
        /// </summary>
        public override string UserControlName
        {
            get { return "Map.ascx"; }
        }

        /// <summary>
        /// Returns the name of the image to display in the search results for this item
        /// </summary>
        public override string SearchLogo
        {
            get { return "map_preview.png"; }
        }

        /// <summary>
        /// Determines whether this grid should be displayed in dashboard mode (without any
        /// options for change) or standard item mode
        /// </summary>
        public override bool DashboardMode
        {
            get
            {
                if (Session[this.ID + "_MapDashboardMode"] == null)
                    Session[this.ID + "_MapDashboardMode"] = false;
                return (bool)Session[this.ID + "_MapDashboardMode"];
            }
            set
            {
                Session[this.ID + "_MapDashboardMode"] = value;
            }
        }

        /// <summary>
        /// The results which are displayed in this map
        /// </summary>
        public override object Results
        {
            get
            {
                if (Session[this.ID + "_MapResults"] == null)
                    Session[this.ID + "_MapResults"] = new DataTable();
                return (object)Session[this.ID + "_MapResults"];
            }
            set
            {
                Session[this.ID + "_MapResults"] = value;
            }
        }        

        /// <summary>
        /// The current display settings for the map control
        /// </summary>
        public MapControlSettings ControlSettings
        {
            get
            {
                if (Session[this.ID + "_Settings"] == null)
                    Session[this.ID + "_Settings"] = new MapControlSettings();
                return (MapControlSettings)Session[this.ID + "_Settings"];
            }
            set
            {
                Session[this.ID + "_Settings"] = value;
            }
        }

        /// <summary>
        /// The Current Axis Level contains the currently selected axis level in our drop down being
        /// edited
        /// </summary>
        public int CurrentAxisLevel
        {
            get
            {
                if (Session[this.ID + "_CurrentAxisLevel"] == null)
                    Session[this.ID + "_CurrentAxisLevel"] = -1;
                return (int)Session[this.ID + "_CurrentAxisLevel"];
            }
            set
            {
                Session[this.ID + "_CurrentAxisLevel"] = value;
            }
        }

        /// <summary>
        /// The identifier of the search this grid is a part of
        /// </summary>
        public override Guid SearchID
        {
            get
            {
                if (Session[this.ID + "_SearchID"] == null)
                    Session[this.ID + "_SearchID"] = Guid.Empty;
                return (Guid)Session[this.ID + "_SearchID"];
            }
            set
            {
                Session[this.ID + "_SearchID"] = value;
            }
        }

        #endregion

        #region Implemented Methods

        /// <summary>
        /// Returns a Base64 encoded version of our current settings
        /// </summary>
        /// <returns>A Base64 encoded string containing the current settings</returns>
        public override string SaveWebDisplaySettings()
        {
            // First update our settings
            if (MapAxis != null && ValueAxis != null &&
                MapAxis.SelectedIndex > -1 && ValueAxis.SelectedIndex > -1)
            {
                // Map Axis members have been selected so choose update our settings
                ControlSettings.DisplaySettings.MapAxis = MapAxis.Text;
                ControlSettings.DisplaySettings.ValueAxis = ValueAxis.Text;
                ControlSettings.DisplaySettings.TrafficLights = TrafficLights.Checked;
                ControlSettings.DisplaySettings.Reversed = ReverseTrafficLights.Checked;
                ReverseTrafficLights.Enabled = TrafficLights.Checked;
            }
            else
            {
                // No members have been selected to set empty defaults
                ControlSettings.DisplaySettings = new MapDisplaySettings();               
            }

            // Then update our decimal places
            int decimalplaces = 2;
            int.TryParse(DecimalPlaces.Text, out decimalplaces);
            ControlSettings.DisplaySettings.DecimalPlaces = decimalplaces;

            // Then grab our high/low values
            double highvalue = 0;
            double.TryParse(HighValue.Text, out highvalue);
            ControlSettings.DisplaySettings.HighValue = highvalue;
            double lowvalue = 0;
            double.TryParse(LowValue.Text, out lowvalue);
            ControlSettings.DisplaySettings.LowValue = lowvalue;

            // Then save the current zoom level
            if (CurrentAxisLevel > -1)
            {
                int currentzoomlevel = -1;
                int.TryParse(ZoomLevels.Text, out currentzoomlevel);
                if (currentzoomlevel > -1)
                {
                    if (!ControlSettings.DisplaySettings.AxisZoomLevel.ContainsKey(CurrentAxisLevel))
                        ControlSettings.DisplaySettings.AxisZoomLevel.Add(CurrentAxisLevel, currentzoomlevel);
                    else
                        ControlSettings.DisplaySettings.AxisZoomLevel[CurrentAxisLevel] = currentzoomlevel;
                }
            }

            // Then our map context and aggregation method
            ControlSettings.DisplaySettings.MapContext = MapContext.Text;
            ControlSettings.DisplaySettings.AggregationType = (MapAggregationType)Enum.Parse(typeof(MapAggregationType),
                AggregationMethod.SelectedValue);

            // Then return our settings
            return ControlSettings.DisplaySettings.ToString();
        }

        /// <summary>
        /// Given the Base64 string containing settings, updates the grid with those settings
        /// </summary>
        /// <param name="settings">A Base64 encoded version of the current settings</param>
        /// <returns>True if updated successfully</returns>
        public override bool LoadWebDisplaySettings(string settings)
        {
            // Then read our settings
            ControlSettings.DisplaySettings.FromString(settings);

            // Check if our Axes and or/members have been loaded and load them if not
            if (MapAxis.Items.Count < 1 || ValueAxis.Items.Count < 1)
                PrepareResults();

            // Then update our Axes if they exist
            if (MapAxis.Items.FindByText(ControlSettings.DisplaySettings.MapAxis) != null)
                MapAxis.Text = ControlSettings.DisplaySettings.MapAxis;
            if (ValueAxis.Items.FindByText(ControlSettings.DisplaySettings.ValueAxis) != null)
                ValueAxis.Text = ControlSettings.DisplaySettings.ValueAxis;

            // Then our traffic lights settings
            TrafficLights.Checked = ControlSettings.DisplaySettings.TrafficLights;
            ReverseTrafficLights.Checked = ControlSettings.DisplaySettings.Reversed;
            ReverseTrafficLights.Enabled = TrafficLights.Checked;

            // Then update our decimal places
            DecimalPlaces.Text = ControlSettings.DisplaySettings.DecimalPlaces.ToString();

            // Then update our high / low values
            HighValue.Text = ControlSettings.DisplaySettings.HighValue.ToString();
            LowValue.Text = ControlSettings.DisplaySettings.LowValue.ToString();

            // Then load our first Axis/Zoom level if it exists
            if (AxisLevels.Items.Count > 0)
            {
                AxisLevels.SelectedIndex = 0;
                int currentaxislevel = -1;
                int.TryParse(AxisLevels.SelectedValue, out currentaxislevel);
                if (currentaxislevel > -1)
                {
                    // The first selected axis level is valid, see if it contains a value in our 
                    // settings or not
                    if (ControlSettings.DisplaySettings.AxisZoomLevel.ContainsKey(currentaxislevel))
                    {
                        // There is a zoom level set for this axis level so read it
                        ZoomLevels.Text = ControlSettings.DisplaySettings.AxisZoomLevel[currentaxislevel].ToString();
                    }
                    else
                        ZoomLevels.Text = "1";
                }
            }

            // Finally our map context and aggregation method
            MapContext.Text = ControlSettings.DisplaySettings.MapContext;
            AggregationMethod.SelectedValue = ControlSettings.DisplaySettings.AggregationType.ToString();

            return true;
        }

        /// <summary>
        /// Returns the current result as a bitmap image ready for inserting into a consolidated document
        /// </summary>
        /// <returns>A bitmap representation of the current chart</returns>
        public override Bitmap GetResultImage()
        {
            return new Bitmap(400, 400);
        }

        #endregion

        #region Events

        /// <summary>
        /// Renders the results to the grid if any results have been specified
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            // Set the attributes of our map frame for viewing
            MapFrame.Attributes.Add("src", "./map/VirtualEarth.aspx?MapID=" + this.ID);

            // Then check whether we are running in dashboard mode and adjust the size of the 
            // map frame
            if (DashboardMode)
            {
                MapFrame.Attributes.Add("height", "210px");
                MapFrame.Attributes.Add("width", "100%");
                ControlSettings.Height = Unit.Pixel(200);
                ControlSettings.Width = Unit.Percentage(95);
            }
            else
            {
                MapFrame.Attributes.Add("height", "310px");
                MapFrame.Attributes.Add("width", "100%");
                ControlSettings.Height = Unit.Pixel(300);
                ControlSettings.Width = Unit.Percentage(95);
            }

            // Check to see all our dependant map files exist
            PrepareMapFiles();

            // Then attempt to load our results
            PrepareResults();

            // Then check to see if we should display the setup options. These should only be displayed
            // when we are using an MDDataSet and we're not in dashboard mode
            if (!DashboardMode && Results is Structures.XMLA.MDDataSet)
            {
                AxisChooserRow.Visible = true;
                AxisZoomRow.Visible = true;
                DecimalPlacesRow.Visible = true;
                MapContextRow.Visible = true;
                TrafficValueRow.Visible = true;
                AxisChooserSpacerRow.Visible = true;
            }
            else if (!DashboardMode)
            {
                AxisChooserRow.Visible = true;
                AxisZoomRow.Visible = false;
                DecimalPlacesRow.Visible = true;
                MapContextRow.Visible = true;
                TrafficValueRow.Visible = true;
                AxisChooserSpacerRow.Visible = true;
            }
            else
            {
                AxisChooserRow.Visible = false;
                AxisZoomRow.Visible = false;
                MapContextRow.Visible = false;
                DecimalPlacesRow.Visible = false;
                TrafficValueRow.Visible = false;
                AxisChooserSpacerRow.Visible = false;
            }

            // Check to see if we have any results and display the no results label if none are 
            // available
            if (ControlSettings.Data == null || ControlSettings.Data.Rows.Count < 1)
            {
                MapFrame.Visible = false;
                NoResultsLabel.Visible = true;
                AxisChooserRow.Visible = false;
                MapContextRow.Visible = false;
                AxisZoomRow.Visible = false;
                DecimalPlacesRow.Visible = false;
                TrafficValueRow.Visible = false;
                AxisChooserSpacerRow.Visible = false;
            }
            else
            {
                MapFrame.Visible = true;
                NoResultsLabel.Visible = false;
            }
        }

        /// <summary>
        ///  When the selected map axis changes, pivot the results
        /// </summary>
        protected void MapAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update our display settings
            string oldmap = ControlSettings.DisplaySettings.MapAxis;
            ControlSettings.DisplaySettings.MapAxis = MapAxis.Text;

            // Then, if the newly selected Map axis is the same as the Value axis, change the Map to the 
            // old Value-Axis
            if (ControlSettings.DisplaySettings.MapAxis.ToUpper() == ControlSettings.DisplaySettings.ValueAxis.ToUpper())
            {
                ControlSettings.DisplaySettings.ValueAxis = oldmap;
                ValueAxis.Text = oldmap;
            }
        }

        /// <summary>
        ///  When the selected Value axis changes, pivot the results
        /// </summary>
        protected void ValueAxis_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update our display settings
            string oldvalue = ControlSettings.DisplaySettings.ValueAxis;
            ControlSettings.DisplaySettings.ValueAxis = ValueAxis.Text;

            // Then, if the newly selected Value axis is the same as the Map axis, change the Map to the 
            // old Value-Axis
            if (ControlSettings.DisplaySettings.ValueAxis.ToUpper() == ControlSettings.DisplaySettings.MapAxis.ToUpper())
            {
                ControlSettings.DisplaySettings.MapAxis = oldvalue;
                MapAxis.Text = oldvalue;
            }
        }

        /// <summary>
        /// When the traffic lights setting changes, update the settings
        /// </summary>
        protected void TrafficLights_CheckedChanged(object sender, EventArgs e)
        {
            // Update our settings
            ControlSettings.DisplaySettings.TrafficLights = TrafficLights.Checked;

            // Then, if traffic lights are disabled, disable the "reverse lights" option and high the
            // values row
            ReverseTrafficLights.Enabled = TrafficLights.Checked;
            TrafficValueRow.Visible = TrafficLights.Checked;
        }

        /// <summary>
        /// When the reversed option changes, update the settings
        /// </summary>
        protected void ReverseTrafficLights_CheckedChanged(object sender, EventArgs e)
        {
            // Update our settings
            ControlSettings.DisplaySettings.Reversed = ReverseTrafficLights.Checked;
        }

        /// <summary>
        /// When the selected axis level changes, load the current axis level zoom
        /// </summary>
        protected void AxisLevels_SelectedIndexChanged(object sender, EventArgs e)
        {
            // First save the previous zoom level
            if (CurrentAxisLevel > -1)
            {
                int currentzoomlevel = -1;
                int.TryParse(ZoomLevels.Text, out currentzoomlevel);
                if (currentzoomlevel > -1)
                {
                    if (!ControlSettings.DisplaySettings.AxisZoomLevel.ContainsKey(CurrentAxisLevel))
                        ControlSettings.DisplaySettings.AxisZoomLevel.Add(CurrentAxisLevel, currentzoomlevel);
                    else
                        ControlSettings.DisplaySettings.AxisZoomLevel[CurrentAxisLevel] = currentzoomlevel;
                }
            }

            // Now check if a valid new axis level has been selected
            int currentaxislevel = -1;
            int.TryParse(AxisLevels.SelectedValue, out currentaxislevel);
            if (currentaxislevel > -1)
            {
                // Update our selected axis level
                CurrentAxisLevel = currentaxislevel;

                // Then load the axis zoom level from our settings if it exists
                if (ControlSettings.DisplaySettings.AxisZoomLevel.ContainsKey(CurrentAxisLevel))
                {
                    ZoomLevels.Text = ControlSettings.DisplaySettings.AxisZoomLevel[CurrentAxisLevel].ToString();
                }
                else
                    ZoomLevels.Text = "1";
            }
        }

        /// <summary>
        /// The selected aggregation method has changed so update our control settings
        /// </summary>
        protected void AggregationMethod_SelectedIndexChanged(object sender, EventArgs e)
        {
            ControlSettings.DisplaySettings.AggregationType = (MapAggregationType)Enum.Parse(typeof(MapAggregationType),
                AggregationMethod.SelectedValue);
        }

        #endregion

        #region Support Methods

        /// <summary>
        /// This ensures necessary map files are available
        /// </summary>
        private void PrepareMapFiles()
        {
            // Grab our current directory
            string urldirectory = Assembly.GetExecutingAssembly().CodeBase;
            string realdirectory = urldirectory.Replace('/', '\\');
            realdirectory = realdirectory.Substring(8, realdirectory.Length - 8);
            realdirectory = realdirectory.Substring(0, realdirectory.LastIndexOf('\\') + 1);
            realdirectory = realdirectory.TrimEnd("bin\\".ToCharArray());
            realdirectory += "\\map\\";

            // Then check whether the subdirectory exists
            if (!Directory.Exists(realdirectory))
                Directory.CreateDirectory(realdirectory);

            // Then check if each required file exists, and if not - add it
            List<string> colours = new List<string>();
            colours.Add("purple");
            colours.Add("green");
            colours.Add("orange");
            colours.Add("red");
            List<string> sizes = new List<string>();
            sizes.Add("16");
            sizes.Add("32");
            sizes.Add("48");
            foreach (string thiscolour in colours)
                foreach (string thissize in sizes)
                {
                    // Check to see if this colour/size file already exists
                    if (!File.Exists(realdirectory + thiscolour + "-" + thissize + ".png"))
                    {
                        // This file doesn't exist so extract it from our stream
                        UnmanagedMemoryStream resourcestream = (UnmanagedMemoryStream)Assembly.GetExecutingAssembly().GetManifestResourceStream("Bamboo.Cubed.AddIns.DesktopResults.map." +
                            thiscolour + "-" + thissize + ".png");
                        byte[] filebuffer = new byte[resourcestream.Length];
                        resourcestream.Read(filebuffer, 0, Convert.ToInt32(resourcestream.Length));
                        resourcestream.Close();
                        File.WriteAllBytes(realdirectory + thiscolour + "-" + thissize + ".png", filebuffer);
                    }
                }

            // Now extract our ASPX file for the map control (overwriting the existing version if it exists)
            if (File.Exists(realdirectory + "VirtualEarth.aspx"))
                File.Delete(realdirectory + "VirtualEarth.aspx");
            UnmanagedMemoryStream veresourcestream = (UnmanagedMemoryStream)Assembly.GetExecutingAssembly().GetManifestResourceStream("Bamboo.Cubed.AddIns.DesktopResults.map." +
                            "VirtualEarth.aspx");
            byte[] vefilebuffer = new byte[veresourcestream.Length];
            veresourcestream.Read(vefilebuffer, 0, Convert.ToInt32(veresourcestream.Length));
            veresourcestream.Close();
            File.WriteAllBytes(realdirectory + "VirtualEarth.aspx", vefilebuffer);
        }

        /// <summary>
        /// This prepares our control based on the given results
        /// </summary>
        private void PrepareResults()
        {
            // First prepare our results data table
            ControlSettings.Data = new DataTable();
            if (Results is DataTable)
                ControlSettings.Data = (DataTable)Results;
            else if (Results is DataSet && ((DataSet)Results).Tables.Count > 0)
                ControlSettings.Data = ((DataSet)Results).Tables[0];
            else if (Results is Structures.XMLA.MDDataSet)
                ControlSettings.Data = ((Structures.XMLA.MDDataSet)Results).ToDataSet().Tables[0];

            // Then load our Axis definitions for selected values
            MapAxis.Items.Clear();
            ValueAxis.Items.Clear();
            if (Results is Structures.XMLA.MDDataSet)
            {
                // Since the results are a multi-dimensional data set, add the axes from the data set
                foreach (Structures.XMLA.Axis thisaxis in ((Structures.XMLA.MDDataSet)Results).Axes)
                {
                    MapAxis.Items.Add(thisaxis.Name);
                    ValueAxis.Items.Add(thisaxis.Name);
                }
            }
            else
            {
                // As the results are not a multi-dimensional dataset, just add 2 default axes
                MapAxis.Items.Add("Axis 1");
                ValueAxis.Items.Add("Axis 1");
                MapAxis.Items.Add("Axis 2");
                ValueAxis.Items.Add("Axis 2");
            }

            // Then, if the selected axes are available, select them
            if (MapAxis.Items.FindByText(ControlSettings.DisplaySettings.MapAxis) != null)
                MapAxis.Text = ControlSettings.DisplaySettings.MapAxis;
            else if (MapAxis.Items.Count > 0)
            {
                MapAxis.SelectedIndex = 0;
                ControlSettings.DisplaySettings.MapAxis = MapAxis.Text;
            }
            if (ValueAxis.Items.FindByText(ControlSettings.DisplaySettings.ValueAxis) != null)
                ValueAxis.Text = ControlSettings.DisplaySettings.ValueAxis;
            else if (ValueAxis.Items.Count > 1)
            {
                ValueAxis.SelectedIndex = 1;
                ControlSettings.DisplaySettings.ValueAxis = ValueAxis.Text;
            }
            else if (ValueAxis.Items.Count > 0)
            {
                ValueAxis.SelectedIndex = 0;
                ControlSettings.DisplaySettings.ValueAxis = ValueAxis.Text;
            }

            // Load our map aggregation types if it hasn't already been done
            if (AggregationMethod.Items.Count < 1)
            {
                foreach (string thismethod in Enum.GetNames(typeof(MapAggregationType)))
                    AggregationMethod.Items.Add(thismethod);
            }

            // Then select our aggregation type
            AggregationMethod.SelectedValue = ControlSettings.DisplaySettings.AggregationType.ToString();

            // Then update our selected axis level definitions
            LoadAxisLevels();

            // Then set our selected axis definition if it exists
            if (CurrentAxisLevel > -1 && AxisLevels.Items.FindByValue(CurrentAxisLevel.ToString()) != null)
            {
                AxisLevels.SelectedValue = CurrentAxisLevel.ToString();
            }

            // Then, provided axes have been selected and we have an MDDataSet - ensure the
            // correct axes have been pivoted
            if (MapAxis.SelectedIndex > -1 && ValueAxis.SelectedIndex > -1 &&
                Results is Structures.XMLA.MDDataSet)
            {
                // Grab the corresponding level to the current map zoom level
                int axislevel = -1;
                if (ControlSettings.DisplaySettings.AxisZoomLevel.Values.Where(testvalue =>
                    testvalue >= ControlSettings.DisplaySettings.MapZoom).Count() > 0)
                {
                    int axiszoomvalue = ControlSettings.DisplaySettings.AxisZoomLevel.Values.Where(testvalue =>
                        testvalue >= ControlSettings.DisplaySettings.MapZoom).Min();
                    axislevel = ControlSettings.DisplaySettings.AxisZoomLevel.Keys.FirstOrDefault(testkey =>
                        ControlSettings.DisplaySettings.AxisZoomLevel[testkey] == axiszoomvalue);
                }

                if (axislevel > -1)
                    ControlSettings.Data = ((Structures.XMLA.MDDataSet)Results).ToDataSet(MapAxis.Text,
                        ValueAxis.Text, axislevel).Tables[0];
                else
                    ControlSettings.Data = ((Structures.XMLA.MDDataSet)Results).ToDataSet(MapAxis.Text,
                        ValueAxis.Text).Tables[0];
            }    
        }

        /// <summary>
        /// This loads the current axis levels based on the given results (if an MDDataSet) and 
        /// the selected axis on the map axis
        /// </summary>
        private void LoadAxisLevels()
        {
            if (Results is Structures.XMLA.MDDataSet && MapAxis.SelectedIndex > -1)
            {
                // As we have an MDDataSet we can use axis levels
                AxisLevels.Items.Clear();

                // Grab the index of the map axis in the MDDataSet
                int mapaxis = ((Structures.XMLA.MDDataSet)Results).Axes.AxisCollection.FindIndex(testaxis =>
                    testaxis.Name.ToUpper() == MapAxis.Text.ToUpper());

                if (mapaxis > -1)
                {
                    // Grab a unique collection of each level in the result set
                    Dictionary<uint, string> axislevels = new Dictionary<uint, string>();
                    foreach (Structures.XMLA.MemberType thismember in ((Structures.XMLA.MDDataSet)Results).Axes[mapaxis].Members)
                    {
                        if (thismember.LName != null && thismember.LName != String.Empty &&
                            !axislevels.ContainsKey(thismember.LNum))
                            axislevels.Add(thismember.LNum, thismember.LName);
                    }

                    // Then update our Axis levels control
                    foreach (uint thislevel in axislevels.Keys)
                        AxisLevels.Items.Add(new ListItem(axislevels[thislevel], thislevel.ToString()));
                }
            }
        }

        #endregion  
    }
}