/*
 * This source file (GridDisplaySettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.UI.WebControls.TrafficLightsGaugeObjects;

namespace Bamboo.Cubed.AddIns.DesktopResults
{
    /// <summary>
    /// This class stores the required settings for displaying the Grid
    /// </summary>
    [Serializable]
    public class TrafficLightsDisplaySettings
    {
        /// <summary>
        /// Initialises a new default instance of the Traffic Lights display settings
        /// </summary>
        public TrafficLightsDisplaySettings()
        {   
        }

        private TrafficLightsGaugeDisplayStyle _displayStyle = TrafficLightsGaugeDisplayStyle.GreenIsHighest;
        /// <summary>
        /// Gets or Sets the Display Style for the Gauge.
        /// </summary>
        public TrafficLightsGaugeDisplayStyle DisplayStyle
        {
            get { return _displayStyle; }
            set
            {
                if (_displayStyle != value)
                {
                    _displayStyle = value;
                }
            }

        }

        private Double _loMaxValue;
        /// <summary>
        /// Gets or sets the lowest Maximum value.
        /// </summary>
        public Double LoMaxValue
        {
            get { return _loMaxValue; }
            set
            {
                if (_loMaxValue != value)
                {
                    _loMaxValue = value;
                }
            }
        }

        private Double _hiMinValue;
        /// <summary>
        /// Gets or sets the highest Minimum value.
        /// </summary>
        public Double HiMinValue
        {
            get { return _hiMinValue; }
            set
            {
                if (_hiMinValue != value)
                {
                    _hiMinValue = value;
                }
            }
        }

        /// <summary>
        /// This sets the settings based on the passed in Base64 encoded string
        /// </summary>
        /// <param name="settings">A base64 encoded string containing settings</param>
        public void FromString(string settings)
        {
            string regularstring = UnicodeEncoding.Unicode.GetString(Convert.FromBase64String(settings));
            string[] settingsstrings = regularstring.Split(';');
            if (settingsstrings.Length == 3)
            {
                _displayStyle = (TrafficLightsGaugeDisplayStyle)Enum.ToObject(typeof(TrafficLightsGaugeDisplayStyle),Double.Parse(settingsstrings[0]));
                _loMaxValue = Double.Parse(settingsstrings[1]);
                _hiMinValue = Double.Parse(settingsstrings[2]);
            }
        }

        /// <summary>
        /// This returns the settings as a Base64 encoded string
        /// </summary>
        /// <returns>A base64 encoded string form of the settings</returns>
        public override string ToString()
        {
            string regularsettings = ((Int32)_displayStyle).ToString() + ";";
            regularsettings += _loMaxValue.ToString() + ";";
            regularsettings += _hiMinValue.ToString() + ";";
            return Convert.ToBase64String(UnicodeEncoding.Unicode.GetBytes(regularsettings));
        }
    }
}