﻿/*
 * This source file (CubeExplorer2.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.AddIns.DesktopResults;
using Bamboo.Cubed.Utilities.Logging;
using System.Text.RegularExpressions;

namespace Bamboo.Cubed.AddIns.DesktopSearches
{
    /// <summary>
    /// Provides the ability to explore underlying cubes and return their results
    /// </summary>
    public partial class CubeExplorer : CubedSearch
    {
        #region Properties

        /// <summary>
        /// If this exists it indicates that a "drill down" has been requested onto the given field
        /// </summary>
        public string DrillOn
        {
            get
            {
                if (Session[SearchID.ToString() + "_DrillOn"] == null)
                    Session[SearchID.ToString() + "_DrillOn"] = String.Empty;
                return (string)Session[SearchID.ToString() + "_DrillOn"];
            }
            set
            {
                Session[SearchID.ToString() + "_DrillOn"] = value;
            }
        }

        /// <summary>
        /// This marks whether drill down capable is provided or not
        /// </summary>
        public bool DrillCapable
        {
            get
            {
                if (Session[SearchID.ToString() + "_DrillCapable"] == null)
                    Session[SearchID.ToString() + "_DrillCapable"] = false;
                return (bool)Session[SearchID.ToString() + "_DrillCapable"];
            }
            set
            {
                Session[SearchID.ToString() + "_DrillCapable"] = value;
            }
        }

        /// <summary>
        /// The identifier of the search this grid is a part of
        /// </summary>
        public override Guid SearchID
        {
            get
            {
                if (Session[this.ID + "_SearchID"] == null)
                    Session[this.ID + "_SearchID"] = Guid.Empty;
                return (Guid)Session[this.ID + "_SearchID"];
            }
            set
            {
                Session[this.ID + "_SearchID"] = value;
            }
        }

        /// <summary>
        /// These are the current settings for the connection to the underlying cube
        /// </summary>
        public CubeExplorerDisplaySettings DisplaySettings
        {
            get
            {
                if (Session[this.ID + "_DisplaySettings"] == null)
                    Session[this.ID + "_DisplaySettings"] = new CubeExplorerDisplaySettings();
                return (CubeExplorerDisplaySettings)Session[this.ID + "_DisplaySettings"];
            }
            set
            {
                Session[this.ID + "_DisplaySettings"] = value;
            }
        }        

        /// <summary>
        /// The unique identifier of the Add In
        /// </summary>
        public override Guid AddInID
        {
            get { return new Guid("5128e8e1-78ed-4beb-b994-a4dc768f638c"); }
        }

        /// <summary>
        /// The name of the Add In
        /// </summary>
        public override string Name
        {
            get { return "Cube Explorer"; }
        }

        /// <summary>
        /// The description of the Add In
        /// </summary>
        public override string Description
        {
            get { return "Permits exploring of Cubes and dynamic creation of queries."; }
        }

        /// <summary>
        /// The current version of the Add In
        /// </summary>
        public override double CurrentVersion
        {
            get { return 1.1; }
        }

        /// <summary>
        /// Returns the name of the ASCX markup file used for this control
        /// </summary>
        public override string UserControlName
        {
            get { return "CubeExplorer.ascx"; }
        }

        /// <summary>
        /// The connection string used to access the search resource
        /// </summary>
        public override string ConnectionString
        {
            get
            {
                if (Session[this.ID + "_ConnectionString"] == null)
                    Session[this.ID + "_ConnectionString"] = String.Empty;
                return (string)Session[this.ID + "_ConnectionString"];
            }
            set
            {
                Session[this.ID + "_ConnectionString"] = value;
            }
        }

        /// <summary>
        /// Returns the list of supported results viewers
        /// </summary>
        public override List<CubedResult> ResultsViewers
        {
            get 
            {
                List<CubedResult> supportedviewers = new List<CubedResult>();
                supportedviewers.Add(new Grid());
                supportedviewers.Add(new Chart());
                supportedviewers.Add(new Map());
                supportedviewers.Add(new TrafficLights());
                supportedviewers.Add(new Gauge());
                return supportedviewers;
            }
        }

        /// <summary>
        /// The list of cubes available for this connection
        /// </summary>
        public GenericKeyCollection<Guid, Cube> AvailableCubes
        {
            get
            {
                if (Session[this.ID + "_AvailableCubes"] == null)
                    Session[this.ID + "_AvailableCubes"] = new GenericKeyCollection<Guid, Cube>();
                return (GenericKeyCollection<Guid, Cube>)Session[this.ID + "_AvailableCubes"];
            }
            set
            {
                Session[this.ID + "_AvailableCubes"] = value;
            }
        }

        #endregion

        #region Implemented Methods

        /// <summary>
        /// Returns the current active settings as a string ready for saving
        /// </summary>
        /// <returns>The current settings as a Base64 encoded string ready for saving</returns>
        public override string SaveWebSearch()
        {
            // First update our settings from selected options
            if (DesignerCodeChooser.SelectedIndex == 0)
            {
                // "Designer" mode is active
                DisplaySettings.SelectedCube = AvailableCubes[new Guid(DesignerSelectedCube.SelectedValue)].ID;
                DisplaySettings.GenerateCode = true;
                DisplaySettings.CurrentMDX = CodeMDXCommand.Text;
                DisplaySettings.RowsMembers = new List<string>();
                foreach (ListItem thisrowmember in DesignerRows.Items)
                    DisplaySettings.RowsMembers.Add(thisrowmember.Text);
                DisplaySettings.ColumnsMembers = new List<string>();
                foreach (ListItem thiscolumnmember in DesignerColumns.Items)
                    DisplaySettings.ColumnsMembers.Add(thiscolumnmember.Text);
                DisplaySettings.IncludeZeros = DesignerIncludeZeroValues.Checked;
                DisplaySettings.FiltersMembers = new List<string>();
                foreach (ListItem thisfiltermember in DesignerFilters.Items)
                    DisplaySettings.FiltersMembers.Add(thisfiltermember.Text);
                DisplaySettings.IncludeZeros = DesignerIncludeZeroValues.Checked;
            }
            else
            {
                // "Code" mode is active
                DisplaySettings.SelectedCube = AvailableCubes[new Guid(DesignerSelectedCube.SelectedValue)].ID;
                DisplaySettings.GenerateCode = false;
                DisplaySettings.CurrentMDX = CodeMDXCommand.Text;
                DisplaySettings.RowsMembers = new List<string>();
                DisplaySettings.ColumnsMembers = new List<string>();
                DisplaySettings.FiltersMembers = new List<string>();
                DisplaySettings.IncludeZeros = CodeIncludeZeroValues.Checked;
            }
            
            // Then return them
            return DisplaySettings.ToString();
        }

        /// <summary>
        /// This loads the current search settings from the given Base64 string
        /// </summary>
        /// <param name="searchdetails">A Base64 string containing the current settings</param>
        /// <returns>True if successful, false if not</returns>
        public override bool LoadWebSearch(string searchdetails)
        {
            // First check we have loaded our cubes and load them if not
            if (AvailableCubes.Count < 1 || DesignerSelectedCube.Items.Count < 1)
                PrepareCubes();

            // Then read our settings
            DisplaySettings.FromString(searchdetails);

            // Then update our selected items and MDX if the selected cube exists
            if (DisplaySettings.GenerateCode)
            {
                // Automatic code generation is being used
                DesignerCodeChooser.SelectedIndex = 0;
                if (AvailableCubes.Contains(DisplaySettings.SelectedCube))
                {
                    DesignerSelectedCube.SelectedValue = DisplaySettings.SelectedCube.ToString();
                    CodeMDXCommand.Text = DisplaySettings.CurrentMDX;
                    foreach (string thisrowmember in DisplaySettings.RowsMembers)
                        DesignerRows.Items.Add(new ListItem(thisrowmember));
                    foreach (string thiscolumnmember in DisplaySettings.ColumnsMembers)
                        DesignerColumns.Items.Add(new ListItem(thiscolumnmember));
                    foreach (string thisfiltermember in DisplaySettings.FiltersMembers)
                        DesignerFilters.Items.Add(new ListItem(thisfiltermember));
                    DisplaySettings.IncludeZeros = DesignerIncludeZeroValues.Checked;
                }
            }
            else
            {
                // Manual MDX writing is being used
                DesignerCodeChooser.SelectedIndex = 1;
                CodeMDXCommand.Text = DisplaySettings.CurrentMDX;
                DisplaySettings.IncludeZeros = CodeIncludeZeroValues.Checked;
            }           

            return true;
        }

        /// <summary>
        /// This executes the current search and returns the results as an object
        /// </summary>
        /// <returns>The current web results as an object</returns>
        public override object GetWebResults()
        {
            try
            {
                // Save our web search to update the display settings
                SaveWebSearch();

                // Determine if we need to drill down or not
                if (!String.IsNullOrEmpty(DrillOn))
                    DrillDown();                   
                else
                    // Check whether drill down support should be provided
                    CheckDrillDown();

                // Execute the given search against the cube
                CubedConnection conn = new CubedConnection(ConnectionString);
                conn.Open();
                CubedCommand command = new CubedCommand(DisplaySettings.CurrentMDX, conn);
                command.Properties.Cube = AvailableCubes[DisplaySettings.SelectedCube].Name;
                command.Properties.Content = Bamboo.Cubed.Structures.XMLA.ContentTypes.SchemaData;
                command.Properties.Format = Bamboo.Cubed.Structures.XMLA.FormatTypes.Multidimensional;
                command.Properties.IncludeZeros = DisplaySettings.IncludeZeros;
                
                // Determine if this is a transactional query or a multi-dimensional
                object results;
                if (DisplaySettings.CurrentMDX.ToUpper().Contains("TRANSACTIONS"))
                    // As this is a transactional query use the standard data set
                    results = command.ExecuteDataSet();
                else
                    // As this is a multi-dimensional query use the MD DataSet
                    results = command.ExecuteMDDataSet();
                conn.Close();
                return results;
            }
            catch
            {
                // An error occured so return an empty MDDataSet
                return new Structures.XMLA.MDDataSet();
            }
        }

        /// <summary>
        /// Check whether drill down capability should be provided and pass through a flag to indicate if
        /// drill down is possible
        /// </summary>
        private void CheckDrillDown()
        {
            try
            {
                // Grab the selected cube
                Cube selectedcube = AvailableCubes[DisplaySettings.SelectedCube];

                // Ensure that there is at least one measure placed on the filters
                DrillCapable = false;
                Measure drillmeasure = null;
                int startindex = DisplaySettings.CurrentMDX.IndexOf("WHERE");
                if (startindex > -1)
                {
                    foreach (Measure thismeasure in selectedcube.Measures)
                        if (DisplaySettings.CurrentMDX.Substring(startindex, DisplaySettings.CurrentMDX.Length - startindex).ToUpper().Contains(thismeasure.Name.ToUpper()))
                            drillmeasure = thismeasure;
                }
                if (drillmeasure == null)
                    return;

                // Now that we know there is a measure on the filters, check to ensure that this measure has
                // drill down transactional support                
                if (drillmeasure.TransactionalFields.Count > 0)
                    DrillCapable = true;
                else
                    DrillCapable = false;
            }
            catch (Exception e1)
            {
                // Log our error
                LoggingManager.Write(e1, Bamboo.Cubed.Utilities.LoggingTypes.MinorError);
            }
        }

        /// <summary>
        /// Forces a "drill down" to the transaction level associated with the given member
        /// </summary>
        private void DrillDown()
        {
            // First determine the dimension the selected member belongs to
            try
            {
                // Grab the selected cube
                Cube selectedcube = AvailableCubes[DisplaySettings.SelectedCube];

                // Find the dimension containing this member
                Dimension selecteddimension = selectedcube.Dimensions.FirstOrDefault(testdim =>
                    testdim.Members.List.Exists(testmember => testmember.Name.ToUpper() ==
                        DrillOn.ToUpper()));

                // Before updating, removing any level markers from the MDX
                foreach (Level thislevel in selecteddimension.Levels)
                    DisplaySettings.CurrentMDX = Regex.Replace(DisplaySettings.CurrentMDX,
                         "." + thislevel.Name, String.Empty, RegexOptions.IgnoreCase);

                // Now update our MDX code to specifically select this member
                DisplaySettings.CurrentMDX = Regex.Replace(DisplaySettings.CurrentMDX,
                    selecteddimension.Name, selecteddimension.Name + "].[" + DrillOn, RegexOptions.IgnoreCase);

                // Then update to instruct the measure to report transactions
                Measure selectedmeasure = selectedcube.Measures.FirstOrDefault(testmeasure =>
                    DisplaySettings.CurrentMDX.ToUpper().Contains(testmeasure.Name.ToUpper()));

                // Now we have the measure update the MDX
                DisplaySettings.CurrentMDX = Regex.Replace(DisplaySettings.CurrentMDX,
                    selectedmeasure.Name, selectedmeasure.Name + ".Transactions", RegexOptions.IgnoreCase);
            }
            catch (Exception e1)
            {
                // Log our error
                LoggingManager.Write(e1, Bamboo.Cubed.Utilities.LoggingTypes.MinorError);
            }
        }

        #endregion

        #region Events

        /// <summary>
        /// On page load the current settings are checked and loaded if needed, as are the 
        /// collection of Cubes available on the given server
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Check if we have any cubes and try and load them if not
            if (AvailableCubes.Count < 1 || DesignerSelectedCube.Items.Count < 1)
                PrepareCubes();
        }

        /// <summary>
        /// If a cube has been selected, fills the dimensions drop down list - otherwise disables 
        /// selectors
        /// </summary>
        protected void SelectedCube_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Check to ensure the selected cube has dimensions
            ClearSettings();
            if (AvailableCubes[new Guid(DesignerSelectedCube.SelectedValue)].Dimensions.Where(testdim => testdim.Enabled).Count() > 0)
            {
                // Prepare our dimensions drop down
                PrepareDimensions(AvailableCubes[new Guid(DesignerSelectedCube.SelectedValue)]);
            }
            else
            {
                CodeMDXCommand.Text = String.Empty;
                CodeMDXCommand.Enabled = false;
                DesignerSelectedDimension.Items.Clear();
                DesignerSelectedDimension.Enabled = false;
            }
        }

        /// <summary>
        /// The selected dimension has changed so update the members
        /// </summary>
        protected void DesignerSelectedDimension_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Check whether the "measures" or a regular dimension have been selected
            if (DesignerSelectedDimension.SelectedValue == "Measures")
            {
                // The measures dimension has been selected, so load the measures
                LoadMeasuresList(AvailableCubes[new Guid(DesignerSelectedCube.SelectedValue)]);
            }
            else
            {
                // A regular dimension has been selected, so load it's members
                LoadMembersList(AvailableCubes[new Guid(DesignerSelectedCube.SelectedValue)].Dimensions[new Guid(DesignerSelectedDimension.SelectedValue)]);
            }
        }

        /// <summary>
        /// Selects either designer or code view, depending on the selected option
        /// </summary>
        protected void DesignerCodeChooser_SelectedIndexChanged(object sender, EventArgs e)
        {
            CubeExplorerViews.ActiveViewIndex = DesignerCodeChooser.SelectedIndex;
        }

        /// <summary>
        /// Remove the selected row from the query
        /// </summary>
        protected void DesignerRemoveColumnsButton_Click(object sender, ImageClickEventArgs e)
        {
            // Remove the selected option from the list
            if (DesignerColumns.SelectedItem != null)
            {
                DesignerColumns.Items.Remove(DesignerColumns.SelectedItem);
            }

            // Update the code
            PrepareCode();
        }

        /// <summary>
        /// Remove the selected column from the query
        /// </summary>
        protected void DesignerRemoveRowsButton_Click(object sender, ImageClickEventArgs e)
        {
            // Remove the selected option from the list
            if (DesignerRows.SelectedItem != null)
            {
                DesignerRows.Items.Remove(DesignerRows.SelectedItem);
            }

            // Update the code
            PrepareCode();
        }

        /// <summary>
        /// Remove the selected filter from the query
        /// </summary>
        protected void DesignerRemoveFiltersButton_Click(object sender, ImageClickEventArgs e)
        {
            // Remove the selected option from the list
            if (DesignerFilters.SelectedItem != null)
            {
                DesignerFilters.Items.Remove(DesignerFilters.SelectedItem);
            }

            // Update the code
            PrepareCode();
        }

        /// <summary>
        /// Add the selected column to the query
        /// </summary>
        protected void DesignerAddColumnsButton_Click(object sender, ImageClickEventArgs e)
        {
            // Add the selected option to the columns
            if (DesignerSelectedMember.SelectedNode != null)
            {
                DesignerColumns.Items.Add(new ListItem(DesignerSelectedMember.SelectedNode.Text));
            }

            // Update the code
            PrepareCode();           
        }

        /// <summary>
        /// Add the selected row to the query
        /// </summary>
        protected void DesignerAddRowsButton_Click(object sender, ImageClickEventArgs e)
        {
            // Add the selected option to the rows
            if (DesignerSelectedMember.SelectedNode != null)
            {
                DesignerRows.Items.Add(new ListItem(DesignerSelectedMember.SelectedNode.Text));
            }

            // Update the code
            PrepareCode();
        }

        /// <summary>
        /// Add the selected filter to the query
        /// </summary>
        protected void DesignerAddFiltersButton_Click(object sender, ImageClickEventArgs e)
        {
            // Add the selected option to the filters
            if (DesignerSelectedMember.SelectedNode != null)
            {
                DesignerFilters.Items.Add(new ListItem(DesignerSelectedMember.SelectedNode.Text));
            }

            // Update the code
            PrepareCode();
        }

        #endregion

        #region Support Methods

        /// <summary>
        /// Clears all selected options
        /// </summary>
        private void ClearSettings()
        {
            CodeMDXCommand.Text = String.Empty;
            DesignerRows.Items.Clear();
            DesignerColumns.Items.Clear();
            DesignerFilters.Items.Clear();
        }

        /// <summary>
        /// Loads the cube list from the cube connection to the cached collection, as well as 
        /// setting up the drop down lists
        /// </summary>
        private void PrepareCubes()
        {
            // Grab the list of cubes from the cube connection
            try
            {
                // Grab the list of cubes (if it isn't loaded already)
                if (AvailableCubes.Count < 1)
                {
                    CubedConnection conn = new CubedConnection(ConnectionString);
                    conn.Open();

                    foreach (Cube thiscubeheader in conn.Administration.CubeHeaders)
                        AvailableCubes.Add(conn.Administration.GetCube(thiscubeheader.ID));
                    conn.Close();
                }

                // Populate our cubes list
                DesignerSelectedCube.Enabled = true;
                DesignerSelectedCube.Items.Clear();
                foreach (Cube thiscube in AvailableCubes.Where(testcube => testcube.Dimensions.Where(testdim
                    => testdim.Enabled).Count() > 0))
                    DesignerSelectedCube.Items.Add(new ListItem(thiscube.Name, thiscube.ID.ToString()));

                // Finally, select the first cube and load it's dimensions
                DesignerSelectedCube.SelectedIndex = 0;
                PrepareDimensions(AvailableCubes[0]);
            }
            catch (Exception e1)
            {
                // An error occured so just leave the cube list empty and everything disabled
                DesignerSelectedCube.Enabled = false;
                DesignerSelectedCube.Items.Clear();
                DesignerSelectedDimension.Enabled = false;
                DesignerSelectedDimension.Items.Clear();
                CodeMDXCommand.Enabled = false;
                CodeMDXCommand.Text = String.Empty;
            }
        }

        /// <summary>
        /// Prepares the dimension list from the given cube
        /// </summary>
        /// <param name="selectedcube">The cube selected</param>
        private void PrepareDimensions(Cube selectedcube)
        {
            // Clear the current list
            DesignerSelectedDimension.Items.Clear();
            DesignerSelectedDimension.Enabled = true;
            CodeMDXCommand.Enabled = true;

            // Add all dimensions from the given cube to the list
            foreach (Dimension thisdim in selectedcube.Dimensions.Where(testdim => testdim.Enabled))
                DesignerSelectedDimension.Items.Add(new ListItem(thisdim.Name, thisdim.ID.ToString()));

            // Then add the "measures" dimension
            DesignerSelectedDimension.Items.Add(new ListItem("Measures", "Measures"));

            // Now update the members list for the first dimension
            DesignerSelectedDimension_SelectedIndexChanged(null, null);
        }

        /// <summary>
        /// Shows the given error to the user
        /// </summary>
        /// <param name="errormessage">The error message to display</param>
        private void ShowDesignerError(string errormessage)
        {
            DesignerErrorImage.Visible = true;
            DesignerErrorLabel.Visible = true;
            DesignerErrorLabel.Text = errormessage;
        }

        /// <summary>
        /// Hides the error label
        /// </summary>
        private void HideDesignerError()
        {
            DesignerErrorImage.Visible = false;
            DesignerErrorLabel.Visible = false;
        }

        /// <summary>
        /// This prepares the MDX code based on the selected rows, columns and filters, and then updates
        /// the MDX string
        /// </summary>
        private void PrepareCode()
        {
            // Prepare a list of errors as well as the MDX code to use
            string errors = String.Empty;
            string mdx = "SELECT {";

            // Prepare the rows
            List<string> rowdimensions = new List<string>();
            bool firstitem = true;
            foreach (ListItem thisitem in DesignerRows.Items)
            {
                // Grab the name of the dimension and member for this item
                string dimension = thisitem.Text.Split('.')[0];
                string member = thisitem.Text.Split('.')[1];
                string function = String.Empty;
                if (thisitem.Text.Split('.').Length > 2)
                    function = thisitem.Text.Split('.')[2];

                if (!rowdimensions.Exists(testitem => testitem.ToUpper() == dimension.ToUpper()))
                    rowdimensions.Add(dimension);

                // Check if more than 2 dimensions have been used
                if (rowdimensions.Count > 1)
                {
                    // There is more than 1 dimension on the rows so raise an error
                    errors += "There is more than 1 dimension on the rows. Please remove the first dimension " +
                        "before adding another." + Environment.NewLine;
                    break;
                }
                else
                {
                    // So far there are no problems with the dimensions so add this member
                    if (firstitem)
                    {
                        firstitem = false;
                        mdx += "[" + dimension + "].[" + member + "]";
                        if (function != String.Empty)
                            mdx += "." + function;
                    }
                    else
                    {
                        mdx += ", [" + dimension + "].[" + member + "]";
                        if (function != String.Empty)
                            mdx += "." + function;
                    }
                }
            }
            mdx += "} ON ROWS, " + Environment.NewLine + "{";

            // Prepare the columns
            List<string> columndimensions = new List<string>();
            firstitem = true;
            foreach (ListItem thisitem in DesignerColumns.Items)
            {
                // Grab the name of the dimension and member for this item
                string dimension = thisitem.Text.Split('.')[0];
                string member = thisitem.Text.Split('.')[1];
                string function = String.Empty;
                if (thisitem.Text.Split('.').Length > 2)
                    function = thisitem.Text.Split('.')[2];

                if (!columndimensions.Exists(testitem => testitem.ToUpper() == dimension.ToUpper()))
                    columndimensions.Add(dimension);

                // Check if more than 2 dimensions have been used
                if (columndimensions.Count > 1)
                {
                    // There is more than 1 dimension on the columns so raise an error
                    errors += "There is more than 1 dimension on the columns. Please remove the first dimension " +
                        "before adding another." + Environment.NewLine;
                    break;
                }
                else
                {
                    // So far there are no problems with the dimensions so add this member
                    if (firstitem)
                    {
                        firstitem = false;
                        mdx += "[" + dimension + "].[" + member + "]";
                        if (function != String.Empty)
                            mdx += "." + function;
                    }
                    else
                    {
                        mdx += ", [" + dimension + "].[" + member + "]";
                        if (function != String.Empty)
                            mdx += "." + function;
                    }
                }
            }
            mdx += "} ON COLUMNS" + Environment.NewLine;

            // Now add the name of the cube
            mdx += "FROM [" + DesignerSelectedCube.SelectedItem.Text + "]" + Environment.NewLine;

            // Finally prepare the filters
            mdx += "WHERE {";
            List<string> filterdimensions = new List<string>();
            firstitem = true;
            foreach (ListItem thisitem in DesignerFilters.Items)
            {
                // Grab the name of the dimension and member for this item
                string dimension = thisitem.Text.Split('.')[0];
                string member = thisitem.Text.Split('.')[1];
                string function = String.Empty;
                if (thisitem.Text.Split('.').Length > 2)
                    function = thisitem.Text.Split('.')[2];

                if (filterdimensions.Exists(testitem => testitem.ToUpper() == dimension.ToUpper()))
                {
                    // This filter dimension has already been added, so raise an error
                    errors += "This dimension has already been specified on the filters. Please remove the first reference " +
                        "before adding another." + Environment.NewLine;
                    break;
                }
                else
                {
                    // This dimension has not been used so add it to the collection
                    if (firstitem)
                    {
                        firstitem = false;
                        mdx += "[" + dimension + "].[" + member + "]";
                        if (function != String.Empty)
                            mdx += "." + function;
                    }
                    else
                    {
                        mdx += ", [" + dimension + "].[" + member + "]";
                        if (function != String.Empty)
                            mdx += "." + function;
                    }
                }
            }
            mdx += "}";

            // Finally, check whether any errors occurred, and either update the MDX, or clear it and
            // update the error labels
            if (errors.Length > 0)
            {
                // Some errors occurred, so update the screen
                CodeMDXCommand.Text = String.Empty;
                ShowDesignerError(errors);
            }
            else
            {
                // No errors occurred, so update the screen
                CodeMDXCommand.Text = mdx;
                HideDesignerError();
            }
        }

        /// <summary>
        /// Loads themeasures list for the current cube
        /// </summary>
        /// <param name="cube">The cube which has been selected</param>
        private void LoadMeasuresList(Cube cube)
        {
            // First clear the current list
            DesignerSelectedMember.Nodes.Clear();

            // Then add all "measures"
            foreach (Measure thismeasure in cube.Measures)
            {
                TreeNode measurenode = new TreeNode("Measures." +
                thismeasure.Name);
                measurenode.PopulateOnDemand = false;
                DesignerSelectedMember.Nodes.Add(measurenode);
            }
        }

        /// <summary>
        /// This loads the list of members for the selected dimension
        /// </summary>
        /// <param name="selecteddimension">The selected dimension</param>
        private void LoadMembersList(Dimension selecteddimension)
        {
            // First clear the current list
            DesignerSelectedMember.Nodes.Clear();

            // Now add "All members" and "Current Member" to the list
            TreeNode allmembersnode = new TreeNode(selecteddimension.Name + "." +
                "*");
            allmembersnode.PopulateOnDemand = false;
            DesignerSelectedMember.Nodes.Add(allmembersnode);
            TreeNode currentmembernode = new TreeNode(selecteddimension.Name + "." +
                "CurrentMember");
            currentmembernode.PopulateOnDemand = false;
            DesignerSelectedMember.Nodes.Add(currentmembernode);

            // Now cycle through and add each level
            foreach (Level thislevel in selecteddimension.Levels)
            {
                // Add the level, and the current member
                TreeNode levelmembersnode = new TreeNode(selecteddimension.Name + "." +
                    thislevel.Name);
                levelmembersnode.PopulateOnDemand = false;
                DesignerSelectedMember.Nodes.Add(levelmembersnode);
                TreeNode levelcurrentmembernode = new TreeNode(selecteddimension.Name + "." +
                    thislevel.Name + ".CurrentMember");
                levelcurrentmembernode.PopulateOnDemand = false;
                DesignerSelectedMember.Nodes.Add(levelcurrentmembernode);
            }

            // Now cycle through each parent, prepare it and it's descendants, and add it
            TreeNode parentnode = new TreeNode("Members");
            parentnode.PopulateOnDemand = false;
            foreach (Member thismember in selecteddimension.Members.Where(testmember =>
                testmember.Parent == null || testmember.Parent == new Guid()))
            {
                AddMemberToNode(parentnode, selecteddimension, thismember);
            }                            
            DesignerSelectedMember.Nodes.Add(parentnode);
        }

        private void AddMemberToNode(TreeNode parentnode, Dimension selecteddimension, Member currentmember)
        {
            // Create a new node
            TreeNode thisnode = new TreeNode(selecteddimension.Name + "." + currentmember.Name);
            thisnode.PopulateOnDemand = false;

            // Now check whether this member has any children
            if (currentmember.Children.Count > 0)
            {
                // First add the "Children" method
                TreeNode allchildren = new TreeNode(selecteddimension.Name + "." +
                    currentmember.Name + ".Children");
                allchildren.PopulateOnDemand = false;
                thisnode.ChildNodes.Add(allchildren);

                // Then call the add method on each child node
                foreach (Guid thischild in currentmember.Children)
                {
                    Member thischildmember = selecteddimension.Members[thischild];
                    AddMemberToNode(thisnode, selecteddimension, thischildmember);
                }

                // Now add the node
                parentnode.ChildNodes.Add(thisnode);
            }
            else
            {
                // This member has no children, so just add it and continue
                parentnode.ChildNodes.Add(thisnode);
            }
        }

        #endregion  
    }
}
