﻿/*
 * This source file (CubeExplorer.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.AddIns.DesktopResults;

namespace Bamboo.Cubed.AddIns.DesktopSearches
{
    /// <summary>
    /// Provides the ability to explore underlying cubes and return their results
    /// </summary>
    public partial class PageExplorer : CubedSearch
    {
        #region Properties

        /// <summary>
        /// The identifier of the search this grid is a part of
        /// </summary>
        public override Guid SearchID
        {
            get
            {
                if (Session[this.ID + "_SearchID"] == null)
                    Session[this.ID + "_SearchID"] = Guid.Empty;
                return (Guid)Session[this.ID + "_SearchID"];
            }
            set
            {
                Session[this.ID + "_SearchID"] = value;
            }
        }

        /// <summary>
        /// These are the current settings for the connection to the underlying cube
        /// </summary>
        public PageExplorerDisplaySettings DisplaySettings
        {
            get
            {
                if (Session[this.ID + "_DisplaySettings"] == null)
                    Session[this.ID + "_DisplaySettings"] = new PageExplorerDisplaySettings();
                return (PageExplorerDisplaySettings)Session[this.ID + "_DisplaySettings"];
            }
            set
            {
                Session[this.ID + "_DisplaySettings"] = value;
            }
        }        

        /// <summary>
        /// The unique identifier of the Add In
        /// </summary>
        public override Guid AddInID
        {
            get { return new Guid("4128e8e1-78ed-4beb-b994-a4dc768f638d"); }
        }

        /// <summary>
        /// The name of the Add In
        /// </summary>
        public override string Name
        {
            get { return "Page Explorer"; }
        }

        /// <summary>
        /// The description of the Add In
        /// </summary>
        public override string Description
        {
            get { return "Permits exploring WEB Pages."; }
        }

        /// <summary>
        /// The current version of the Add In
        /// </summary>
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        /// <summary>
        /// Returns the name of the ASCX markup file used for this control
        /// </summary>
        public override string UserControlName
        {
            get { return "PageExplorer.ascx"; }
        }

        /// <summary>
        /// The connection string used to access the search resource
        /// </summary>
        public override string ConnectionString
        {
            get
            {
                if (Session[this.ID + "_ConnectionString"] == null)
                    Session[this.ID + "_ConnectionString"] = String.Empty;
                return (string)Session[this.ID + "_ConnectionString"];
            }
            set
            {
                Session[this.ID + "_ConnectionString"] = value;
            }
        }

        /// <summary>
        /// Returns the list of supported results viewers
        /// </summary>
        public override List<CubedResult> ResultsViewers
        {
            get 
            {
                List<CubedResult> supportedviewers = new List<CubedResult>();
                supportedviewers.Add(new IFrame());
                return supportedviewers;
            }
        }

        /// <summary>
        /// The list of cubes available for this connection
        /// </summary>
        public GenericKeyCollection<Guid, Cube> AvailableCubes
        {
            get
            {
                if (Session[this.ID + "_AvailableCubes"] == null)
                    Session[this.ID + "_AvailableCubes"] = new GenericKeyCollection<Guid, Cube>();
                return (GenericKeyCollection<Guid, Cube>)Session[this.ID + "_AvailableCubes"];
            }
            set
            {
                Session[this.ID + "_AvailableCubes"] = value;
            }
        }

        #endregion

        #region Implemented Methods

        /// <summary>
        /// Returns the current active settings as a string ready for saving
        /// </summary>
        /// <returns>The current settings as a Base64 encoded string ready for saving</returns>
        public override string SaveWebSearch()
        {
                DisplaySettings.CurrentURL = URLCommand.Text;
            
            // Then return them
            return DisplaySettings.ToString();
        }

        /// <summary>
        /// This loads the current search settings from the given Base64 string
        /// </summary>
        /// <param name="searchdetails">A Base64 string containing the current settings</param>
        /// <returns>True if successful, false if not</returns>
        public override bool LoadWebSearch(string searchdetails)
        {

            // Read our settings
            DisplaySettings.FromString(searchdetails);

            // Then update our URL
            URLCommand.Text = DisplaySettings.CurrentURL;


            return true;
        }

        /// <summary>
        /// This executes the current search and returns the results as an object
        /// </summary>
        /// <returns>The current web results as an object</returns>
        public override object GetWebResults()
        {
            try
            {
                return URLCommand.Text;
            }
            catch
            {
                // An error occured so return an empty MDDataSet
                return "";
            }
        }

        #endregion

        #region Events

        /// <summary>
        /// On page load the current settings are checked and loaded if needed, as are the 
        /// collection of Cubes available on the given server
        /// </summary>
        protected void Page_Load(object sender, EventArgs e)
        {
            // Check if we have any cubes and try and load them if not
            //if (AvailableCubes.Count < 1 || SelectedCube.Items.Count < 1)
            //    PrepareCubes();
        }

        #endregion

        #region Support Methods

        /// <summary>
        /// Prepares the dimension list from the given cube
        /// </summary>
        /// <param name="selectedcube">The cube selected</param>
        private void PrepareDimensions(Cube selectedcube)
        {
            // Clear the current list
            URLCommand.Enabled = true;

        }

        #endregion        
    }
}
