/*
 * This source file (RelationalExplorerDisplaySettings.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.AddIns.DesktopSearches
{
    /// <summary>
    /// This class stores the required settings for displaying the Relational Explorer
    /// </summary>
    [Serializable]
    public class RelationalExplorerDisplaySettings
    {
        /// <summary>
        /// Initialises a new default instance of the Relational Explorer display settings
        /// </summary>
        public RelationalExplorerDisplaySettings()
        {
            _selectedcube = new Guid();
            _selecteddatasource = new Guid();
            _currentsql = String.Empty;
            _includezeros = true;
        }

        private Guid _selectedcube;
        /// <summary>
        /// The currently selected cube
        /// </summary>
        public Guid SelectedCube
        {
            get { return _selectedcube; }
            set { _selectedcube = value; }
        }

        private Guid _selecteddatasource;
        /// <summary>
        /// The currently selected data source
        /// </summary>
        public Guid SelectedDataSource
        {
            get { return _selecteddatasource; }
            set { _selecteddatasource = value; }
        }

        private string _currentsql;
        /// <summary>
        /// The current SQL command
        /// </summary>
        public string CurrentSQL
        {
            get { return _currentsql; }
            set { _currentsql = value; }
        }

        private bool _includezeros;
        /// <summary>
        /// Whether to include zeros in returned results
        /// </summary>
        public bool IncludeZeros
        {
            get { return _includezeros; }
            set { _includezeros = value; }
        }

        /// <summary>
        /// This sets the settings based on the passed in Base64 encoded string
        /// </summary>
        /// <param name="settings">A base64 encoded string containing settings</param>
        public void FromString(string settings)
        {
            string regularstring = UnicodeEncoding.Unicode.GetString(Convert.FromBase64String(settings));
            string[] settingsstrings = regularstring.Split(';');
            if (settingsstrings.Length == 4)
            {
                _selectedcube = new Guid(settingsstrings[0]);
                _selecteddatasource = new Guid(settingsstrings[1]);
                _currentsql = settingsstrings[2];
                bool includezeros = true;
                bool.TryParse(settingsstrings[3], out includezeros);
                _includezeros = includezeros;
            }
        }

        /// <summary>
        /// This returns the settings as a Base64 encoded string
        /// </summary>
        /// <returns>A base64 encoded string form of the settings</returns>
        public override string ToString()
        {
            string regularsettings = _selectedcube.ToString() + ";";
            regularsettings += _selecteddatasource.ToString() + ";";
            regularsettings += _currentsql + ";";
            regularsettings += _includezeros.ToString();
            return Convert.ToBase64String(UnicodeEncoding.Unicode.GetBytes(regularsettings));
        }
    }
}