/*
 * This source file (Percentage.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using System.IO;

namespace Bamboo.Cubed.AddIns.KPICalculators
{
    /// <summary>
    /// This AddIn provides a Percentage KPI Calculator for Cubed
    /// </summary>
    [XmlRoot("Percentage")]
    public class Percentage : CubedKPICalculator
    {
        /// <summary>
        /// Initialises a new default instance of the Add In
        /// </summary>
        public Percentage()
        {
            _properties = new PercentageProperties();
            base.InitialiseProperties(String.Empty, 1.0);
        }

        /// <summary>
        /// Initialises the Add In based on the passed in _properties
        /// </summary>
        /// <param name="configurationxml">The Base64 encoded XML string containing the configuration</param>
        /// <param name="configurationversion">The version of the configuration that was used</param>
        public Percentage(string configurationxml, double configurationversion)
        {
            _properties = new PercentageProperties();
            base.InitialiseProperties(configurationxml, configurationversion);
        }

        /// <summary>
        /// The unique identifier of the Percentage KPI Calculator
        /// </summary>
        [XmlElement("ID")]
        public override Guid ID
        {
            get { return new Guid("bb2d6e91-d13e-48a1-b948-bc5505328b41"); }
        }

        /// <summary>
        /// The name of the Percentage KPI Calculator
        /// </summary>
        [XmlElement("Name")]
        public override string Name
        {
            get
            {
                return "Percentage";
            }
        }

        /// <summary>
        /// The description of the Percentage KPI Calculator
        /// </summary>
        [XmlElement("Description")]
        public override string Description
        {
            get { return "A Percentage KPI returns a percentage value relative to a set target."; }
        }

        /// <summary>
        /// The current version of the Percentage KPI Calculator
        /// </summary>
        [XmlElement("CurrentVersion")]
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        /// <summary>
        /// The configuration control for Percentage KPI Calculator
        /// </summary>
        [XmlElement("ConfigurationControl")]
        public override CubedKPICalculatorConfiguration ConfigurationControl
        {
            get { return new PercentageConfiguration(); }
        }

        private PercentageProperties _properties;
        /// <summary>
        /// This represents the _properties of the Add In
        /// </summary>
        public PercentageProperties Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        /// <summary>
        /// This deserialises the current properties of the Add In from the serialised string
        /// </summary>
        /// <param name="configurationstring">The serialised XML string</param>
        /// <returns>True if the _properties were deserialised successfully</returns>
        public override bool WriteConfiguration(string configurationstring)
        {
            if (configurationstring.Length > 0)
            {
                // The configuration string is complete, so lets use it                    
                //

                // Return true to indicate it was read correctly
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// This updates a version of the Add In from its' previous configuration to the latest
        /// </summary>
        /// <param name="currentconfiguration">The saved (previous) configuration XML</param>
        /// <param name="configurationversion">The current version of the Add In</param>
        /// <returns>The updated configuration</returns>
        public override string UpdateConfiguration(string currentconfiguration, double configurationversion)
        {
            // As this is the first release, there should be no need to update
            return currentconfiguration;
        }

        /// <summary>
        /// This evaluates the values of the Percentage KPI, for the members, measure and dimension set. 
        /// It then sets the values in the data matrix to reflect the evaluated values.
        /// </summary>
        /// <param name="kpi">The KPI to evaluate</param>
        /// <param name="cube">The Cube the KPI is part of</param>
        /// <param name="datamatrix">The DataMatrix for the specified cube</param>
        /// <returns>A string containing the results of the KPI evaluation (a percentage value)</returns>
        public override string Evaluate(KPI kpi, Cube cube, DataMatrix datamatrix)
        {
            return String.Empty;
        }
    }
}