/*
 * This source file (RankingConfiguration.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.AddIns;
using System.IO;

namespace Bamboo.Cubed.AddIns.KPICalculators
{
    /// <summary>
    /// This class configures the Ranking KPI Calculator
    /// </summary>
    [XmlRoot("RankingConfiguration")]
    public partial class RankingConfiguration : CubedKPICalculatorConfiguration
    {
        /// <summary>
        /// Initialise the new Add In Configuration conntrol
        /// </summary>
        public RankingConfiguration()
        {
            InitializeComponent();
            _properties = new RankingProperties();
            InitialiseProperties(String.Empty, 1.0);
            UpdateScreen();
        }

        /// <summary>
        /// Initialises a new instance of the Add In configuration control, based on the passed in 
        /// XML representing the current configuration and the version of the Add In the configuration was
        /// created under
        /// </summary>
        /// <param name="configurationxml">The Base64 encoded XML string containing the stored _properties of
        /// the Add In</param>
        /// <param name="configurationversion">The version the _properties were created for</param>
        public RankingConfiguration(string configurationxml, double configurationversion)
        {
            InitializeComponent();
            _properties = new RankingProperties();
            InitialiseProperties(configurationxml, configurationversion);
            UpdateScreen();
        }

        private void UpdateScreen()
        {
        }

        /// <summary>
        /// The unique identifier of the Ranking KPI Calculator
        /// </summary>
        [XmlElement("ID")]
        public override Guid ID
        {
            get { return new Guid("f8966f0e-8efc-40f8-a5ff-697afee83697"); }
        }

        /// <summary>
        /// The name of the Ranking KPI Calculator
        /// </summary>
        [XmlElement("Name")]
        public override string AddInName
        {
            get
            {
                return "Ranking";
            }
        }        

        /// <summary>
        /// The current version of the Ranking KPI Calculator
        /// </summary>
        [XmlElement("CurrentVersion")]
        public override double CurrentVersion
        {
            get { return 1.0; }
        }

        private RankingProperties _properties;
        /// <summary>
        /// This represents the _properties of the Add In
        /// </summary>
        public RankingProperties Properties
        {
            get { return _properties; }
            set { _properties = value; }
        }

        /// <summary>
        /// This deserialises the current properties of the Add In from the serialised string
        /// </summary>
        /// <param name="configurationstring">The serialised XML string</param>
        /// <returns>True if the _properties were deserialised successfully</returns>
        public override bool WriteConfiguration(string configurationstring)
        {
            if (configurationstring.Length > 0)
            {
                // The configuration string is complete, so lets use it                    
                //

                // Return true to indicate it was read correctly
                UpdateScreen();
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// This reads the current configuration and returns a string containing the configuration
        /// </summary>
        /// <returns>A serialised string</returns>
        public override string ReadConfiguration()
        {
            return String.Empty;
        }

        /// <summary>
        /// This updates a version of the Add In from its' previous configuration to the latest
        /// </summary>
        /// <param name="currentconfiguration">The saved (previous) configuration XML</param>
        /// <param name="configurationversion">The current version of the Add In</param>
        /// <returns>The updated configuration</returns>
        public override string UpdateConfiguration(string currentconfiguration, double configurationversion)
        {
            // As this is the first release, there should be no need to update
            return currentconfiguration;
        }

        /// <summary>
        /// This initialises the Ranking KPI configuration with the KPI and Cube details
        /// </summary>
        /// <param name="_kpi">The KPI being configured</param>
        /// <param name="_cube">The Cube the KPI exists in</param>
        public override void Initialise(KPI _kpi, Cube _cube)
        {
            MyKPI = _kpi;
            MyCube = _cube;
        }

        /// <summary>
        /// This is the KPI being configured by the control
        /// </summary>
        private KPI MyKPI;
        /// <summary>
        /// This is the Cube which the KPI exists in
        /// </summary>
        private Cube MyCube;
    }
}