/*
 * This source file (CubeConnectionException.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This class represents an exception generated by errors connecting to a cube
    /// </summary>
    public class CubeConnectionException : Exception
    {
        /// <summary>
        /// Initialises a new instance of the Cube Connection based on the passed in details
        /// </summary>
        /// <param name="ErrorMessage">The custom error message to return</param>
        /// <param name="OriginalException">The original exception thrown internally</param>
        /// <param name="Details">Other details of the error</param>
        public CubeConnectionException(string ErrorMessage, Exception OriginalException, string Details)
        {
            innerexception = OriginalException;
            message = ErrorMessage;
            stacktrace = Details;
        }

        private string message;
        /// <summary>
        /// The Error Message provided by the Exception
        /// </summary>
        public override string Message
        {
            get
            {
                return message;
            }
        }

        private string stacktrace;
        /// <summary>
        /// The Stack Trace containing information about the Exception
        /// </summary>
        public override string StackTrace
        {
            get
            {
                return stacktrace;
            }
        }

        private Exception innerexception;
        /// <summary>
        /// The original Exception that caused this exception
        /// </summary>
        public new Exception InnerException
        {
            get
            {
                return new Exception();
            }
        }
    }
}
