/*
 * This source file (CubedParameter.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;

namespace Bamboo.Cubed.Connect
{
    /// <summary>
    /// This provides a parameter class for attaching parameters to a Cubed Command
    /// </summary>
    public class CubedParameter : IDbDataParameter
    {
        private byte _precision;
        /// <summary>
        /// The precision of the parameter (if this has a numeric value)
        /// </summary>
        public byte Precision
        {
            get { return _precision; }
            set { _precision = value; }
        }

        private byte _scale;
        /// <summary>
        /// The scale of the parameter (if this has a numeric value)
        /// </summary>
        public byte Scale
        {
            get { return _scale; }
            set { _scale = value; }
        }

        private int _size;
        /// <summary>
        /// The size of the parameter (if this has a numeric value)
        /// </summary>
        public int Size
        {
            get { return _size; }
            set { _size = value; }
        }

        private DbType _dbtype;
        /// <summary>
        /// The type of value stored in the parameter
        /// </summary>
        public DbType DbType
        {
            get { return _dbtype; }
            set { _dbtype = value; }
        }

        private ParameterDirection _direction;
        /// <summary>
        /// The direction of the parameter (whether it is sending values to a function or returning them from
        /// a function
        /// </summary>
        public ParameterDirection Direction
        {
            get { return _direction; }
            set { _direction = value; }
        }

        private bool _isnullable;
        /// <summary>
        /// Whether this parameter can contain a null value or not
        /// </summary>
        public bool IsNullable
        {
            get { return _isnullable; }
            set { _isnullable = value; }
        }

        private string _parametername;
        /// <summary>
        /// The name of the parameter
        /// </summary>
        public string ParameterName
        {
            get { return _parametername; }
            set { _parametername = value; }
        }

        private string _sourcecolumn;
        /// <summary>
        /// The source column for this parameter's value
        /// </summary>
        public string SourceColumn
        {
            get { return _sourcecolumn; }
            set { _sourcecolumn = value; }
        }

        private DataRowVersion _sourceversion;
        /// <summary>
        /// The version of the source of this parameter's value
        /// </summary>
        public DataRowVersion SourceVersion
        {
            get { return _sourceversion; }
            set { _sourceversion = value; }
        }

        private object _value;
        /// <summary>
        /// The value of this parameter (which should match the value type indicated in the DBType property)
        /// </summary>
        public object Value
        {
            get { return _value; }
            set { _value = value; }
        }
    }
}
