﻿/*
 * This source file (Default.aspx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Xml.Linq;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Engine;
using Bamboo.Cubed.Engine.Desktop;
using Bamboo.Cubed.AddIns;
using AjaxControlToolkit;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Utilities.Logging;
using System.Threading;

namespace Bamboo.Cubed.Desktop
{
    /// <summary>
    /// This is the primary web page used by the desktop
    /// </summary>
    public partial class Default : System.Web.UI.Page
    {
        #region Internal Properties

        /// <summary>
        /// This contains the current Cubed connection string
        /// </summary>
        public string ConnectionString
        {
            get
            {
                if (Session["ConnectionString"] == null || Session["ConnectionString"].ToString() == String.Empty)
                {
                    // Standard integrated authentication is being used
                    if (ConfigurationManager.AppSettings.AllKeys.Contains("Server") &&
                        ConfigurationManager.AppSettings.AllKeys.Contains("Port") &&
                        ConfigurationManager.AppSettings.AllKeys.Contains("Protocol"))
                    {
                        Session["ConnectionString"] = "Server=" + ConfigurationManager.AppSettings["Server"] +
                            ";Protocol=" + ConfigurationManager.AppSettings["Protocol"] + ";Port=" +
                            ConfigurationManager.AppSettings["Port"] + ";User ID=" +
                             HttpContext.Current.User.Identity.Name + ";Integrated Security=true";
                        return (string)Session["ConnectionString"];
                    }
                    else
                        return null;
                }
                else
                    return (string)Session["ConnectionString"];
            }
            set
            {
                Session["ConnectionString"] = value;
            }
        }

        #endregion

        #region Events

        /// <summary>
        /// Initialises the default settings on page load
        /// </summary>
        protected override void OnInit(EventArgs e)
        {
            DevExpress.Web.ASPxPivotGrid.ASPxPivotGrid.RegisterBaseScript(this.Page);
            DevExpress.Web.ASPxPivotGrid.ASPxPivotGrid.RegisterUtilsScript(this.Page);

            SearchResults.SearchDataChange += new SearchEventDataChange(SearchResults_DataChange);

            if (!Page.IsPostBack)
            {
                // Set out active component property for logging purposes
                Information.ActiveComponent = "Cubed Desktop";

                // Set our logging level
                if (ConfigurationManager.AppSettings.AllKeys.Contains("LoggingLevel") && 
                    Enum.GetNames(typeof(LoggingLevels)).Contains(ConfigurationManager.AppSettings["LoggingLevel"]))
                {
                    Information.LoggingLevel = (LoggingLevels)Enum.Parse(typeof(LoggingLevels),
                        ConfigurationManager.AppSettings["LoggingLevel"]);
                }
            }

            try
            {
                // Check to see if the current user is permitted to access at least one cube in the 
                // collection, otherwise return an access denied page
                if (!CheckAccess())                    
                    Response.Redirect("AccessDenied.aspx");
                else
                {
                    // We have a valid connection string so create a connection and assign it to our active connection
                    // manager
                    if (((string)Session["ConnectionString"]) == null || ((string)Session["ConnectionString"]) == String.Empty)
                        Session["ConnectionString"] = ConnectionString;

                    LoggingManager.Write("User Connection String", ((string)Session["ConnectionString"]), LoggingTypes.Event);
                }
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }

            base.OnInit(e);

            try
            {
                // Load all controls
                LoadControls();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        void SearchResults_DataChange(object sender, SearchDataChangeEventArgs e)
        {
            if (e.Action == SearchDataChangeEventArgs.SearchDataChangeAction.DataDeleted)
            {
                executeSearch(null);
                SearchPanel.Update();
            }
            else if (e.Action == SearchDataChangeEventArgs.SearchDataChangeAction.EditData)
            {
                //string ViewId = e.SearchID + "_Tab";
                //Session["activeTabId"] = ViewId;
                //View view = DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == ViewId);
                //if (view == null)
                //{
                    DesktopSearch search = WebDesktopManager.ReadSearch(new Guid(e.SearchID));
                    //SearchCreatorControl searcher = (SearchCreatorControl)LoadControl("SearchCreatorControl.ascx");
                    //searcher.ID = e.SearchID + "_SearchControl";
                    //searcher.SearchHeaderLabel.Text = search.Name;
                    //searcher.SearchDescriptionLabel.Text = search.Description;
                    //searcher.IsEditing = true;
                    //searcher.CurrentSearch = search;


                    WebDesktopManager.OpenSearch(HttpContext.Current.User.Identity.Name, search.ID, search.DefaultResult, true);
                    Session["activeTabId"] = search.ID + "_EditItemTab";
                    //view = new View();
                    //view.ID = ViewId;
                    //view.Controls.Add(searcher);
                    //DesktopTabs.Views.Add(view);
                    //((System.Web.UI.WebControls.BulletedList)apExplorers.FindControl("blExplorers")).Items.Add(new ListItem(searcher.CurrentSearch.Name, view.ID));
                //}
                //DesktopTabs.SetActiveView(view);
                
            }
            else if (e.Action == SearchDataChangeEventArgs.SearchDataChangeAction.ExportData)
            {
                DesktopSearch search = WebDesktopManager.ReadSearch(new Guid(e.SearchID));
                byte[] data = XMLManager<DesktopSearch>.SerialiseByteArray(search);

                Response.Clear();
                Response.Buffer = false;
                Response.AppendHeader("Content-Type", "text/plain");
                Response.AppendHeader("Content-Transfer-Encoding", "binary");
                Response.AppendHeader("Content-Disposition", "attachment; filename=DesktopSearchExport.xml");
                Response.BinaryWrite(data);
                Response.End();
            }
        }

        /// <summary>
        /// Creates all child controls ready for display
        /// </summary>
        protected void LoadControls()
        {
            try
            {
                // Create our search controls if the user has permission to create either public or
                // private searches
                if (WebDesktopManager.AuthenticatedUser().PublicSearchRights != AccessTypes.None ||
                    WebDesktopManager.AuthenticatedUser().PrivateSearchRights != AccessTypes.None)
                {
                    foreach (CubedSearchDescription thissearch in AddInManager.SearchControls)
                    {
                        // Initialise a search control for this search
                        SearchCreatorControl searcher = (SearchCreatorControl)LoadControl("SearchCreatorControl.ascx");
                        searcher.ID = thissearch.ID + "_SearchControl";
                        searcher.CurrentSearch.SearchID = thissearch.ID;
                        searcher.CurrentSearch.SearchVersion = thissearch.CurrentVersion;
                        searcher.SearchHeaderLabel.Text = thissearch.Name;
                        searcher.SearchDescriptionLabel.Text = thissearch.Description;
                        searcher.SearchCategoryCreated += new SearchCategoryEventDelegate(searcher_SearchCategoryCreated);

                        // Then add all the default viewers for this search if they haven't already been added
                        if (searcher.CurrentSearch.AvailableViewers.Count < 1)
                        {
                            foreach (CubedResult thisresult in thissearch.SearchInstance.ResultsViewers)
                            {
                                searcher.CurrentSearch.AvailableViewers.Add(new DesktopSearchResult(thisresult.AddInID,
                                    thisresult.CurrentVersion, String.Empty, true, false));
                            }

                            // And if at least one viewer is available, add it as the default
                            if (searcher.CurrentSearch.AvailableViewers.Count > 0)
                                searcher.CurrentSearch.DefaultResult = searcher.CurrentSearch.AvailableViewers[0].ResultID;
                        }

                        // Now create a new tab for this search creator
                        View newcreatortab = new View();
                        newcreatortab.ID = thissearch.ID + "_Tab";
                        newcreatortab.Controls.Add(searcher);
                        DesktopTabs.Views.Add(newcreatortab);
                        ((System.Web.UI.WebControls.BulletedList)apExplorers.FindControl("blExplorers")).Items.Add(new ListItem(thissearch.Name, newcreatortab.ID));
                    }
                }


                // Call the LoadDashBoardTabs function to get the names of all dashbords to the menu
                LoadDashBoardTabs();

                // Call the LoadItemTabs function to initiate the loading of any search items which have been
                // previously opened (allows for the re-loading of their session if they have made changes to
                // the searches)
                LoadItemTabs();

                LoadBookmarks();

                LoadCategoriesTabs();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        void searcher_SearchCategoryCreated(string categoryname)
        {
            MainUpdatePanel.Update();
        }

        /// <summary>
        /// This runs to check whether any new item tabs have been loaded and adds them to the collection
        /// </summary>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            try
            {
                // Call the LoadDashBoardTabs function again to load any new dashboard 
                LoadDashBoardTabs();

                // Call the LoadItemTabs function again to load any extra search items which have been 
                // open subsequently (by clicking on a search result)
                LoadItemTabs();

                LoadBookmarks();

                LoadCategoriesTabs();

                //Set the active view to the value stored in session
                if (Session["activeTabId"] != null)
                {
                    View v = DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == Session["activeTabId"].ToString());
                    if (v != null)
                    {
                        DesktopTabs.SetActiveView(v);
                    }
                }
            }
            catch (ThreadAbortException te)
            {
                // Thread abort exceptions are caused when the page is refreshed and so we don't need to log them
                // unless we log warnings
                LoggingManager.Write(te, LoggingTypes.Event);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// This initiates a new search
        /// </summary>
        protected void SearchButton_Click(object sender, EventArgs e)
        {
            executeSearch(null);
        }

        /// <summary>
        /// This initiates a new search from the top right search box
        /// </summary>
        protected void MiniSearchButton_Click(object sender, ImageClickEventArgs e)
        {
            DesktopTabs.ActiveViewIndex = 0;
            Session["activeTabId"] = "SearchTab";

            SearchBox.Text = MiniSearchBox.Text;
            executeSearch(null);


        }
        /// <summary>
        /// This executes a new search
        /// </summary>

        protected void executeSearch(SearchCategory category)
        {
            try
            {
                GenericKeyCollection<Guid, DesktopSearch> results;
                // Grab the results from the server
                if (category == null)
                   results  = WebDesktopManager.MatchSearches(new List<string>(SearchBox.Text.Split(' ')));
                else
                    results = WebDesktopManager.MatchSearches(new List<string>(SearchBox.Text.Split(' ')), category);

                Session["SearchResults"] = results;
                SearchResults.Results = results;
                SearchResults.DataBind();

                // Update the labels
                if (results.Count > 0)
                {
                    ResultsHeaderPanel.Visible = true;
                    SearchResultsCountLabel.Text = "(" + results.Count + " results found for <b>" + SearchBox.Text + "</b>.)";
                    NoResultsLabel.Visible = false;

                    System.Web.UI.WebControls.BulletedList bl = (System.Web.UI.WebControls.BulletedList)apSearch.FindControl("blSearch");
                    if (bl.Items.FindByText(SearchBox.Text) == null && SearchBox.Text != "")
                    {
                        if (bl.Items.Count >= 10) bl.Items.RemoveAt(bl.Items.Count - 2);
                        bl.Items.Insert(0, new ListItem(SearchBox.Text, "SearchTab"));
                        //MainUpdatePanel.Update();
                    }

                }
                else
                {
                    ResultsHeaderPanel.Visible = false;
                    NoResultsLabel.Visible = true;
                }


            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }

        }

        /// <summary>
        /// Changes the Active View to the one selected in the left menu
        /// </summary>
        protected void bl_Click(object sender, BulletedListEventArgs e)
        {
            changeActiveView(((System.Web.UI.WebControls.BulletedList)sender).Items[e.Index].Value);
        }
        /// Changes the Active View to the one selected in the left menu and reload a previous search
        /// </summary>
        protected void blSearch_Click(object sender, BulletedListEventArgs e)
        {
            changeActiveView(((System.Web.UI.WebControls.BulletedList)sender).Items[e.Index].Value);

            string blText = ((System.Web.UI.WebControls.BulletedList)sender).Items[e.Index].Text;
            SearchBox.Text = blText;
            MiniSearchBox.Text = blText;
            executeSearch(null);
        }
        /// <summary>
        /// Changes the Active View to the one selected in the left menu
        /// </summary>
        protected void changeActiveView(string ViewId)
        {
            Session["activeTabId"] = ViewId;
            DesktopTabs.SetActiveView(DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == ViewId)); 
            cpeMenu.ClientState = "true";
            cpeMenu.Collapsed = true;

        }
        /// <summary>
        /// This creates a new DashBoard for the user
        /// </summary>
        protected void newDashButton_Click(object sender, ImageClickEventArgs e)
        {
            if (WebDesktopManager.CreateDashboard(HttpContext.Current.User.Identity.Name, newDashBox.Text))
            {
                newDashBox.Text = "";
                MainUpdatePanel.Update();
            }
        }

        /// <summary>
        /// Changes the Active View to the one selected in the left menu and reload a previous search
        /// </summary>
        protected void blBookmarks_Click(object sender, BulletedListEventArgs e)
        {
            changeActiveView(((System.Web.UI.WebControls.BulletedList)sender).Items[e.Index].Value);
        }

        /// <summary>
        /// Displays the Categories to browse all the searches defined on the selected category.
        /// </summary>
        protected void blCategories_Click(object sender, BulletedListEventArgs e)
        {
            DesktopTabs.ActiveViewIndex = 0;
            Session["activeTabId"] = "SearchTab";
            SearchBox.Text = "";
            SearchCategory category = WebDesktopManager.ReadSearchCategory(new Guid(((System.Web.UI.WebControls.BulletedList)sender).Items[e.Index].Value));
            executeSearch(category);
        }

        #endregion

        #region Support Functions

        /// <summary>
        /// This checks the current users access to see if they are permitted to connect to at least
        /// one cube in the collection, otherwise it returns false
        /// </summary>
        /// <returns>True if the user is legitimately authenticated and has permission to access at
        /// least one cube in the collection</returns>
        private bool CheckAccess()
        {
            if (ConnectionString == null)
            {
                // Mark the failed access attempt in the logs as a minor error
                LoggingManager.Write("Failed user access attempt detected", "User: " + HttpContext.Current.User.Identity.Name +
                    " attempted to access Cubed Desktop but the could not authenticate to the server.", LoggingTypes.MinorError);
                return false;
            }
            else if (Session["Authenticated"] != null && Session["Authenticated"] is bool &&
                ((bool)Session["Authenticated"]) == true)
                // This user has already been authenticated this session so don't re-authenticate
                return true;
            else
            {
                // The user is authenticated, so now check if they can access any cubes
                CubedConnection connection = new CubedConnection(ConnectionString);
                connection.Open();
                if (connection.Administration.CubeHeaders.Count < 1)
                {
                    // Mark the failed access attempt in the logs as a minor error
                    LoggingManager.Write("Failed user access attempt detected", "User: " + HttpContext.Current.User.Identity.Name +
                        " attempted to access Cubed Desktop but had no rights to any cubes.", LoggingTypes.MinorError);
                    return false;
                }
                {
                    // The user has access to at least one cube so allow them in
                    Session["Authenticated"] = true;
                    return true;
                }
            }
        }

        /// <summary>
        /// This creates the loaded results items as new tabs
        /// </summary>
        private void LoadItemTabs()
        {
            // Check if there are any items open to load
            if (WebDesktopManager.UserSearches(HttpContext.Current.User.Identity.Name).Count > 0)
            {
                // We have at least one open search item and all search items open have matching 
                // result items
                foreach (UserSearchResult thisopensearch in WebDesktopManager.UserSearches(HttpContext.Current.User.Identity.Name))
                {
                    // Load the search
                    DesktopSearch search = WebDesktopManager.ReadSearch(thisopensearch.Search);

                    // Then search the default result on the search to the selected result
                    if (thisopensearch.IsEditing)
                    {
                        if (DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == search.ID.ToString() + "_EditItemTab") == null)
                        {
                            // There is no search open with this combination, so load it
                            SearchCreatorControl searcher = (SearchCreatorControl)LoadControl("SearchCreatorControl.ascx");
                            searcher.ID = search.ID.ToString() + "_EditItemControl";
                            searcher.SearchHeaderLabel.Text = search.Name;
                            searcher.SearchDescriptionLabel.Text = search.Description;
                            searcher.IsEditing = true;
                            searcher.CurrentSearch = search;

                            // Now create a new tab for this search creator
                            View newitemtab = new View();
                            newitemtab.ID = search.ID.ToString()  + "_EditItemTab";
                            newitemtab.Controls.Add(searcher);
                            DesktopTabs.Views.Add(newitemtab);
                            ((System.Web.UI.WebControls.BulletedList)apExplorers.FindControl("blExplorers")).Items.Add(new ListItem(search.Name + " - " + AddInManager.ResultControls[search.DefaultResult].ResultInstance.Name, newitemtab.ID));
                            DesktopTabs.SetActiveView(newitemtab);
                        }
                    }
                    // Now check if there is already an open tab with this search/result combination
                    else if (DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == search.ID.ToString() + "_" + search.DefaultResult.ToString() + "_ItemTab") == null)
                    {
                        search.DefaultResult = thisopensearch.Result;
                        // There is no search open with this combination, so load it
                        SearchItemControl item = (SearchItemControl)LoadControl("SearchItemControl.ascx");
                        item.ID = search.ID.ToString() + "_" + search.DefaultResult.ToString() + "_ItemControl";
                        item.CurrentSearch = search;
                        item.SearchItemNameLabel.Text = search.Name;
                        item.SearchItemDescriptionLabel.Text = search.Description;

                        // Now create a new tab for this search creator
                        View newitemtab = new View();
                        newitemtab.ID = search.ID.ToString() + "_" + search.DefaultResult.ToString() + "_ItemTab";
                        newitemtab.Controls.Add(item);
                        
                        // Add the view if it hasn't already been added
                        if (DesktopTabs.Views.Cast<View>().FirstOrDefault(testview =>
                            testview.ID == newitemtab.ID) == null)
                        {
                            DesktopTabs.Views.Add(newitemtab);
                            DesktopTabs.SetActiveView(newitemtab);
                        }
                        System.Web.UI.WebControls.BulletedList apopenitems = (System.Web.UI.WebControls.BulletedList)apOpenItems.FindControl("blOpenItems");
                        if (apopenitems.Items.FindByValue(newitemtab.ID) == null)
                            apopenitems.Items.Add(new ListItem(search.Name + " - " + AddInManager.ResultControls[search.DefaultResult].ResultInstance.Name, newitemtab.ID));                        
                    }
                }
                MainUpdatePanel.Update();

            }

            // Then check if there are any tabs open which shouldn't be (check whether any have been closed)
            for (int i = DesktopTabs.Views.Count - 1; i > -1; i--)
            {
                if (DesktopTabs.Views[i].ID.Contains("_ItemTab"))
                {
                    // This is an item tab so we need to check the search and result ID to check they 
                    // are still valid
                    Guid itemsearchid = new Guid(DesktopTabs.Views[i].ID.Substring(0, 36));
                    Guid itemresultid = new Guid(DesktopTabs.Views[i].ID.Substring(37, 36));

                    bool valid = false;
                    if (WebDesktopManager.UserSearches(HttpContext.Current.User.Identity.Name).Exists(testsearch =>
                        testsearch.Search == itemsearchid && testsearch.Result == itemresultid))
                        valid = true;

                    if (!valid)
                    {
                        // This combination of the search and result are not found in our collection so
                        // we assume that it is incorrectly displayed and close it (it may have been removed)
                        DesktopTabs.Views.RemoveAt(i);                        
                        MainUpdatePanel.Update();

                        // Then we need to check if this is the last tab panel - and if it is we need
                        // to reload the page due to an issue with the tab panel
                        if (i >= DesktopTabs.Views.Count)
                        {
                            Response.Redirect("Default.aspx");
                        }
                    }
                }
            }            
        }

        /// <summary>
        /// This creates the loaded results items as new tabs
        /// </summary>
        private void LoadDashBoardTabs()
        {
            ((System.Web.UI.WebControls.BulletedList)apDashboards.FindControl("blDashBoard")).Items.Clear();

            // Get the dashboards for the current user
            foreach (DesktopDashboard dashboard in WebDesktopManager.ReadDashboards(HttpContext.Current.User.Identity.Name).OrderBy(dash => dash.Description))
            {
                string newitemtabId = dashboard.ID.ToString() + "_DashboardTab";
                // Now check if there is already an open tab with this dashboard
                if (DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == newitemtabId) == null)
                {
                    DesktopDashboardControl item = (DesktopDashboardControl)LoadControl("DesktopDashboardControl.ascx");
                    item.ID = dashboard.ID.ToString() + "_DesktopDashboardControl";
                    item.DashBoardID = dashboard.ID;
                    item.DashboardDeleted += new DesktopDashboardControlEventDelegate(item_DashboardDeleted);
                    item.DashboardRenamed += new DesktopDashboardControlEventDelegate(item_DashboardRenamed);

                    // Now create a new tab for this dashboard
                    View newitemtab = new View();
                    newitemtab.ID = newitemtabId; 
                    newitemtab.Controls.Add(item);
                    DesktopTabs.Views.Add(newitemtab);
                    DesktopTabs.SetActiveView(newitemtab);

                }
                ((System.Web.UI.WebControls.BulletedList)apDashboards.FindControl("blDashBoard")).Items.Add(new ListItem(dashboard.Description, newitemtabId));
            }
            MainUpdatePanel.Update();
        }

        /// <summary>
        /// This loads all the categories as new tabs.
        /// </summary>
        private void LoadCategoriesTabs()
        {
            ((System.Web.UI.WebControls.BulletedList)apCategories.FindControl("blCategories")).Items.Clear();

            // Get the current Search Categories.
            SearchCategory[] searchcategories = WebDesktopManager.SearchCategories();
            foreach (SearchCategory category in searchcategories)
            {
                ((System.Web.UI.WebControls.BulletedList)apCategories.FindControl("blCategories")).Items.Add(new ListItem(category.Name, category.ID.ToString()));
            }
            MainUpdatePanel.Update();

        }

        void item_DashboardRenamed(Guid dashboardID)
        {
            MainUpdatePanel.Update();
        }

        void item_DashboardDeleted(Guid dashboardID)
        {
            // Get the view corresponding to the deleted dashboard.
            string dashboardviewid = dashboardID.ToString() + "_DashboardTab";
            View v = DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == dashboardviewid);
            if (v != null)
            {
                DesktopTabs.Views.Remove(v);
            }

            MainUpdatePanel.Update();
        }

        /// <summary>
        /// This creates the loaded results items as new tabs
        /// </summary>
        private void LoadBookmarks()
        {
            // Check if there are any bookmarks to load.
            if (WebDesktopManager.BookmarkedSearches().Count > 0)
            {
                foreach (DesktopSearch bookmarkedsearch in WebDesktopManager.BookmarkedSearches())
                {
                    if (DesktopTabs.Views.Cast<View>().FirstOrDefault(thistab => thistab.ID == bookmarkedsearch.ID.ToString() + "_" + bookmarkedsearch.DefaultResult.ToString() + "_BookmarkItemTab") == null)
                    {
                        // There is no search open with this combination, so load it
                        SearchItemControl item = (SearchItemControl)LoadControl("SearchItemControl.ascx");
                        item.ID = bookmarkedsearch.ID.ToString() + "_" + bookmarkedsearch.DefaultResult.ToString() + "_BookmarkItemControl";
                        item.CurrentSearch = bookmarkedsearch;
                        item.SearchItemNameLabel.Text = bookmarkedsearch.Name;
                        item.SearchItemDescriptionLabel.Text = bookmarkedsearch.Description;

                        // Now create a new tab for this search creator
                        View newitemtab = new View();
                        newitemtab.ID = bookmarkedsearch.ID.ToString() + "_" + bookmarkedsearch.DefaultResult.ToString() + "_BookmarkItemTab";
                        newitemtab.Controls.Add(item);
                        DesktopTabs.Views.Add(newitemtab);
                        ((System.Web.UI.WebControls.BulletedList)apBookmarks.FindControl("blBookmarks")).Items.Add(new ListItem(bookmarkedsearch.Name + " - " + AddInManager.ResultControls[bookmarkedsearch.DefaultResult].ResultInstance.Name, newitemtab.ID));
                        DesktopTabs.SetActiveView(newitemtab);
                    }
                }
                MainUpdatePanel.Update();
            }
        }
        #endregion
    }
}
