﻿/*
 * This source file (DesktopDashboardControl.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Engine.Desktop;
using System.Xml.Linq;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Desktop
{
    /// <summary>
    /// This control contains a single search item, allowing for choosing of different result viewers and
    /// adding to a users dashboard
    /// </summary>
    public partial class DesktopDashboardControl : System.Web.UI.UserControl
    {
        public event DesktopDashboardControlEventDelegate DashboardDeleted;
        public event DesktopDashboardControlEventDelegate DashboardRenamed;

        #region Internal Properties

        /// <summary>
        /// This is the table which stores the dashboard items
        /// </summary>
        //private HtmlTable dashtable;

        private Guid _dashBoardID;
        public Guid DashBoardID
        {
            get { return _dashBoardID; }
            set { _dashBoardID = value; }
        }
        #endregion

        #region Events

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void RenameDashboard_Click(object sender, ImageClickEventArgs e)
        {
            try
            {
                //If the current Dashboard's ID is not Empty
                if (_dashBoardID != Guid.Empty)
                {
                    // Remove it.
                    if (WebDesktopManager.RenameDashboard(_dashBoardID, renameDashTxt.Text))
                    {
                        // Raise the event.
                        OnDashboardRenamed();

                        // Show the dashboard descripton as Title
                        DashBoradDescriptionDiv.InnerText = renameDashTxt.Text;
                    }
                }
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void DeleteDashboard_Click(object sender, ImageClickEventArgs e)
        {
            try
            {
                //If the current Dashboard's ID is not Empty
                if (_dashBoardID != Guid.Empty)
                {
                    // Remove it.
                    WebDesktopManager.RemoveDashboard(_dashBoardID);
                    
                    // Raise the event.
                    OnDashboardDeleted();
                }
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        protected void OnDashboardDeleted()
        {
            if (DashboardDeleted != null)
                DashboardDeleted(_dashBoardID);
        }

        /// <summary>
        /// 
        /// </summary>
        protected void OnDashboardRenamed()
        {
            if (DashboardRenamed != null)
                DashboardRenamed(_dashBoardID);
        }

        /// <summary>
        /// On initialisation load our dashboard based on the current settings for this user
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            try
            {
                if (_dashBoardID != Guid.Empty)
                {
                    // Grab the current users dashboard
                    DesktopDashboard dashboard = WebDesktopManager.ReadDashboard(_dashBoardID);

                    // Show the dashboard descripton as Title
                    DashBoradDescriptionDiv.InnerText = dashboard.Description;

                    // Load the DashBoaid Item Controls
                    LoadDashboardItems(dashboard);
                }
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// This occurs when an item action event fires
        /// </summary>
        private void item_ItemEvent(DashboardItemActionArgs args)
        {
            try
            {
                // First grab the current users dashboard
                //DesktopDashboard dashboard = WebDesktopManager.ReadDashboard(HttpContext.Current.User.Identity.Name);
                DesktopDashboard dashboard = WebDesktopManager.ReadDashboard(_dashBoardID);

                // Then determine what type of event has been fired
                switch (args.Action)
                {
                    case DashboardItemActionTypes.Remove:
                        // Check if the given item exists in the dashboard collection and remove it if
                        // it does
                        if (dashboard.Items.Contains(args.Item.ID))
                        {
                            dashboard.Items.Remove(args.Item.ID);
                        }
                        break;
                    case DashboardItemActionTypes.Open:
                        // Add the items search / result to the open collection
                        WebDesktopManager.OpenSearch(HttpContext.Current.User.Identity.Name,
                            args.Item.Search, args.Item.Result, false);
                        Session["activeTabId"] = args.Item.Search + "_" + args.Item.Result + "_ItemTab";

                        break;
                    case DashboardItemActionTypes.ChangeRefresh:
                        // First check if the given item exists in the dashboard collection and remove it if
                        // it does
                        if (dashboard.Items.Contains(args.Item.ID))
                            dashboard.Items.Remove(args.Item.ID);

                        // Then re-add the changed item
                        dashboard.Items.Add(args.Item);
                        break;
                    case DashboardItemActionTypes.MoveLeft:
                        // Check if the given item exists in the dashboard collection and remove it if
                        // it does
                        DesktopDashboardItem currentItem = dashboard.Items.FirstOrDefault(curItem => curItem.ID == args.Item.ID);

                        //Get the current position
                        short oldrow = currentItem.Row;
                        short oldcolumn = currentItem.Column;

                        // Find the first element at left to exchange positions (if it exists)
                        DesktopDashboardItem clashingitem = dashboard.Items.FirstOrDefault(nextitem => nextitem.Row == dashboard.Items.Where(testitem => testitem.Row < oldrow).Max(testitem => testitem.Row));
                        if (clashingitem != null)
                        {
                            // An item already exists so set it to this item's old location
                            short clashingitemrow = clashingitem.Row;
                            short clasingitemcolumn = clashingitem.Column;
                            clashingitem.Row = oldrow;
                            clashingitem.Column = oldcolumn;

                            currentItem.Row = clashingitemrow;
                            currentItem.Column = clasingitemcolumn;

                            //Finally, swap items in the collection
                            SwapItemsOnDashboard(dashboard, currentItem, clashingitem);
                        }


                        break;
                    case DashboardItemActionTypes.MoveRight:
                        // Check if the given item exists in the dashboard collection and remove it if
                        // it does
                        currentItem = dashboard.Items.FirstOrDefault(curItem => curItem.ID == args.Item.ID);

                        // Move the item to the right
                        oldrow = currentItem.Row;
                        oldcolumn = currentItem.Column;


                        // Find the first element at right to exchange positions
                        clashingitem = dashboard.Items.FirstOrDefault(nextitem => nextitem.Row == dashboard.Items.Where(testitem => testitem.Row > oldrow).Min(testitem => testitem.Row));
                        if (clashingitem != null)
                        {
                            // An item already exists so set it to this item's old location
                            short clashingitemrow = clashingitem.Row;
                            short clasingitemcolumn = clashingitem.Column;
                            clashingitem.Row = oldrow;
                            clashingitem.Column = oldcolumn;

                            currentItem.Row = clashingitemrow;
                            currentItem.Column = clasingitemcolumn;

                            //Finally, swap items in the collection
                            SwapItemsOnDashboard(dashboard, currentItem, clashingitem);
                        }

                        break;
                }

                // Then, save the changes to the users dashboard
                WebDesktopManager.SaveDashboard(dashboard);

                // Finally, update the desktop dashboard panel to reflect any changes
                LoadDashboardItems(dashboard);
                DesktopDashboardPanel.Update();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }
        /// <summary>
        /// This runs every minute, refreshing any dashboard items which need to be refreshed
        /// </summary>
        protected void RefreshTimer_Tick(object sender, EventArgs e)
        {
            // Cycle through each dashboard item and run it's "refresh" method
            foreach (Control thiscontrol in DesktopDashboardPanel.ContentTemplateContainer.Controls)
                if (thiscontrol is DashboardItemControl)
                {
                    // We have found one of our dashboard items, so call its' refresh method
                    ((DashboardItemControl)thiscontrol).Refresh();
                }
        }

        #endregion 
     
        #region Support Methods

        /// <summary>
        /// This removes the given item from the control dashboard if it has already been placed there
        /// </summary>
        /// <param name="item">The item to remove</param>
        private void RemoveItemFromDashboard(DesktopDashboardItem item)
        {
            //if (dashtable.Rows.Count >= item.Row && dashtable.Rows[item.Row-1].Cells.Count >= item.Column &&
            //    dashtable.Rows[item.Row-1].Cells[item.Column-1].Controls.Count > 0)
            //{
            //    // There is a control at the given items location so remove it
            //    dashtable.Rows[item.Row-1].Cells[item.Column-1].Controls.Clear();
            //}
        }

        /// <summary>
        /// This swaps the location of the items at the two given row/column coordinates
        /// </summary>
        /// <param name="item1row">The first items original row</param>
        /// <param name="item1column">The first items original column</param>
        /// <param name="item2row">The second items original row</param>
        /// <param name="item2column">The second items original column</param>
        private void SwapItemsOnDashboard(DesktopDashboard dashboard, DesktopDashboardItem item1, DesktopDashboardItem item2)
        {
            //for (int i = 0; i < dashboard.Items.Count; i++)
            //{
            //    if (dashboard.Items[i] == item1) dashboard.Items[i] = item2;
            //    if (dashboard.Items[i] == item2) dashboard.Items[i] = item1;
            //}
        }
        protected void LoadDashboardItems(DesktopDashboard dashboard)
        {
            //Remove previously loaded controls
            DesktopDashboardPanel.ContentTemplateContainer.Controls.Clear();

            // Load the DashBoaid Item Controls
            if (dashboard.Items.Count > 0)
            {
                foreach (DesktopDashboardItem thisitem in dashboard.Items.OrderBy(i => i.Row))
                {
                    DashboardItemControl item = (DashboardItemControl)LoadControl("DashboardItemControl.ascx");
                    item.ID = thisitem.ID + "_DashboardItem";
                    item.DashboardItem = thisitem;
                    item.ItemEvent += new DashboardItemEventDelegate(item_ItemEvent);
                    DesktopDashboardPanel.ContentTemplateContainer.Controls.Add(item);
                }
            }
        }

        #endregion
    }
}
