﻿/*
 * This source file (InstallOptions.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Bamboo.Cubed.Structures;
using System.IO;

namespace Bamboo.Cubed.Install
{
    /// <summary>
    /// Defines the options for a Cubed installation
    /// </summary>
    public class InstallOptions
    {
        /// <summary>
        /// Initialises an empty set of options
        /// </summary>
        public InstallOptions()
        {
            _ostype = OSTypes.Win32;
            _installtype = InstallTypes.Basic;
            _installdesigner = true;
            _designerpath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) +
                "\\Bamboo Software\\Cubed\\Designer\\";
            _installservice = true;
            _servicepath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) +
                "\\Bamboo Software\\Cubed\\Service\\";
            _servicename = "Cubed Service";
            _serviceendpoint = new ServiceEndPoint();
            _installdesktop = true;
            _desktoppath = Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles) +
                "\\Bamboo Software\\Cubed\\Desktop\\";
            _webport = 80;
            _webalias = "Cubed";
            _key = String.Empty;
        }

        private OSTypes _ostype;
        /// <summary>
        /// The type of operating system being installed on
        /// </summary>
        public OSTypes OSType
        {
            get { return _ostype; }
            set { _ostype = value; }
        }

        private InstallTypes _installtype;
        /// <summary>
        /// The type of install being done
        /// </summary>
        public InstallTypes InstallType
        {
            get { return _installtype; }
            set { _installtype = value; }
        }

        private bool _installdesigner;
        /// <summary>
        /// Whether to install Cubed Designer
        /// </summary>
        public bool InstallDesigner
        {
            get { return _installdesigner; }
            set { _installdesigner = value; }
        }

        private string _designerpath;
        /// <summary>
        /// The path to install Cubed Designer
        /// </summary>
        public string DesignerPath
        {
            get { return _designerpath; }
            set { _designerpath = value; }
        }

        private bool _installservice;
        /// <summary>
        /// Whether to install Cubed Service
        /// </summary>
        public bool InstallService
        {
            get { return _installservice; }
            set { _installservice = value; }
        }

        private string _servicepath;
        /// <summary>
        /// The path to install Cubed Service
        /// </summary>
        public string ServicePath
        {
            get { return _servicepath; }
            set { _servicepath = value; }
        }

        private ServiceEndPoint _serviceendpoint;
        /// <summary>
        /// The end point to configure initially for the service
        /// </summary>
        public ServiceEndPoint ServiceEndPoint
        {
            get { return _serviceendpoint; }
            set { _serviceendpoint = value; }
        }

        private string _servicename;
        /// <summary>
        /// The name to install the service under
        /// </summary>
        public string ServiceName
        {
            get { return _servicename; }
            set { _servicename = value; }
        }

        private bool _installdesktop;
        /// <summary>
        /// Whether to install Cubed Desktop
        /// </summary>
        public bool InstallDesktop
        {
            get { return _installdesktop; }
            set { _installdesktop = value; }
        }

        private string _desktoppath;
        /// <summary>
        /// The path to install Cubed Desktop
        /// </summary>
        public string DesktopPath
        {
            get { return _desktoppath; }
            set { _desktoppath = value; }
        }

        private int _webport;
        /// <summary>
        /// The port to install the web application on
        /// </summary>
        public int WebPort
        {
            get { return _webport; }
            set { _webport = value; }
        }

        private string _webalias;
        /// <summary>
        /// The alias to install the web application
        /// </summary>
        public string WebAlias
        {
            get { return _webalias; }
            set { _webalias = value; }
        }

        private string _key;
        /// <summary>
        /// The license key for Cubed
        /// </summary>
        public string Key
        {
            get { return _key; }
            set { _key = value; }
        }
    }
}
