﻿/*
 * This source file (ServiceOptions.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Bamboo.Cubed.Utilities;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.Install
{
    /// <summary>
    /// The allows the user to select options for installing the Cubed Service
    /// </summary>
    public partial class ServiceOptions : Form
    {
        /// <summary>
        /// Initialises the install form
        /// </summary>
        public ServiceOptions()
        {
            InitializeComponent();
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void InstallForm_Load(object sender, EventArgs e)
        {
            VersionLabel.Text = Information.Version;
            CopyrightLabel.Text = Information.FullCopyright;

            // Load our end point types drop down
            ServiceEndPointType.Items.Clear();
            foreach (string thisendpoint in Enum.GetNames(typeof(ServiceEndPointType)))
                ServiceEndPointType.Items.Add(thisendpoint);

            // Set the default settings based on the one passed in from the options
            InstallPath.Text = OptionsManager.CurrentOptions.ServicePath;
            ServiceName.Text = OptionsManager.CurrentOptions.ServiceName;
            ServiceEndPointType.Text = OptionsManager.CurrentOptions.ServiceEndPoint.EndpointType.ToString();
            ServerName.Text = OptionsManager.CurrentOptions.ServiceEndPoint.Server;
            Port.Value = Convert.ToDecimal(OptionsManager.CurrentOptions.ServiceEndPoint.Port);
        }

        private void FolderSelectorButton_Click(object sender, EventArgs e)
        {
            if (InstallPathDialog.ShowDialog() == DialogResult.OK)
            {
                InstallPath.Text = InstallPathDialog.SelectedPath;
            }
        }

        private void OKButton_Click(object sender, EventArgs e)
        {
            // Check a valid path has been selected
            if (InstallPath.Text == String.Empty)
                MessageBox.Show("Please select a location to install Cubed Service", "No install location select",
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else
            {
                // Set our install options
                OptionsManager.CurrentOptions.ServicePath = InstallPath.Text;
                OptionsManager.CurrentOptions.ServiceName = ServerName.Text;
                OptionsManager.CurrentOptions.ServiceEndPoint.EndpointType = (ServiceEndPointType)Enum.Parse(typeof(ServiceEndPointType), ServiceEndPointType.Text);
                OptionsManager.CurrentOptions.ServiceEndPoint.Server = ServerName.Text;
                OptionsManager.CurrentOptions.ServiceEndPoint.Port = Convert.ToInt32(Port.Value);

                // Check where to go next
                if (OptionsManager.CurrentOptions.InstallDesktop)
                {
                    // Go to the desktop options screen
                    DesktopOptions desktopoptions = new DesktopOptions();
                    desktopoptions.Show();
                    this.Close();
                }
                else
                {
                    // Go directly to the install begin screen
                    BeginInstall begininstall = new BeginInstall();
                    begininstall.Show();
                    this.Close();
                }
            }
        }
    }
}
