﻿/*
 * This source file (DesktopDashboardControl.ascx.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using Bamboo.Cubed.AddIns;
using Bamboo.Cubed.Structures;
using Bamboo.Cubed.Connect;
using Bamboo.Cubed.Engine.Desktop;
using System.Xml.Linq;
using Bamboo.Cubed.Utilities.Logging;
using Bamboo.Cubed.Utilities;

namespace Bamboo.Cubed.Desktop
{
    /// <summary>
    /// This control contains a single search item, allowing for choosing of different result viewers and
    /// adding to a users dashboard
    /// </summary>
    public partial class DesktopDashboardControl : System.Web.UI.UserControl
    {
        #region Internal Properties

        /// <summary>
        /// This is the table which stores the dashboard items
        /// </summary>
        private HtmlTable dashtable;

        #endregion

        #region Events

        /// <summary>
        /// On initialisation load our dashboard based on the current settings for this user
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            try
            {
                // Grab the current users dashboard
                DesktopDashboard dashboard = WebDesktopManager.ReadDashboard(HttpContext.Current.User.Identity.Name);

                // Create a table to house the items
                dashtable = new HtmlTable();

                // Now check the highest row number in order to determine how many rows to create
                int rows = 0;
                if (dashboard.Items.Count > 0)
                    rows = dashboard.Items.Max(testitem => testitem.Row);


                // Then determine the width and height of items depending on how many rows & columns there are
                int maxcolumns = 0;
                if (dashboard.Items.Count > 0)
                    maxcolumns = dashboard.Items.Max(testitem => testitem.Column);
                double ditemheight = 100;
                if (rows > 0)
                    ditemheight = 100 / rows;
                int iitemheight = Convert.ToInt32(ditemheight);
                double ditemwidth = 100;
                if (maxcolumns > 0)
                    ditemwidth = 100 / maxcolumns;
                int iitemwidth = Convert.ToInt32(ditemwidth);
                while ((iitemheight * rows) > 100)
                    iitemheight--;
                while ((iitemwidth * maxcolumns) > 100)
                    iitemwidth--;
                string itemheight = iitemheight.ToString() + "%";
                string itemwidth = iitemwidth.ToString() + "%";
                dashtable.Height = "100%";
                dashtable.Width = "100%";

                // Cycle through each row and add cells for each column which has a dashboard item
                for (int row = 1; row < rows + 1; row++)
                {
                    // Create a HTML row for this row
                    HtmlTableRow thisdashrow = new HtmlTableRow();
                    thisdashrow.Height = itemheight;

                    // Now determine how many columns we need 
                    int columns = dashboard.Items.Where(testitem => testitem.Row == row).Max(testitem => testitem.Column);

                    // Cycle through each column and add a value if it exists
                    for (int column = 1; column < columns + 1; column++)
                    {
                        // Create a cell for this column
                        HtmlTableCell thisdashcell = new HtmlTableCell();
                        thisdashcell.VAlign = "top";
                        thisdashcell.Align = "center";
                        thisdashcell.Width = itemwidth;
                        thisdashcell.Height = itemheight;

                        // Check whether there is an item at this place and add it if it exists
                        DesktopDashboardItem thisitem = dashboard.Items.FirstOrDefault(testitem => testitem.Row == row &&
                            testitem.Column == column);
                        if (thisitem != null)
                        {
                            // An item exists so add it to the cell
                            DashboardItemControl item = (DashboardItemControl)LoadControl("DashboardItemControl.ascx");
                            item.ID = thisitem.ID + "_DashboardItem";
                            item.DashboardItem = thisitem;
                            item.ItemEvent += new DashboardItemEventDelegate(item_ItemEvent);
                            thisdashcell.Controls.Add(item);
                        }

                        // Then add the cell to the row
                        thisdashrow.Cells.Add(thisdashcell);
                    }

                    // Then add the row to the table
                    dashtable.Rows.Add(thisdashrow);
                }

                // Then add the table to the page
                DesktopDashboardPanel.ContentTemplateContainer.Controls.Add(dashtable);
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// This occurs when an item action event fires
        /// </summary>
        private void item_ItemEvent(DashboardItemActionArgs args)
        {
            try
            {
                // First grab the current users dashboard
                DesktopDashboard dashboard = WebDesktopManager.ReadDashboard(HttpContext.Current.User.Identity.Name);

                // Then determine what type of event has been fired
                switch (args.Action)
                {
                    case DashboardItemActionTypes.Remove:
                        // Check if the given item exists in the dashboard collection and remove it if
                        // it does
                        if (dashboard.Items.Contains(args.Item.ID))
                        {
                            dashboard.Items.Remove(args.Item.ID);
                            RemoveItemFromDashboard(args.Item);
                        }
                        break;
                    case DashboardItemActionTypes.Open:
                        // Add the items search / result to the open collection
                        WebDesktopManager.OpenSearch(HttpContext.Current.User.Identity.Name,
                            args.Item.Search, args.Item.Result);
                        break;
                    case DashboardItemActionTypes.ChangeRefresh:
                        // First check if the given item exists in the dashboard collection and remove it if
                        // it does
                        if (dashboard.Items.Contains(args.Item.ID))
                            dashboard.Items.Remove(args.Item.ID);

                        // Then re-add the changed item
                        dashboard.Items.Add(args.Item);
                        break;
                    case DashboardItemActionTypes.MoveLeft:
                        // Check if the given item exists in the dashboard collection and remove it if
                        // it does
                        if (dashboard.Items.Contains(args.Item.ID))
                            dashboard.Items.Remove(args.Item.ID);

                        // Move the item to the left
                        short oldrow = args.Item.Row;
                        short oldcolumn = args.Item.Column;
                        if (args.Item.Column == 1 && args.Item.Row > 1)
                        {
                            // As this item is in the first column, move it back a row
                            args.Item.Column = 3;
                            args.Item.Row--;
                        }
                        else if (args.Item.Column == 1 && args.Item.Row < 2)
                        {
                            // As this item is in the first row and column, move it to the end
                            args.Item.Row = 3;
                            args.Item.Column = 3;
                        }
                        else
                        {
                            // As the item isn't in the first column, move it back a column
                            args.Item.Column--;
                        }

                        // Then check if an item already exists at this column / row - and if so - update it
                        DesktopDashboardItem clashingitem = dashboard.Items.FirstOrDefault(testitem => testitem.Column == args.Item.Column &&
                            testitem.Row == args.Item.Row);
                        if (clashingitem != null)
                        {
                            // An item already exists so set it to this item's old location
                            short clashingitemrow = clashingitem.Row;
                            short clasingitemcolumn = clashingitem.Column;
                            clashingitem.Row = oldrow;
                            clashingitem.Column = oldcolumn;

                            // Then update the dashboard
                            SwapItemsOnDashboard(oldrow, oldcolumn, clashingitemrow, clasingitemcolumn);
                        }

                        // And finally re-add the moved item back into the collection
                        dashboard.Items.Add(args.Item);
                        break;
                    case DashboardItemActionTypes.MoveRight:
                        // Check if the given item exists in the dashboard collection and remove it if
                        // it does
                        if (dashboard.Items.Contains(args.Item.ID))
                            dashboard.Items.Remove(args.Item.ID);

                        // Move the item to the right
                        oldrow = args.Item.Row;
                        oldcolumn = args.Item.Column;
                        if (args.Item.Column == 3 && args.Item.Row < 3)
                        {
                            // As this item is in the last column, move it down a row
                            args.Item.Column = 1;
                            args.Item.Row++;
                        }
                        else if (args.Item.Column == 3 && args.Item.Row > 2)
                        {
                            // We are at the last row and column so we move the item to the first row
                            // and column
                            args.Item.Row = 1;
                            args.Item.Column = 1;
                        }
                        else
                        {
                            // As the item isn't in the last column, move it forward a column
                            args.Item.Column++;
                        }

                        // Then check if an item already exists at this column / row - and if so - update it
                        clashingitem = dashboard.Items.FirstOrDefault(testitem => testitem.Column == args.Item.Column &&
                           testitem.Row == args.Item.Row);
                        if (clashingitem != null)
                        {
                            // An item already exists so set it to this item's old location
                            short clashingitemrow = clashingitem.Row;
                            short clasingitemcolumn = clashingitem.Column;
                            clashingitem.Row = oldrow;
                            clashingitem.Column = oldcolumn;

                            // Then update the dashboard
                            SwapItemsOnDashboard(oldrow, oldcolumn, clashingitemrow, clasingitemcolumn);
                        }

                        // And finally re-add the moved item back into the collection
                        dashboard.Items.Add(args.Item);
                        break;
                }

                // Then, save the changes to the users dashboard
                WebDesktopManager.SaveDashboard(dashboard);

                // Finally, update the desktop dashboard panel to reflect any changes
                DesktopDashboardPanel.Update();
            }
            catch (Exception e1)
            {
                LoggingManager.Write(e1, LoggingTypes.MajorError);
            }
        }

        /// <summary>
        /// This runs every minute, refreshing any dashboard items which need to be refreshed
        /// </summary>
        protected void RefreshTimer_Tick(object sender, EventArgs e)
        {
            // Cycle through each dashboard item and run it's "refresh" method
            foreach (HtmlTableRow thisdashrow in dashtable.Rows)
                foreach (HtmlTableCell thisdashcell in thisdashrow.Cells)
                    foreach (Control thiscontrol in thisdashcell.Controls)
                        if (thiscontrol is DashboardItemControl)
                        {
                            // We have found one of our dashboard items, so call its' refresh method
                            ((DashboardItemControl)thiscontrol).Refresh();
                        }
        }    

        #endregion 
     
        #region Support Methods

        /// <summary>
        /// This removes the given item from the control dashboard if it has already been placed there
        /// </summary>
        /// <param name="item">The item to remove</param>
        private void RemoveItemFromDashboard(DesktopDashboardItem item)
        {
            if (dashtable.Rows.Count >= item.Row && dashtable.Rows[item.Row-1].Cells.Count >= item.Column &&
                dashtable.Rows[item.Row-1].Cells[item.Column-1].Controls.Count > 0)
            {
                // There is a control at the given items location so remove it
                dashtable.Rows[item.Row-1].Cells[item.Column-1].Controls.Clear();
            }
        }

        /// <summary>
        /// This swaps the location of the items at the two given row/column coordinates
        /// </summary>
        /// <param name="item1row">The first items original row</param>
        /// <param name="item1column">The first items original column</param>
        /// <param name="item2row">The second items original row</param>
        /// <param name="item2column">The second items original column</param>
        private void SwapItemsOnDashboard(short item1row, short item1column, short item2row,
            short item2column)
        {
            if (dashtable.Rows.Count >= item1row && dashtable.Rows[item1row-1].Cells.Count >= item1column &&
                dashtable.Rows[item1row-1].Cells[item1column-1].Controls.Count > 0 &&
                dashtable.Rows.Count >= item2row && dashtable.Rows[item2row-1].Cells.Count >= item2column &&
                dashtable.Rows[item2row-1].Cells[item2column-1].Controls.Count > 0)
            {
                // Both items row/column definitions contain a control so swap them
                Control item1 = dashtable.Rows[item1row-1].Cells[item1column-1].Controls[0];
                dashtable.Rows[item1row-1].Cells[item1column-1].Controls.Clear();
                Control item2 = dashtable.Rows[item2row-1].Cells[item2column-1].Controls[0];
                dashtable.Rows[item2row-1].Cells[item2column-1].Controls.Clear();
                dashtable.Rows[item1row-1].Cells[item1column-1].Controls.Add(item2);
                dashtable.Rows[item2row-1].Cells[item2column-1].Controls.Add(item1);
            }
        }

        #endregion
    }
}