/*
 * This source file (MDMCalculatedMemberHelper.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Data;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.Structures.XMLA;

namespace Bamboo.Cubed.Engine.QueryModel
{
    /// <summary>
    /// An extended Calculated Member
    /// </summary>
    public class MDMCalculatedMemberHelper : MDMCalculatedMember
    {
        private MDMMemberHelper member = new MDMMemberHelper();
        private MDMFunctionHelper function = new MDMFunctionHelper();
        private List<MdmOperationMember> operations = new List<MdmOperationMember>();

        /// <summary>
        /// Adds a Member in the Calculated Member
        /// </summary>
        /// <param name="memberdimension">The dimension Name of the member</param>
        /// <param name="memberdepth">The dimension depth of the member</param>
        /// <param name="membername">The name of the member</param>
        /// <returns>The MDM Member Helper</returns>
        public MDMMemberHelper addMember(string memberdimension, int memberdepth, string membername)
        {
            member.MemberDimension = memberdimension;
            member.MemberDepth = memberdepth;
            member.MemberName = membername;
            this.Member = (MDMMember)this.member;
            return member;
        }

        /// <summary>
        /// Adds a Member in the tuple
        /// </summary>
        /// <param name="member">The member to add to the Calculated Member</param>
        /// <returns>The MDM Member Helper</returns>
        public MDMMemberHelper addMember(MDMMemberHelper member)
        {
            this.member = member;
            this.Member = (MDMMember)this.member;
            return member;
        }

        /// <summary>
        /// Retrieves a member 
        /// </summary>
        /// <returns>The member</returns>
        public MDMMemberHelper getMember()
        {
            return this.member;
        }

        /// <summary>
        /// add a function  to the calculated member
        /// </summary>
        /// <returns>the added function</returns>
        public MDMFunctionHelper addFunction()
        {
            this.function = new MDMFunctionHelper();
            this.Function = this.function;
            return this.function;
        }

        /// <summary>
        /// add a function set to the calculated member
        /// </summary>
        /// <param name="function">The function to add to the calculated member</param>
        /// <returns>the added function</returns>
        public MDMFunctionHelper addFunction(MDMFunctionHelper function)
        {
            this.function = function;
            this.Function = this.function;
            return this.function;
        }

        /// <summary>
        /// Adds an operation token
        /// </summary>
        /// <param name="operationToken">The operation token to add</param>
        /// <returns>The added tokens member</returns>
        public MdmOperationMember[] addOperationToken(MDMOperationTokenHelper operationToken)
        {
            MdmOperationMember tmpOpMember = new MdmOperationMember();
            tmpOpMember.Member = operationToken;
            this.operations.Add(tmpOpMember);
            foreach (MdmOperationMember thismember in this.operations)
                this.Operations.Add(thismember.Member.ToString());
            return this.operations.ToArray();
        }

        /// <summary>
        /// Adds an operation member
        /// </summary>
        /// <param name="operationMember">The member to add</param>
        /// <returns>The Operation Member array</returns>
        public MdmOperationMember[] addOperationMember(MDMMemberHelper operationMember)
        {
            MdmOperationMember tmpOpMember = new MdmOperationMember();
            tmpOpMember.Member = operationMember;
            this.operations.Add(tmpOpMember);
            foreach (MdmOperationMember thismember in this.operations)
                this.Operations.Add(thismember.Member.ToString());
            return this.operations.ToArray();
        }
    }
}
