/*
 * This source file (MDMQueryAxisHelper.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Data;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.Structures.XMLA;

namespace Bamboo.Cubed.Engine.QueryModel
{
    /// <summary>
    /// An extended QueryAxis with methods for adding sets and set functions.
    /// </summary>
    public class MDMQueryAxisHelper : MDMQueryAxis
    {
        private MDMSetHelper set;
        private MDMFunctionHelper setfunction;

        /// <summary>
        /// Initialises an empty default Query Axis Helper
        /// </summary>
        public MDMQueryAxisHelper()
        {
        }
        
        /// <summary>
        /// Initialises a new query axis helper
        /// </summary>
        /// <param name="name">The name of the helper</param>
        /// <param name="type">The type of the helper</param>
        public MDMQueryAxisHelper(string name, string type)
        {
            this.AxisName = name;
            switch (type)
            {
                case "ROWS":
                    {
                        this.AxisType = MDMQueryAxisType.ROWS;
                        break;
                    }
                case "COLUMNS":
                    {
                        this.AxisType = MDMQueryAxisType.COLUMNS;
                        break;
                    }
                case "PAGES":
                    {
                        this.AxisType = MDMQueryAxisType.PAGES;
                        break;
                    }
                case "CHAPTERS":
                    {
                        this.AxisType = MDMQueryAxisType.CHAPTERS;
                        break;
                    }
                default:
                    {
                        this.AxisType = MDMQueryAxisType.ROWS;
                        break;
                    }
            }
        }        

        /// <summary>
        /// gets the set of the axis
        /// </summary>
        /// <returns>the set of the axis</returns>
        public MDMSetHelper getSet()
        {
            return this.set;
        }

        /// <summary>
        /// Adds a set to the axis
        /// </summary>
        /// <returns>the added set</returns>
        public MDMSetHelper addSet()
        {
            this.set = new MDMSetHelper();
            this.Set = (MDMSet)this.set;
            return this.set;
        }

        /// <summary>
        /// Adds a set to the axis
        /// </summary>
        /// <param name="set">The set to add</param>
        /// <returns>the added set</returns>
        public MDMSetHelper addSet(MDMSetHelper set)
        {
            this.set = set;
            this.Set = (MDMSet)this.set;
            return this.set;
        }

        /// <summary>
        /// add a function set to the axis
        /// </summary>
        /// <returns>the added function</returns>
        public MDMFunctionHelper addFunction()
        {
            this.setfunction = new MDMFunctionHelper();
            this.SetFunction = this.setfunction;
            return this.setfunction;
        }

        /// <summary>
        /// add a function set to the axis
        /// </summary>
        /// <param name="function">The function to add to the axis</param>
        /// <returns>the added function</returns>
        public MDMFunctionHelper addFunction(MDMFunctionHelper function)
        {
            this.setfunction = function;
            this.SetFunction = this.setfunction;
            return this.setfunction;
        }
    }
}
