/*
 * This source file (StatementBuilder.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Data;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.Structures.XMLA;

namespace Bamboo.Cubed.Engine.QueryModel
{
    /// <summary>
    /// A helper class to build statements from scratch or from XML files
    /// </summary>
    /// <remarks>
    /// The StatementBuilder class is used for construct queries in an straight foward way.
    /// It also provides a mechanism for loading parameters into queries
    /// </remarks>
    public class StatementBuilder : MDMStatement
    {
        ///// <summary>
        ///// Returns a string when given a parser
        ///// </summary>
        ///// <param name="statement"></param>
        ///// <param name="parser"></param>
        ///// <returns></returns>
        //public string Unbuild(Structures.MDMStatement statement, Interfaces.IParser parser)
        //{
        //    if (statement != null)
        //    {
        //        if (statement.StatementType == Structures.MDMStatementTypes.SELECT)
        //        {
        //            QueryBuilder b = new QueryBuilder();
        //            return b.Unbuild((Structures.MDMQuery)statement, parser);
        //        }
        //        if (statement.StatementType == Structures.MDMStatementTypes.UPDATE)
        //        {
        //            UpdateBuilder b = new UpdateBuilder();
        //            return b.Unbuild((Structures.MDMUpdate)statement, parser);
        //        }
        //    }
        //    return "";
        //}

        /// <summary>
        /// Stores the query as XML in a given location
        /// </summary>
        /// <param name="statement">The MDMStatement object to serialize</param>
        /// <param name="location">The path to the result XML</param>
        public void SerializeStatement(MDMQuery statement, string location)
        {
            if (statement.StatementType == MDMStatementTypes.SELECT)
            {
                QueryBuilder b = new QueryBuilder();
                b.SerializeQuery((MDMQuery)statement, location);
            }
        }

        /// <summary>
        /// Serializes the given statement
        /// </summary>
        /// <param name="statement">The MDMStatement object to serialize</param>
        /// <returns>The serialized string</returns>
        public string SerializeStatement(MDMStatement statement)
        {
            if (statement != null)
            {
                if (statement.StatementType == MDMStatementTypes.SELECT)
                {
                    QueryBuilder b = new QueryBuilder();
                    return b.SerializeQuery((MDMQuery)statement);
                }
                if (statement.StatementType == MDMStatementTypes.UPDATE)
                {
                    UpdateBuilder b = new UpdateBuilder();
                    return b.SerializeUpdate((MDMUpdate)statement);
                }
            }
            return "";
        }
    }
}
