/*
 * This source file (TokenTypes.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Collections;
using System.Text;
using System.Data;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures.Query;
using Bamboo.Cubed.Structures.XMLA;

namespace Bamboo.Cubed.Engine.QueryModel
{
    /// <summary>
    /// Different type of tokens expected in query languages
    /// </summary>
    [XmlRoot("TokenTypes")]
    [Flags]
    public enum TokenTypes
    {
        /// <summary>
        /// A Keyword token
        /// </summary>
        [XmlEnum("Keyword")]
        KEYWORD = 1,
        /// <summary>
        /// A directive token
        /// </summary>
        [XmlEnum("Directive")]
        DIRECTIVE = 2,
        /// <summary>
        /// An expression token
        /// </summary>
        [XmlEnum("Expression")]
        EXPRESSION = 3,
        /// <summary>
        /// A function token
        /// </summary>
        [XmlEnum("Function")]
        FUNCTION = 4,
        /// <summary>
        /// A generic token
        /// </summary>
        [XmlEnum("Token")]
        TOKEN = 5,
        /// <summary>
        /// An operator token
        /// </summary>
        [XmlEnum("Operator")]
        OPERATOR = 6,
        /// <summary>
        /// An expected member token
        /// </summary>
        [XmlEnum("ExpMember")]
        EXPMEMBER = 7,
        /// <summary>
        ///  A statement token
        /// </summary>
        [XmlEnum("Statement")]
        STATEMENT = 8,
        /// <summary>
        /// A unary operator token
        /// </summary>
        [XmlEnum("UnaryOperator")]
        UNARYOPERATOR = 9,
        /// <summary>
        /// An enclosing start token
        /// </summary>
        [XmlEnum("EnclosingStart")]
        ENCLOSINGSTART = 10,
        /// <summary>
        /// An enclosing end token
        /// </summary>
        [XmlEnum("EnclosingEnd")]
        ENCLOSINGEND = 11
    }
}
