/*
 * This source file (ConsolidationCompletedEventArgs.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Collections;
using System.Collections.Specialized;
using Bamboo.Cubed;
using Bamboo.Cubed.Structures;

namespace Bamboo.Cubed.Engine
{
    /// <summary>
    /// This class contains event arguments resulting from a consolidating completed event
    /// </summary>
    public class ConsolidationCompletedEventArgs : EventArgs
    {
        /// <summary>
        /// This initialises a new set of event args for a completed consolidation
        /// </summary>
        /// <param name="cube">The cube which has been consolidated</param>
        /// <param name="finalcube">Whether this is the final cube in the consolidation set</param>
        /// <param name="exception">If any errors occured, this will store the exception</param>
        /// <param name="wascancelled">Indicates whether the consolidation was cancelled, or whether 
        /// it exited normally</param>
        /// <param name="cubesprocessed">Whether any cubes were processed, or - if false - it indicates
        /// that the cube did not exist or there were none to process</param>
        public ConsolidationCompletedEventArgs(Cube cube, bool finalcube,
            ConsolidationManagerException exception,  bool wascancelled, bool cubesprocessed)
        {
            _cube = cube;
            _finalcube = finalcube;
            _exception = exception;
            _wascancelled = wascancelled;
            _cubesprocessed = cubesprocessed;
        }

        private Cube _cube;
        /// <summary>
        /// The cube which has been consolidated
        /// </summary>
        public Cube Cube
        {
            get { return _cube; }
            set { _cube = value; }
        }

        private bool _finalcube;
        /// <summary>
        /// Whether this is the final cube in the consolidation to be completed (that is, there are no more 
        /// cubes to be consolidated as part of this consolidation
        /// </summary>
        public bool FinalCube
        {
            get { return _finalcube; }
            set { _finalcube = value; }
        }

        private ConsolidationManagerException _exception;
        /// <summary>
        /// If this consolidation failed, this contains the details of the exception
        /// </summary>
        public ConsolidationManagerException Exception
        {
            get { return _exception; }
            set { _exception = value; }
        }

        private bool _wascancelled;
        /// <summary>
        /// Indicates whether this thread was cancelled, or whether it exited normally
        /// </summary>
        public bool WasCancelled
        {
            get { return _wascancelled; }
            set { _wascancelled = value; }
        }

        private bool _cubesprocessed;
        /// <summary>
        /// Indicates whether there were any cubes process (true), or if none were processed - indicating that
        /// no cubes existed
        /// </summary>
        public bool CubesProcessed
        {
            get { return _cubesprocessed; }
            set { _cubesprocessed = value; }
        }
    }
}
