/*
 * This source file (MDMCalculatedMember.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace Bamboo.Cubed.Structures.Query
{
    /// <summary>
    /// A calculated member for a query 
    /// </summary>
    [Serializable]
    [XmlRoot("CalculatedMember")]
    public class MDMCalculatedMember
    {
        private int _solveorder;
        /// <summary>
        /// This is the order in which this calculated member is calculated, compared with other calculated 
        /// members on the same axis
        /// </summary>
        [XmlElement("SolveOrder")]
        public int SolveOrder
        {
            get { return _solveorder; }
            set { _solveorder = value; }
        }

        private MDMMember _member;
        /// <summary>
        /// The member which will result after the calculation
        /// </summary>
        [XmlElement("Member")]
        public MDMMember Member
        {
            get { return _member; }
            set { _member = value; }
        }

        private List<string> _operations;
        /// <summary>
        /// An Array of Operations. Each of these can either be the name of a member or a simple
        /// calculation token (+, -, / or *)
        /// </summary>
        [XmlArray("Operation")]
        public List<string> Operations
        {
            get 
            { 
                if (_operations == null) _operations = new List<string>();
                return _operations;
            }
            set { _operations = value; }
        }

        private MDMFunction _function;
        /// <summary>
        /// The function used to calculate the members value
        /// </summary>
        [XmlElement("Function")]
        public MDMFunction Function
        {
            get { return _function; }
            set { _function = value; }
        }
    }
}