/*
 * This source file (MDMFunction.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace Bamboo.Cubed.Structures.Query
{
    /// <summary>
    /// The representation of a function passed in the query. For example in MDX could be Descendants or Crossjoin
    /// </summary>
    [Serializable]
    [XmlRoot("MDMFunction")]
    public class MDMFunction
    {
        /// <summary>
        /// Initialies a default instance of MDMFunction
        /// </summary>
        public MDMFunction()
        {

            _parameters = new MDMFunctionParameter[0];
            _name = String.Empty;
            _returntypename = String.Empty;
        }
        
        /// <summary>
        /// Initialises an instance of MDMFunction
        /// </summary>
        /// <param name="functionname">The name of the function</param>
        /// <param name="parameters">An array of parameters that recieve the function</param>
        /// <param name="returntypename">The return type of the function</param>
        public MDMFunction(string functionname, MDMFunctionParameter[] parameters, string returntypename)
        {
            _parameters = parameters;
            _name = functionname;
            _returntypename = returntypename;
        }        

        private MDMFunctionParameter[] _parameters;
        /// <summary>
        /// The parameters that receive the function
        /// </summary>
        [XmlElement("Parameters")]
        public MDMFunctionParameter[] Parameters
        {
            get { return _parameters; }
            set { _parameters = value; }
        }

        private string _name;
        /// <summary>
        /// The name of the function
        /// </summary>
        [XmlAttribute("FunctionName")]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private string _returntypename;
        /// <summary>
        /// The return type of the function
        /// </summary>
        [XmlAttribute("ReturnTypeName")]
        public string ReturnTypeName
        {
            get { return _returntypename; }
            set { _returntypename = value; }
        }
    }
}