/*
 * This source file (MDMQuery.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace Bamboo.Cubed.Structures.Query
{
    /// <summary>
    /// The MDMQuery is the full result intended for the parser.
    /// It is constructed over the MDM (multidimensional) basic objects and represent a basic Query
    /// The query may be contructed over MDX, another query language or by instantiating directly
    /// </summary>
    [Serializable]
    [XmlRoot("MDMQuery")]
    public class MDMQuery : MDMStatement
    {
        private MDMNamedSet[] namedsets;
        /// <summary>
        /// The Named Sets defined for the query
        /// </summary>
        [XmlElement("NamedSets")]
        public MDMNamedSet[] NamedSets
        {
            get
            {
                return this.namedsets;
            }
            set
            {
                this.namedsets = value;
            }
        }

        private MDMCalculatedMember[] calculatedmembers;
        /// <summary>
        /// The calculated members for the query.
        /// </summary>
        [XmlElement("CalculatedMember")]
        public MDMCalculatedMember[] CalculatedMembers
        {
            get
            {
                return this.calculatedmembers;
            }
            set
            {
                this.calculatedmembers = value;
            }
        }

        private List<MDMQueryAxis> axes;
        /// <summary>
        /// The Axes that compose the query except the slicer
        /// </summary>
        [XmlElement("Axis")]
        public List<MDMQueryAxis> Axes
        {
            set { this.axes = value; }
            get { return this.axes;  }
        }

        private MDMQueryAxis slieceraxis;
        /// <summary>
        /// The Slicer Axis
        /// </summary>
        [XmlElement("Slicer")]
        public MDMQueryAxis SlicerAxis
        {
            set
            {
                this.slieceraxis = value;
            }
            get
            {
                return this.slieceraxis;
            }
        }

        private bool _transactional;
        /// <summary>
        /// Whether this is a transactional query or a consolidated query
        /// </summary>
        [XmlElement("Transactional")]
        public bool Transactional
        {
            get { return _transactional; }
            set { _transactional = value; }
        }
    }
}