/*
 * This source file (Axis.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Contains a single Axis
    /// </summary>
    [Serializable]
    [DataContract]
    public class Axis
    {
        private string _name;
        /// <summary>
        /// The name of the Axis
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        private List<CrossProductType> _crossproducts;
        /// <summary>
        /// Contains a collection of Cross Product Types
        /// </summary>
        [DataMember]
        public List<CrossProductType> CrossProducts
        {
            get
            {
                if (_crossproducts == null) _crossproducts = new List<CrossProductType>();
                return _crossproducts;
            }
            set { _crossproducts = value; }
        }

        private List<TupleType> _tuples;
        /// <summary>
        /// Contains the collection of tuples
        /// </summary>
        [DataMember]
        public List<TupleType> Tuples
        {
            get
            {
                if (_tuples == null) _tuples = new List<TupleType>();
                return _tuples;
            }
            set { _tuples = value; }
        }

        private MembersType _members;
        /// <summary>
        /// Contains a collection of members
        /// </summary>
        [DataMember]
        public MembersType Members
        {
            get
            {
                if (_members == null) _members = new MembersType();
                return _members;
            }
            set { _members = value; }
        }

        private FunctionType _function;
        /// <summary>
        /// Contains the function used to provide the value of the Axis
        /// </summary>
        [DataMember]
        public FunctionType Function
        {
            get
            {
                if (_function == null) _function = new FunctionType();
                return _function;
            }
            set { _function = value; }
        }

        private List<MeasureType> _measures;
        /// <summary>
        /// Contains the measures of this Axis, if this is a Measure axis
        /// </summary>
        [DataMember]
        public List<MeasureType> Measures
        {
            get
            {
                if (_measures == null) _measures = new List<MeasureType>();
                return _measures;
            }
            set { _measures = value; }
        }
    }
}
