﻿/*
 * This source file (AxisCellHelper.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed™
 * Trademarked & Copyright © 2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright © their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. “Commercial” purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// This class helps to create cell indices by combining the current members of multiple axes
    /// </summary>
    public class AxisCellHelper
    {
        /// <summary>
        /// This prepares the axis cell help and provides a space for each axis
        /// </summary>
        /// <param name="axiscount">The number of axes represented</param>
        public AxisCellHelper(int axiscount)
        {
            _axismembers = new Dictionary<int, int>();
            for (int i = 0; i < axiscount; i++)
                _axismembers.Add(i, 0);
        }

        /// <summary>
        /// This private collection contains (up to) one selected member for each axis
        /// </summary>
        private Dictionary<int, int> _axismembers;

        /// <summary>
        /// This returns the current set of axis selected member indicies
        /// </summary>
        public int[] CellIndices
        {
            get
            {
                // Create a temporary index collection
                List<int> cellindices = new List<int>();

                // Cycle through from 0 to the end of the axis collection and add the selected members
                // in order
                int currentaxis = 0;
                while (_axismembers.ContainsKey(currentaxis))
                {
                    // Add the current axes selected member to the cell indices collection
                    cellindices.Add(_axismembers[currentaxis]);

                    // Move to the next axis (if it exists)
                    currentaxis++;
                }

                // Now return the collection
                return cellindices.ToArray();
            }
        }

        /// <summary>
        /// Updates the collection to ensure that the given member is selected for the current axis
        /// </summary>
        /// <param name="axis">The axis to update</param>
        /// <param name="member">The index of the selected member</param>
        public void AddSelectedMember(int axis, int member)
        {
            if (_axismembers.ContainsKey(axis))
                _axismembers[axis] = member;
        }
    }
}
