/*
 * This source file (CellInfo.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Contains the details of a single cell
    /// </summary>
    [Serializable]
    [DataContract]
    public class CellInfo
    {
        private double _value;
        /// <summary>
        /// The value of the cell
        /// </summary>
        [DataMember]
        public double Value
        {
            get {return _value; }            
            set { _value = value; }
        }

        /// <summary>
        /// The formatted value of the cell
        /// </summary>
        [DataMember]
        public string FmtValue
        {
            get { return _value.ToString(FormatString); }
        }

        private string _backcolor;
        /// <summary>
        /// The HTML colour code of the back colour of the cell
        /// </summary>
        [DataMember]
        public string BackColor
        {
            get { return _backcolor; }
            set { _backcolor = value; }
        }

        private string _forecolor;
        /// <summary>
        /// The HTML colour code of the foreground colour of the cell
        /// </summary>
        [DataMember]
        public string ForeColor
        {
            get { return _forecolor; }
            set { _forecolor = value; }
        }

        private string _fontname;
        /// <summary>
        /// The name of the font
        /// </summary>
        [DataMember]
        public string FontName
        {
            get { return _fontname; }
            set { _fontname = value; }
        }

        private double _fontsize;
        /// <summary>
        /// The size of the font
        /// </summary>
        [DataMember]
        public double FontSize
        {
            get {return _fontsize; }
            set { _fontsize = value; }
        }

        private string _fontflags;
        /// <summary>
        /// A string containing font flags, formated as HTML style commands
        /// </summary>
        [DataMember]
        public string FontFlags
        {
            get { return _fontflags; }
            set { _fontflags = value; }
        }

        private string _formatstring;
        /// <summary>
        /// The format string
        /// </summary>
        [DataMember]
        public string FormatString
        {
            get
            {
                if (_formatstring == String.Empty || _formatstring == null)
                    _formatstring = "F2";
                return _formatstring;
            }
            set { _formatstring = value; }
        }

        private string _nonemptybehavior;
        /// <summary>
        /// What to do with non empty cells
        /// </summary>
        [DataMember]
        public string NonEmptyBehavior
        {
            get { return _nonemptybehavior; }
            set { _nonemptybehavior = value; }
        }

        private int _solveorder;
        /// <summary>
        /// The solve order for the cell
        /// </summary>
        [DataMember]
        public int SolveOrder
        {
            get { return _solveorder; }
            set { _solveorder = value; }
        }

        private bool _updateable;
        /// <summary>
        /// Whether the cell is updateable
        /// </summary>
        [DataMember]
        public bool Updateable
        {
            get { return _updateable; }
            set { _updateable = value; }
        }

        private bool _visible;
        /// <summary>
        /// Whether the cell is visible
        /// </summary>
        [DataMember]
        public bool Visible
        {
            get { return _visible; }
            set { _visible = value; }
        }

        private string _expression;
        /// <summary>
        /// The expression of the cell
        /// </summary>
        [DataMember]
        public string Expression
        {
            get { return _expression; }
            set { _expression = value; }
        }
    }
}
