/*
 * This source file (MemberType.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Represents a type of Member
    /// </summary>
    [Serializable]
    [DataContract]
    public class MemberType
    {
        private string _dimension;
        /// <summary>
        /// The Dimension the member belongs to
        /// </summary>
        [DataMember]
        public string Dimension
        {
            get { return _dimension; }
            set { _dimension = value; }
        }

        /// <summary>
        /// The Hierarchy the member belongs to (which is the same as the dimension in this single dimensional
        /// model)
        /// </summary>
        [DataMember]
        public string Hierarchy
        {
            get { return _dimension; }
            set { _dimension = value; }
        }

        private Guid _uname;
        /// <summary>
        /// The unique name of the member
        /// </summary>
        [DataMember]
        public Guid UName
        {
            get { return _uname; }
            set { _uname = value; }
        }

        private string _caption;
        /// <summary>
        /// The member caption
        /// </summary>
        [DataMember]
        public string Caption
        {
            get { return _caption; }
            set { _caption = value; }
        }

        private string _lname;
        /// <summary>
        /// The name of the level the member belongs to
        /// </summary>
        [DataMember]
        public string LName
        {
            get { return _lname; }
            set { _lname = value; }
        }

        private uint _lnum;
        /// <summary>
        /// The number of the level depth that the member belongs to
        /// </summary>
        [DataMember]
        public uint LNum
        {
            get { return _lnum; }
            set { _lnum = value; _lnumspecified = true; }
        }

        private bool _lnumspecified;
        /// <summary>
        /// Whether a level has been specified for the member
        /// </summary>
        [DataMember]
        public bool LNumSpecified
        {
            get { return _lnumspecified; }
            set { _lnumspecified = value; }
        }

        private uint _displayinfo;
        /// <summary>
        /// The Display Info
        /// </summary>
        [DataMember]
        public uint DisplayInfo
        {
            get { return _displayinfo; }
            set { _displayinfo = value; _displayinfospecified = true; }
        }

        private bool _displayinfospecified;
        /// <summary>
        /// Whether Display info has been specified
        /// </summary>
        [DataMember]
        public bool DisplayInfoSpecified
        {
            get { return _displayinfospecified; }
            set { _displayinfospecified = value; }
        }

        private int _solveorder;
        /// <summary>
        /// This is the order in which this calculated member is calculated, compared with other calculated 
        /// members on the same axis
        /// </summary>
        public int SolveOrder
        {
            get { return _solveorder; }
            set { _solveorder = value; }
        }

        private List<string> _operations;
        /// <summary>
        /// An Array of Operations. Each of these can either be the name of a member or a simple
        /// calculation token (+, -, / or *)
        /// (If this is a calcualted member)
        /// </summary>
        public List<string> Operations
        {
            get
            {
                if (_operations == null) _operations = new List<string>();
                return _operations;
            }
            set { _operations = value; }
        }

        private FunctionType _function;
        /// <summary>
        /// The function used to calculate the members value (if this is a calculated member)
        /// </summary>
        public FunctionType Function
        {
            get { return _function; }
            set { _function = value; }
        }
    }
}
