/*
 * This source file (RequestTypes.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Determines the returned rowset from an XMLA Discover method
    /// </summary>
    [Serializable]
    [DataContract]
    public enum RequestTypes
    {
        /// <summary>
        /// Returns the Discover Datasources rowset
        /// </summary>
        [EnumMember]
        DISCOVER_DATASOURCES = 1,
        /// <summary>
        /// Returns the Discover Properties rowset
        /// </summary>
        [EnumMember]
        DISCOVER_PROPERTIES = 2,
        /// <summary>
        /// Returns the Discover Schema Rowsets rowset
        /// </summary>
        [EnumMember]
        DISCOVER_SCHEMA_ROWSETS = 3,
        /// <summary>
        /// Returns the Discover Enumerators rowset
        /// </summary>
        [EnumMember]
        DISCOVER_ENUMERATORS = 4,
        /// <summary>
        /// Returns the Discover Keywords rowset
        /// </summary>
        [EnumMember]
        DISCOVER_KEYWORDS = 5,
        /// <summary>
        /// Returns the Discover Literals rowset
        /// </summary>
        [EnumMember]
        DISCOVER_LITERALS = 6,
        /// <summary>
        /// Returns the DBSchema Catalogs rowset
        /// </summary>
        [EnumMember]
        DBSCHEMA_CATALOGS = 7,
        /// <summary>
        /// Returns the MDSchema Actions rowset
        /// </summary>
        [EnumMember]
        MDSCHEMA_ACTIONS = 8,
        /// <summary>
        /// Returns teh MDSchema Cubes rowset
        /// </summary>
        [EnumMember]
        MDSCHEMA_CUBES = 9,
        /// <summary>
        /// Returns the MDSchema Dimensions rowset
        /// </summary>
        [EnumMember]
        MDSCHEMA_DIMENSIONS = 10,
        /// <summary>
        /// Returns a list of the MDX and external functions available
        /// </summary>
        [EnumMember]
        MDSCHEMA_FUNCTIONS = 11,
        /// <summary>
        /// Returns a list of the hierarchies in the cube
        /// </summary>
        [EnumMember]
        MDSCHEMA_HIERARCHIES = 12,
        /// <summary>
        /// Returns a list of the measures available in the selected cube
        /// </summary>
        [EnumMember]
        MDSCHEMA_MEASURES = 13,
        /// <summary>
        /// Returns a list of the members available in the selected cube
        /// </summary>
        [EnumMember]
        MDSCHEMA_MEMBERS = 14,
        /// <summary>
        /// Returns a list of the properties on the server
        /// </summary>
        [EnumMember]
        MDSCHEMA_PROPERTIES = 15,
        /// <summary>
        /// Returns a list of the sets available in the selected cube
        /// </summary>
        [EnumMember]
        MDSCHEMA_SETS = 16,
        /// <summary>
        /// Returns a list of the KPIs available in the selected cube
        /// </summary>
        [EnumMember]
        MDSCHEMA_KPIS = 17
    }
}
