/*
 * This source file (RestrictionList.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Allows for the setting of XMLA restrictions on an Execute or Discover method
    /// </summary>
    [Serializable]
    [DataContract]
    public class RestrictionList
    {
        /// <summary>
        /// Initialises a new default instance of the RestrictionList
        /// </summary>
        public RestrictionList()
        {
            datasourcename = String.Empty;
            datasourcedescription = String.Empty;
            url = String.Empty;
            catalogname = String.Empty;
            providername = String.Empty;
            providertype = new List<ProviderTypes>();
            authenticationmode = AuthenticationModeTypes.Authenticated;
            propertyname = new List<string>();
            schemaname = new List<string>();
            enumname = new List<string>();
            keyword = new List<string>();
            literalname = new List<string>();
        }

        private string datasourcename;
        /// <summary>
        /// The datasource name
        /// </summary>
        [DataMember]
        public string DataSourceName
        {
            get { return datasourcename; }
            set { datasourcename = value; }
        }

        private string datasourcedescription;
        /// <summary>
        /// The datasource description
        /// </summary>
        [DataMember]
        public string DataSourceDescription
        {
            get { return datasourcedescription; }
            set { datasourcedescription = value; }
        }

        private string catalogname;
        /// <summary>
        /// The name of a catalog in the system
        /// </summary>
        [DataMember]
        public string CatalogName
        {
            get { return catalogname; }
            set { catalogname = value; }
        }

        private string url;
        /// <summary>
        /// The URL of the datasource
        /// </summary>
        [DataMember]
        public string URL
        {
            get { return url; }
            set { url = value; }
        }

        private string providername;
        /// <summary>
        /// The Provider Name of the datasource
        /// </summary>
        [DataMember]
        public string ProviderName
        {
            get { return providername; }
            set { providername = value; }
        }

        private List<ProviderTypes> providertype;
        /// <summary>
        /// Returns the support data types for this provider
        /// </summary>
        [DataMember]
        public List<ProviderTypes> ProviderType
        {
            get { return providertype; }
            set { providertype = value; }
        }

        private AuthenticationModeTypes authenticationmode;
        /// <summary>
        /// The type of Authentication mode required by this provider
        /// </summary>
        [DataMember]
        public AuthenticationModeTypes AuthenticationMode
        {
            get { return authenticationmode; }
            set { authenticationmode = value; }
        }

        private List<string> propertyname;
        /// <summary>
        /// The Property Name(s)
        /// </summary>
        [DataMember]
        public List<string> PropertyName
        {
            get { return propertyname; }
            set { propertyname = value; }
        }

        private List<string> schemaname;
        /// <summary>
        /// The name(s) of the Schema
        /// </summary>
        [DataMember]
        public List<string> SchemaName
        {
            get { return schemaname; }
            set { schemaname = value; }
        }

        private List<string> enumname;
        /// <summary>
        /// The name(s) of the enumeration
        /// </summary>
        [DataMember]
        public List<string> EnumName
        {
            get { return enumname; }
            set { enumname = value; }
        }

        private List<string> keyword;
        /// <summary>
        /// The keyword(s) which are reserved
        /// </summary>
        [DataMember]
        public List<string> KeyWord
        {
            get { return keyword; }
            set { keyword = value; }
        }

        private List<string> literalname;
        /// <summary>
        /// The name(s) of the literal
        /// </summary>
        [DataMember]
        public List<string> LiteralName
        {
            get { return literalname; }
            set { literalname = value; }
        }
    }
}
