/*
 * This source file (TupleType.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;
using System.Security.Cryptography;
using System.IO;
using System.Reflection;
using Bamboo.Cubed.Utilities;
using System.Xml.Schema;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures.XMLA
{
    /// <summary>
    /// Represents a Typle Type collection of members
    /// </summary>
    [Serializable]
    [DataContract]
    public class TupleType
    {
        /// <summary>
        /// Initialises a new Tuple Type
        /// </summary>
        public TupleType()
        {
            _members = new MembersType();
        }

        /// <summary>
        /// Returns the collection Enumerator
        /// </summary>
        /// <returns>The enumerator</returns>
        public IEnumerator GetEnumerator()
        {
            return Members.GetEnumerator();
        }

        /// <summary>
        /// Adds a new member
        /// </summary>
        /// <param name="member">The member to add</param>
        public void Add(MemberType member)
        {
            Members.Add(member);
        }

        /// <summary>
        /// Returns the member stored at the specified index
        /// </summary>
        /// <param name="index">The index to retrieve</param>
        /// <returns>The member</returns>
        public MemberType this[int index]
        {
            get { return Members[index]; }
        }

        /// <summary>
        /// Returns how many members are in the Tuple
        /// </summary>
        public int Count
        {
            get { return Members.Count; }
        }

        /// <summary>
        /// Clears the Members
        /// </summary>
        public void Clear()
        {
            Members.Clear();
        }

        /// <summary>
        /// Removes the member specified by the index
        /// </summary>
        /// <param name="index">The index to remove</param>
        /// <returns>The removed member</returns>
        public MemberType Remove(int index)
        {
            MemberType obj = Members[index];
            Members.Remove(obj);
            return obj;
        }

        /// <summary>
        /// Removes the member specified
        /// </summary>
        /// <param name="membertype">The removed member</param>
        public void Remove(MemberType membertype)
        {
            Members.Remove(membertype);
        }       
        
        private MembersType _members;
        /// <summary>
        /// The collection of members for the Tuple
        /// </summary>
        [DataMember]
        public MembersType Members
        {
            get
            {
                if (_members == null) _members = new MembersType();
                return _members;
            }
            set { _members = value; }
        }
    }
}
