/*
 * This source file (AttributeValue.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is the class used for storing values against attributes for a member
    /// </summary>
    [Serializable]
    [DataContract]
    public class AttributeValue
    {
        /// <summary>
        /// Initialises a new default instance of an attribute value
        /// </summary>
        public AttributeValue()
        {
            _id = Guid.NewGuid();
            _attribute = new Guid();
            _values = new List<string>();
        }

        /// <summary>
        /// Initialises a new attribute value for the passed in attribute with the specified values
        /// </summary>
        /// <param name="attribute">The attribute this value corresponds to</param>
        /// <param name="values">The values for this attribute for this member</param>
        public AttributeValue(Attribute attribute, List<string> values)
        {
            _id = Guid.NewGuid();
            _attribute = attribute.ID;
            _values = values;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of the attribute value
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private Guid _attribute;
        /// <summary>
        /// The attribute this value is a value of
        /// </summary>
        [DataMember]
        public Guid Attribute
        {
            get { return _attribute; }
            set { _attribute = value; }
        }

        private List<string> _values;
        /// <summary>
        /// The actual values of this attribute for this member
        /// </summary>
        [DataMember]
        public List<string> Values
        {
            get { return _values; }
            set { _values = value; }
        }
    }
}