/*
 * This source file (DataDefinitions.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This class contains the data definitions which are used for persisting data back to the storage database
    /// </summary>
    public static class DataDefinitions
    {
        /// <summary>
        /// Cube Field Names
        /// </summary>        
        public static string[] CubeFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("description");
                fieldnames.Add("consolidationinterval");
                fieldnames.Add("lastconsolidation");
                fieldnames.Add("consolidatinguser");
                fieldnames.Add("consolidationresult");
                fieldnames.Add("nextconsolidation");
                fieldnames.Add("startconsolidation");
                fieldnames.Add("consolidationstarted");
                fieldnames.Add("consolidationinprogress");
                fieldnames.Add("consolidationnodesprocessed");
                fieldnames.Add("notificationusers");
                fieldnames.Add("pointconsolidation");
                fieldnames.Add("grantreadroles");
                fieldnames.Add("grantwriteroles");
                fieldnames.Add("storagemode");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Cube Field Types
        /// </summary>
        public static DataCreationTypes[] CubeFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.DateTimeField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.DateTimeField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.DateTimeField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Datasource Field Names
        /// </summary>
        public static string[] DatasourceFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("cubeid");
                fieldnames.Add("description");
                fieldnames.Add("providerid");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Datasource Field Types
        /// </summary>
        public static DataCreationTypes[] DatasourceFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Data Provider Field Names
        /// </summary>
        public static string[] DataProviderFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("connectorid");
                fieldnames.Add("connectorconfiguration");
                fieldnames.Add("connectorversion");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Data Provider Field Types
        /// </summary>
        public static DataCreationTypes[] DataProviderFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.ValueField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Dimension Field Names
        /// </summary>
        public static string[] DimensionFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("cubeid");
                fieldnames.Add("name");
                fieldnames.Add("description");
                fieldnames.Add("enabled");
                fieldnames.Add("membertype");
                fieldnames.Add("datasourceid");
                fieldnames.Add("addtotalmember");
                fieldnames.Add("istime");
                fieldnames.Add("timeperiod");
                fieldnames.Add("autoorder");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Dimension Field Types
        /// </summary>
        public static DataCreationTypes[] DimensionFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Dimension Table Indexes
        /// </summary>
        public static DataIndexDefinition[] DimensionIndexes
        {
            get
            {
                List<DataIndexDefinition> indexes = new List<DataIndexDefinition>();
                indexes.Add(new DataIndexDefinition(false, false, "cubeid"));
                return indexes.ToArray();
            }
        }

        /// <summary>
        /// Attribute Field Names
        /// </summary>
        public static string[] AttributeFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("dimensionid");
                fieldnames.Add("valuetype");
                fieldnames.Add("values");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Attribute Field Types
        /// </summary>
        public static DataCreationTypes[] AttributeFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// KPI Field Names
        /// </summary>
        public static string[] KPIFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("dimensionid");
                fieldnames.Add("measureid");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// KPI Field Types
        /// </summary>
        public static DataCreationTypes[] KPIFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                return fieldtypes.ToArray();
            }
        }
        
        /// <summary>
        /// Member Generator Field Names
        /// </summary>
        public static string[] MemberGeneratorFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("dimensionid");
                fieldnames.Add("rangegeneratorid");
                fieldnames.Add("overwrite");
                fieldnames.Add("aggregation");
                fieldnames.Add("signage");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Member Generator Field Types
        /// </summary>
        public static DataCreationTypes[] MemberGeneratorFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Range Generator Field Names
        /// </summary>
        public static string[] RangeGeneratorFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("rangetype");
                fieldnames.Add("membergeneratorid");
                fieldnames.Add("table");
                fieldnames.Add("namefield");
                fieldnames.Add("descriptionfield");
                fieldnames.Add("datelowerrange");
                fieldnames.Add("dateupperrange");
                fieldnames.Add("yearstartmonth");
                fieldnames.Add("numberlowerrange");
                fieldnames.Add("numberupperrange");
                fieldnames.Add("numberseed");
                fieldnames.Add("numberformatstring");
                fieldnames.Add("dateformatstring");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Range Generator Field Types
        /// </summary>
        public static DataCreationTypes[] RangeGeneratorFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.DateTimeField);
                fieldtypes.Add(DataCreationTypes.DateTimeField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.ValueField);
                fieldtypes.Add(DataCreationTypes.ValueField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Member Table Field Names
        /// </summary>
        public static string[] MemberTableFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("table");
                fieldnames.Add("membergeneratorid");
                fieldnames.Add("primarykey");
                fieldnames.Add("name");
                fieldnames.Add("description");
                fieldnames.Add("relatedtable");
                fieldnames.Add("relatedkey");
                fieldnames.Add("filterid");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Member Table Field Types
        /// </summary>
        public static DataCreationTypes[] MemberTableFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Member Table Parent Field Names
        /// </summary>
        public static string[] MemberTableParentFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("membertableid");
                fieldnames.Add("parenttable");
                fieldnames.Add("memberidfield");
                fieldnames.Add("parentidfield");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Member Table Parent Field Types
        /// </summary>
        public static DataCreationTypes[] MemberTableParentFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Attribute Value Field Field Names
        /// </summary>
        public static string[] AttributeValueFieldFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("attributeid");
                fieldnames.Add("membertableid");
                fieldnames.Add("fieldname");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Attribute Value Field Field Types
        /// </summary>
        public static DataCreationTypes[] AttributeValueFieldFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Role Field Names
        /// </summary>
        public static string[] RoleFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("enabled");
                fieldnames.Add("privatesearchrights");
                fieldnames.Add("publicsearchrights");
                fieldnames.Add("members");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Role Field Types
        /// </summary>
        public static DataCreationTypes[] RoleFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// User Field Names
        /// </summary>
        public static string[] UserFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("domain");
                fieldnames.Add("username");
                fieldnames.Add("fullname");
                fieldnames.Add("email");
                fieldnames.Add("mobile");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// User Field Types
        /// </summary>
        public static DataCreationTypes[] UserFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Descendants Field Names
        /// </summary>
        public static string[] DescendantsFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("descendantid");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Descendants Field Types
        /// </summary>
        public static DataCreationTypes[] DescendantsFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Member Field Names
        /// </summary>
        public static string[] MemberFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("description");
                fieldnames.Add("dimensionid");
                fieldnames.Add("filterid");
                fieldnames.Add("table");
                fieldnames.Add("primarykey");
                fieldnames.Add("parentid");
                fieldnames.Add("level");
                fieldnames.Add("rollup");
                fieldnames.Add("signage");
                fieldnames.Add("aggregation");
                fieldnames.Add("consolidation");
                fieldnames.Add("calculationid");
                fieldnames.Add("discovered");
                fieldnames.Add("grantreadroles");
                fieldnames.Add("grantwriteroles");
                fieldnames.Add("denyreadroles");
                fieldnames.Add("denywriteroles");
                fieldnames.Add("iscurrent");
                fieldnames.Add("displayorder");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Member Field Types
        /// </summary>
        public static DataCreationTypes[] MemberFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.DateTimeField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Member Table Indexes
        /// </summary>
        public static DataIndexDefinition[] MemberIndexes
        {
            get
            {
                List<DataIndexDefinition> indexes = new List<DataIndexDefinition>();
                indexes.Add(new DataIndexDefinition(false, false, "parentid"));
                indexes.Add(new DataIndexDefinition(false, false, "dimensionid"));
                return indexes.ToArray();
            }
        }

        /// <summary>
        /// Value Filter Field Names
        /// </summary>
        public static string[] ValueFilterFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("fieldname");
                fieldnames.Add("filtertype");
                fieldnames.Add("acceptablevalues");
                fieldnames.Add("lowervalue");
                fieldnames.Add("uppervalue");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Value Filter Field Types
        /// </summary>
        public static DataCreationTypes[] ValueFilterFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Attribute Value Field Names
        /// </summary>
        public static string[] AttributeValueFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("attributeid");
                fieldnames.Add("memberid");
                fieldnames.Add("values");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Attribute Value Field Types
        /// </summary>
        public static DataCreationTypes[] AttributeValueFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Formula Evaluator Field Names
        /// </summary>
        public static string[] FormulaEvaluatorFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("evaluatorid");
                fieldnames.Add("evaluatorconfiguration");
                fieldnames.Add("evaluatorversion");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Formula Evaluator Field Types
        /// </summary>
        public static DataCreationTypes[] FormulaEvaluatorFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.ValueField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// KPI Calculator Field Names
        /// </summary>
        public static string[] KPICalculatorFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("kpiid");
                fieldnames.Add("calculatorid");
                fieldnames.Add("calculatorconfiguration");
                fieldnames.Add("calculatorversion");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// KPI Calculator Field Types
        /// </summary>
        public static DataCreationTypes[] KPICalculatorFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.ValueField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Level Field Names
        /// </summary>
        public static string[] LevelFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("dimensionid");
                fieldnames.Add("name");
                fieldnames.Add("description");
                fieldnames.Add("depth");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Level Field Types
        /// </summary>
        public static DataCreationTypes[] LevelFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Level Table Indexes
        /// </summary>
        public static DataIndexDefinition[] LevelIndexes
        {
            get
            {
                List<DataIndexDefinition> indexes = new List<DataIndexDefinition>();
                indexes.Add(new DataIndexDefinition(false, false, "dimensionid"));
                return indexes.ToArray();
            }
        }

        /// <summary>
        /// Measure Link Field Names
        /// </summary>
        public static string[] MeasureLinkFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("dimensionid");
                fieldnames.Add("measureid");
                fieldnames.Add("filterfieldcolumn");
                fieldnames.Add("measuretablememberidcolumn");
                fieldnames.Add("linkingtable");
                fieldnames.Add("linkingtablemeasurelinkidcolumn");
                fieldnames.Add("linkingtablememberidcolumn");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Measure Link Field Types
        /// </summary>
        public static DataCreationTypes[] MeasureLinkFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Measure Field Names
        /// </summary>
        public static string[] MeasureFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("type");
                fieldnames.Add("cubeid");
                fieldnames.Add("datasourceid");
                fieldnames.Add("table");
                fieldnames.Add("measurevaluefield");
                fieldnames.Add("filterid");
                fieldnames.Add("consolidation");
                fieldnames.Add("aggregation");
                fieldnames.Add("calculatorid");
                fieldnames.Add("pointconsolidationfield");
                fieldnames.Add("transactionalfields");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Measure Field Types
        /// </summary>
        public static DataCreationTypes[] MeasureFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Measures Table Indexes
        /// </summary>
        public static DataIndexDefinition[] MeasureIndexes
        {
            get
            {
                List<DataIndexDefinition> indexes = new List<DataIndexDefinition>();
                indexes.Add(new DataIndexDefinition(false, false, "cubeid"));
                return indexes.ToArray();
            }
        }

        /// <summary>
        /// Desktop Search Field Names
        /// </summary>
        public static string[] DesktopSearchFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                fieldnames.Add("description");
                fieldnames.Add("keywords");
                fieldnames.Add("defaultresultid");
                fieldnames.Add("searchid");
                fieldnames.Add("searchversion");
                fieldnames.Add("searchdetails");
                fieldnames.Add("supportsweb");
                fieldnames.Add("supportswindows");
                fieldnames.Add("ispublic");
                fieldnames.Add("privateuser");
                fieldnames.Add("categoryid");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Desktop Search Field Types
        /// </summary>
        public static DataCreationTypes[] DesktopSearchFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.ValueField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Desktop Search Result Field Names
        /// </summary>
        public static string[] DesktopSearchResultFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("desktopsearchid");
                fieldnames.Add("resultid");
                fieldnames.Add("resultversion");
                fieldnames.Add("displaysettings");
                fieldnames.Add("supportsweb");
                fieldnames.Add("supportswindows");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Desktop Search Result Field Types
        /// </summary>
        public static DataCreationTypes[] DesktopSearchResultFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.ValueField);
                fieldtypes.Add(DataCreationTypes.LargeStringField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                fieldtypes.Add(DataCreationTypes.YesNoField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Desktop Search Index Field Names
        /// </summary>
        public static string[] DesktopSearchIndexFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("desktopsearchid");
                fieldnames.Add("keyword");
                fieldnames.Add("weighting");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Desktop Search Index Field Types
        /// </summary>
        public static DataCreationTypes[] DesktopSearchIndexFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Desktop Dashboard Field Names
        /// </summary>
        public static string[] DesktopDashboardFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("username");
                fieldnames.Add("description");
                fieldnames.Add("subscription");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Desktop Dashboard Field Types
        /// </summary>
        public static DataCreationTypes[] DesktopDashboardFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// Desktop Dashboard Item Field Names
        /// </summary>
        public static string[] DesktopDashboardItemFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("desktopdashboardid");
                fieldnames.Add("desktopsearchid");
                fieldnames.Add("desktopresultid");
                fieldnames.Add("column");
                fieldnames.Add("row");
                fieldnames.Add("width");
                fieldnames.Add("height");
                fieldnames.Add("refreshinterval");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Desktop Dashboard Item Field Types
        /// </summary>
        public static DataCreationTypes[] DesktopDashboardItemFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                fieldtypes.Add(DataCreationTypes.NumberField);
                return fieldtypes.ToArray();
            }
        }

        /// <summary>
        /// The parameter names for the GetMemberAncestor function
        /// </summary>
        public static string[] GetMemberAncestorParameterNames
        {
            get
            {
                List<string> parameternames = new List<string>();
                parameternames.Add("@member");
                parameternames.Add("@level");
                return parameternames.ToArray();
            }
        }

        /// <summary>
        /// The parameter types for the GetMemberAncestor function
        /// </summary>
        public static DataCreationTypes[] GetMemberAncestorParameterTypes
        {
            get
            {
                List<DataCreationTypes> parametertypes = new List<DataCreationTypes>();
                parametertypes.Add(DataCreationTypes.UniqueIdentifier);
                parametertypes.Add(DataCreationTypes.NumberField);
                return parametertypes.ToArray();
            }
        }

        /// <summary>
        /// Returns the main code block for the Get Member Ancestor function
        /// </summary>
        public static string GetMemberAncestorCode
        {
            get
            {
                string functioncode = @"
                    DECLARE @ascendant UNIQUEIDENTIFIER
	                SET @ascendant = @member

	                DECLARE @currentlevel INT 
	                SET @currentlevel =
		                (SELECT level FROM Members
		                WHERE id = @ascendant)

	                WHILE @currentlevel > @level
		            BEGIN
		            SET @ascendant =
			            (SELECT parentid FROM Members
            			 WHERE id = @ascendant)

		            SET @currentlevel =
			            (SELECT level FROM Members
			            WHERE id = @ascendant)	
		            END

	                RETURN @ascendant
                ";
                return functioncode;
            }
        }

        /// <summary>
        /// The parameter names for the GetMemberDescendants function
        /// </summary>
        public static string[] GetMemberDescendantsParameterNames
        {
            get
            {
                List<string> parameternames = new List<string>();
                parameternames.Add("@memberid");
                return parameternames.ToArray();
            }
        }

        /// <summary>
        /// The parameter types for the GetMemberDescendants function
        /// </summary>
        public static DataCreationTypes[] GetMemberDescendantsParameterTypes
        {
            get
            {
                List<DataCreationTypes> parametertypes = new List<DataCreationTypes>();
                parametertypes.Add(DataCreationTypes.UniqueIdentifier);
                return parametertypes.ToArray();
            }
        }

        /// <summary>
        /// Returns the main code block for the GetMemberDescendants function
        /// </summary>
        public static string GetMemberDescendantsCode
        {
            get
            {
                string functioncode = @"
                    SELECT descendantid AS ID FROM Cubed_Structures_Descendants WHERE id = @memberid
                    UNION
                    SELECT @memberid AS ID";
                return functioncode;
            }
        }

        /// <summary>
        /// Search Categories Field Names
        /// </summary>
        public static string[] SearchCategoriesFieldNames
        {
            get
            {
                List<string> fieldnames = new List<string>();
                fieldnames.Add("id");
                fieldnames.Add("name");
                return fieldnames.ToArray();
            }
        }

        /// <summary>
        /// Search Categories Field Types
        /// </summary>
        public static DataCreationTypes[] SearchCategoriesFieldTypes
        {
            get
            {
                List<DataCreationTypes> fieldtypes = new List<DataCreationTypes>();
                fieldtypes.Add(DataCreationTypes.UniqueIdentifier);
                fieldtypes.Add(DataCreationTypes.StringField);
                return fieldtypes.ToArray();
            }
        }
    }
}
