/*
 * This source file (DesktopSearchResult.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Contains the details of a single result viewing available for a Cubed Search
    /// </summary>
    [Serializable]
    [DataContract]
    public class DesktopSearchResult
    {
        /// <summary>
        /// Initialises a new, blank default search result
        /// </summary>
        public DesktopSearchResult()
        {
            _id = Guid.NewGuid();
            _resultid = new Guid();
            _resultversion = 1.0;
            _displaysettings = String.Empty;
            _supportsweb = true;
            _supportswindows = false;
        }

        /// <summary>
        /// Initialises a new search result based on the details given
        /// </summary>
        /// <param name="resultid">The unique identifier of the Result Add In used for this search</param>
        /// <param name="resultversion">The version of the Result Add In used for this search</param>
        /// <param name="supportsweb">Whether this result supports web viewing</param>
        /// <param name="supportswindows">Whether this result supports windows viewing</param>
        /// <param name="displaysettings">The display settings of the result</param>
        public DesktopSearchResult(Guid resultid, double resultversion, string displaysettings, 
            bool supportsweb, bool supportswindows)
        {
            // First we assign all fields
            _id = Guid.NewGuid();
            _resultid = resultid;
            _resultversion = resultversion;
            _displaysettings = displaysettings;
            _supportsweb = supportsweb;
            _supportswindows = supportswindows;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of this new search result
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private Guid _resultid;
        /// <summary>
        /// This is the unique identifier of the Result Add In used to generate this search
        /// </summary>
        [DataMember]
        public Guid ResultID
        {
            get { return _resultid; }
            set { _resultid = value; }
        }

        private double _resultversion;
        /// <summary>
        /// This is the version of the Result Add In used to generatethis search
        /// </summary>
        [DataMember]
        public double ResultVersion
        {
            get { return _resultversion; }
            set { _resultversion = value; }
        }

        private string _displaysettings;
        /// <summary>
        /// This is the display settings configuration for this result
        /// </summary>
        [DataMember]
        public string DisplaySettings
        {
            get { return _displaysettings; }
            set { _displaysettings = value; }
        }

        private bool _supportsweb;
        /// <summary>
        /// Whether this search result supports viewing in the web desktop
        /// </summary>
        [DataMember]
        public bool SupportsWeb
        {
            get { return _supportsweb; }
            set { _supportsweb = value; }
        }

        private bool _supportswindows;
        /// <summary>
        /// Whether this search result supports viewing in the windows desktop
        /// </summary>
        [DataMember]
        public bool SupportsWindows
        {
            get { return _supportswindows; }
            set { _supportswindows = value; }
        }
    }
}