/*
 * This source file (FormulaEvaluator.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Formula Evaluators are configured against measures and members and provide calculations through an 
    /// AddIn calculator which allow for complex calculations rather than simple aggregations of values
    /// </summary>
    [Serializable]
    [DataContract]
    public class FormulaEvaluator
    {
        /// <summary>
        /// Initialises a new default instance of a Formula Evaluator
        /// </summary>
        public FormulaEvaluator()
        {
            _id = Guid.NewGuid();
            _evaluatorid = new Guid();
            _evaluatorconfiguration = String.Empty;
            _evaluatorversion = 1.0;
        }

        /// <summary>
        /// Initialises a new Formula Evaluator based on the passed in values
        /// </summary>
        /// <param name="evaluatorid">The unique identifier of the Formula Evaluator used by this parser</param>
        /// <param name="evaluatorconfiguration">The encrypted serialised string containing the configuration
        /// for this Formula Evaluator</param>
        /// <param name="evaluatorversion">The version of the Add In this Evaluator was created under</param>
        public FormulaEvaluator(Guid evaluatorid, string evaluatorconfiguration,
            double evaluatorversion)
        {
            _id = Guid.NewGuid();
            _evaluatorid = evaluatorid;
            _evaluatorconfiguration = evaluatorconfiguration;
            _evaluatorversion = evaluatorversion;
        }

        private Guid _id;
        /// <summary>
        /// The unique identifier of the Formula Evaluator
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private Guid _evaluatorid;
        /// <summary>
        /// This is the unique identifier of the Add In which provides the Formula Evaluator
        /// </summary>
        [DataMember]
        public Guid EvaluatorID
        {
            get { return _evaluatorid; }
            set { _evaluatorid = value; }
        }

        private string _evaluatorconfiguration;
        /// <summary>
        /// This is the encrypted serialised string representing the configuration of the Evaluator Add In
        /// </summary>
        [DataMember]
        public string EvaluatorConfiguration
        {
            get { return _evaluatorconfiguration; }
            set { _evaluatorconfiguration = value; }
        }

        private double _evaluatorversion;
        /// <summary>
        /// This is the version of the Formula Evaluator Add In which was used to configure this Evaluator
        /// </summary>
        [DataMember]
        public double EvaluatorVersion
        {
            get { return _evaluatorversion; }
            set { _evaluatorversion = value; }
        }
    }
}