/*
 * This source file (Level.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using Bamboo.Cubed.Utilities;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// This is the class used for representing each level of this dimension
    /// </summary>
    [Serializable]
    [DataContract]
    public class Level
    {
        /// <summary>
        /// Creates a new default instance of the level class
        /// </summary>
        public Level()
        {
            _id = Guid.NewGuid();
            _dimension = new Guid();
            _members = new List<Guid>();
            _depth = 0;
            _name = "New Level";
            _description = "New Level";
        }

        /// <summary>
        /// Creates a new level based on the name and dimension given
        /// </summary>
        /// <param name="name">The name of the new level</param>
        /// <param name="dimension">The dimension this level belongs to</param>
        /// <param name="depth">The depth of the new member</param>
        public Level(string name, Dimension dimension, int depth)
        {
            _id = Guid.NewGuid();
            _dimension = dimension.ID;
            _members = new List<Guid>();
            _depth = depth;
            _name = name;
            _description = name;
        }        

        private Guid _id;
        /// <summary>
        /// The unique identifier of the level
        /// </summary>
        [DataMember]
        public Guid ID
        {
            get { return _id; }
            set { _id = value; }
        }

        private Guid _dimension;
        /// <summary>
        /// The dimension that this level belongs to
        /// </summary>
        [DataMember]
        public Guid Dimension
        {
            get { return _dimension; }
            set { _dimension = value; }
        }

        private int _depth;
        /// <summary>
        /// This is the zero based ordinal position of the level within the current dimension
        /// </summary>
        [DataMember]
        public int Depth
        {
            get { return _depth; }
            set { _depth = value; }
        }

        /// <summary>
        /// The unique name of the new level. This is provided for compatibility purposes with SQL Server Analysis Services. It is
        /// the same as the ID property.
        /// </summary>                    
        public Guid UniqueName
        {
            get { return _id; }
            set { _id = value; }
        }

        private string _name;
        /// <summary>
        /// The name of the new level
        /// </summary>
        [DataMember]
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        /// <summary>
        /// The caption of the level. This is provided for compatibility purposes with SQL Server Analysis Services. It is
        /// the same as the Name property.                    
        /// </summary>                    
        public string Caption
        {
            get { return _name; }
        }

        private List<Guid> _members;
        /// <summary>
        /// Returns all of the members of this level
        /// </summary>
        [DataMember]
        public List<Guid> Members
        {
            get { return _members; }
            set { _members = value; }
        }

        private string _description;
        /// <summary>
        /// The description of the level
        /// </summary>
        [DataMember]
        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }
    }  
}
