﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace Bamboo.Cubed.Structures
{
    /// <summary>
    /// Provides the details of a single endpoint for a Cubed Service
    /// </summary>
    [DataContract]
    public class ServiceEndPoint
    {
        /// <summary>
        /// Initialises a default Message Service End Point
        /// </summary>
        public ServiceEndPoint()
        {
            _endpointtype = ServiceEndPointType.NetTcp;
            _server = System.Environment.MachineName;
            _port = 10000;
            _integratedauthentication = true;
        }

        private ServiceEndPointType _endpointtype;
        /// <summary>
        /// The type of this end point
        /// </summary>
        [DataMember]
        public ServiceEndPointType EndpointType
        {
            get { return _endpointtype; }
            set { _endpointtype = value; }
        }

        private string _server;
        /// <summary>
        /// The name of the server hosting the service
        /// </summary>
        [DataMember]
        public string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        private int _port;
        /// <summary>
        /// The port number for the end point
        /// </summary>
        [DataMember]
        public int Port
        {
            get { return _port; }
            set { _port = value; }
        }

        private bool _integratedauthentication;
        /// <summary>
        /// Whether this server uses integrated authentication, or username/password authentication
        /// </summary>
        [DataMember]
        public bool IntegratedAuthentication
        {
            get { return _integratedauthentication; }
            set { _integratedauthentication = value; }
        }
    }
}
