﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Threading;

namespace Bamboo.Cubed.UI.WebControls
{
    public class ImageGaugeSubControl : WebControl
    {
        #region enum, var, delegate, event
        public enum NeedleColorEnum
        {
            Blue = 0,
            Gray = 1,
            Violet = 2,
            Magenta = 3,
            Red = 4,
            Green = 5,
            Yellow = 6
        };

        //private float _lastValue = 0;

        private const Byte ZERO = 0;
        private const Int32 FPS = 15;

        private Single fontBoundY1;
        private Single fontBoundY2;
        //private Bitmap gaugeBitmap;
        private Boolean drawGaugeBackground = true;

        //private Single m_value = 0;
        private Point m_Center = new Point(115, 115);
        private Single m_MinValue = 0;
        private Single m_MaxValue = 400;

        private Int32 _colorGuideXPos = 228;

        private Color m_BaseArcColor = Color.Gray;
        private Int32 m_BaseArcRadius = 80;
        private Int32 m_BaseArcStart = 135;
        private Int32 m_BaseArcSweep = 270;
        private Int32 m_BaseArcWidth = 2;

        private Color m_ScaleLinesInterColor = Color.Black;
        private Int32 m_ScaleLinesInterInnerRadius = 73;
        private Int32 m_ScaleLinesInterOuterRadius = 80;
        private Int32 m_ScaleLinesInterWidth = 1;

        private Int32 m_ScaleLinesMinorNumOf = 9;
        private Color m_ScaleLinesMinorColor = Color.Gray;
        private Int32 m_ScaleLinesMinorInnerRadius = 75;
        private Int32 m_ScaleLinesMinorOuterRadius = 80;
        private Int32 m_ScaleLinesMinorWidth = 1;

        private Single m_ScaleLinesMajorStepValue = 100.0f;
        private Color m_ScaleLinesMajorColor = Color.Black;
        private Int32 m_ScaleLinesMajorInnerRadius = 70;
        private Int32 m_ScaleLinesMajorOuterRadius = 80;
        private Int32 m_ScaleLinesMajorWidth = 2;

        private Int32 m_ScaleNumbersRadius = 95;
        private Color m_ScaleNumbersColor = Color.Gray;
        private String m_ScaleNumbersFormat;
        private Int32 m_ScaleNumbersStartScaleLine;
        private Int32 m_ScaleNumbersStepScaleLines = 1;
        private Int32 m_ScaleNumbersRotation = 0;

        private Int32 m_NeedleType = 0;
        private Int32 m_NeedleRadius = 80;
        private NeedleColorEnum m_NeedleColor1 = NeedleColorEnum.Gray;
        private Color m_NeedleColor2 = Color.DimGray;
        private Int32 m_NeedleWidth = 2;

        public class ValueInRangeChangedEventArgs : EventArgs
        {
            public Int32 valueInRange;

            public ValueInRangeChangedEventArgs(Int32 valueInRange)
            {
                this.valueInRange = valueInRange;
            }
        }

        public delegate void ValueInRangeChangedDelegate(Object sender, ValueInRangeChangedEventArgs e);
        public event ValueInRangeChangedDelegate ValueInRangeChanged;
        #endregion

        #region hidden , overridden inherited properties
        public new Boolean AllowDrop
        {
            get
            {
                return false;
            }
            set
            {

            }
        }
        public new Boolean AutoSize
        {
            get
            {
                return false;
            }
            set
            {

            }
        }
        public new Boolean ForeColor
        {
            get
            {
                return false;
            }
            set
            {
            }
        }
        public new Boolean ImeMode
        {
            get
            {
                return false;
            }
            set
            {
            }
        }

        public override System.Drawing.Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            set
            {
                base.BackColor = value;
                drawGaugeBackground = true;
            }
        }
        private Int32 _width = 220;
        private Int32 _height = 220;
        #endregion

        public ImageGaugeSubControl()
        {
            base.BackColor = Color.White;
        }

        #region properties

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The center of the gauge (in the control's client area).")]
        public Point Center
        {
            get
            {
                return m_Center;
            }
            set
            {
                if (m_Center != value)
                {
                    m_Center = new Point(value.X, value.Y+10);
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The minimum value to show on the scale.")]
        public Single MinValue
        {
            get
            {
                return m_MinValue;
            }
            set
            {
                if ((m_MinValue != value)
                && (value < m_MaxValue))
                {
                    m_MinValue = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The maximum value to show on the scale.")]
        public Single MaxValue
        {
            get
            {
                return m_MaxValue;
            }
            set
            {
                if ((m_MaxValue != value)
                && (value > m_MinValue))
                {
                    m_MaxValue = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The color of the base arc.")]
        public Color BaseArcColor
        {
            get
            {
                return m_BaseArcColor;
            }
            set
            {
                if (m_BaseArcColor != value)
                {
                    m_BaseArcColor = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The radius of the base arc.")]
        public Int32 BaseArcRadius
        {
            get
            {
                return m_BaseArcRadius;
            }
            set
            {
                if (m_BaseArcRadius != value)
                {
                    m_BaseArcRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The start angle of the base arc.")]
        public Int32 BaseArcStart
        {
            get
            {
                return m_BaseArcStart;
            }
            set
            {
                if (m_BaseArcStart != value)
                {
                    m_BaseArcStart = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The sweep angle of the base arc.")]
        public Int32 BaseArcSweep
        {
            get
            {
                return m_BaseArcSweep;
            }
            set
            {
                if (m_BaseArcSweep != value)
                {
                    m_BaseArcSweep = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The width of the base arc.")]
        public Int32 BaseArcWidth
        {
            get
            {
                return m_BaseArcWidth;
            }
            set
            {
                if (m_BaseArcWidth != value)
                {
                    m_BaseArcWidth = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The color of the inter scale lines which are the middle scale lines for an uneven number of minor scale lines.")]
        public Color ScaleLinesInterColor
        {
            get
            {
                return m_ScaleLinesInterColor;
            }
            set
            {
                if (m_ScaleLinesInterColor != value)
                {
                    m_ScaleLinesInterColor = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The inner radius of the inter scale lines which are the middle scale lines for an uneven number of minor scale lines.")]
        public Int32 ScaleLinesInterInnerRadius
        {
            get
            {
                return m_ScaleLinesInterInnerRadius;
            }
            set
            {
                if (m_ScaleLinesInterInnerRadius != value)
                {
                    m_ScaleLinesInterInnerRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The outer radius of the inter scale lines which are the middle scale lines for an uneven number of minor scale lines.")]
        public Int32 ScaleLinesInterOuterRadius
        {
            get
            {
                return m_ScaleLinesInterOuterRadius;
            }
            set
            {
                if (m_ScaleLinesInterOuterRadius != value)
                {
                    m_ScaleLinesInterOuterRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The width of the inter scale lines which are the middle scale lines for an uneven number of minor scale lines.")]
        public Int32 ScaleLinesInterWidth
        {
            get
            {
                return m_ScaleLinesInterWidth;
            }
            set
            {
                if (m_ScaleLinesInterWidth != value)
                {
                    m_ScaleLinesInterWidth = value;
                    drawGaugeBackground = true;

                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The number of minor scale lines.")]
        public Int32 ScaleLinesMinorNumOf
        {
            get
            {
                return m_ScaleLinesMinorNumOf;
            }
            set
            {
                if (m_ScaleLinesMinorNumOf != value)
                {
                    m_ScaleLinesMinorNumOf = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The color of the minor scale lines.")]
        public Color ScaleLinesMinorColor
        {
            get
            {
                return m_ScaleLinesMinorColor;
            }
            set
            {
                if (m_ScaleLinesMinorColor != value)
                {
                    m_ScaleLinesMinorColor = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The inner radius of the minor scale lines.")]
        public Int32 ScaleLinesMinorInnerRadius
        {
            get
            {
                return m_ScaleLinesMinorInnerRadius;
            }
            set
            {
                if (m_ScaleLinesMinorInnerRadius != value)
                {
                    m_ScaleLinesMinorInnerRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The outer radius of the minor scale lines.")]
        public Int32 ScaleLinesMinorOuterRadius
        {
            get
            {
                return m_ScaleLinesMinorOuterRadius;
            }
            set
            {
                if (m_ScaleLinesMinorOuterRadius != value)
                {
                    m_ScaleLinesMinorOuterRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The width of the minor scale lines.")]
        public Int32 ScaleLinesMinorWidth
        {
            get
            {
                return m_ScaleLinesMinorWidth;
            }
            set
            {
                if (m_ScaleLinesMinorWidth != value)
                {
                    m_ScaleLinesMinorWidth = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The step value of the major scale lines.")]
        public Single ScaleLinesMajorStepValue
        {
            get
            {
                return m_ScaleLinesMajorStepValue;
            }
            set
            {
                if ((m_ScaleLinesMajorStepValue != value) && (value > 0))
                {
                    //KO-what the ....?
                    //m_ScaleLinesMajorStepValue = Math.Max(Math.Min(value, m_MaxValue), m_MinValue);
                    m_ScaleLinesMajorStepValue = value;
                    drawGaugeBackground = true;
                }
            }
        }
        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The color of the major scale lines.")]
        public Color ScaleLinesMajorColor
        {
            get
            {
                return m_ScaleLinesMajorColor;
            }
            set
            {
                if (m_ScaleLinesMajorColor != value)
                {
                    m_ScaleLinesMajorColor = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The inner radius of the major scale lines.")]
        public Int32 ScaleLinesMajorInnerRadius
        {
            get
            {
                return m_ScaleLinesMajorInnerRadius;
            }
            set
            {
                if (m_ScaleLinesMajorInnerRadius != value)
                {
                    m_ScaleLinesMajorInnerRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The outer radius of the major scale lines.")]
        public Int32 ScaleLinesMajorOuterRadius
        {
            get
            {
                return m_ScaleLinesMajorOuterRadius;
            }
            set
            {
                if (m_ScaleLinesMajorOuterRadius != value)
                {
                    m_ScaleLinesMajorOuterRadius = value;
                    drawGaugeBackground = true;

                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The width of the major scale lines.")]
        public Int32 ScaleLinesMajorWidth
        {
            get
            {
                return m_ScaleLinesMajorWidth;
            }
            set
            {
                if (m_ScaleLinesMajorWidth != value)
                {
                    m_ScaleLinesMajorWidth = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The radius of the scale numbers.")]
        public Int32 ScaleNumbersRadius
        {
            get
            {
                return m_ScaleNumbersRadius;
            }
            set
            {
                if (m_ScaleNumbersRadius != value)
                {
                    m_ScaleNumbersRadius = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The color of the scale numbers.")]
        public Color ScaleNumbersColor
        {
            get
            {
                return m_ScaleNumbersColor;
            }
            set
            {
                if (m_ScaleNumbersColor != value)
                {
                    m_ScaleNumbersColor = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The format of the scale numbers.")]
        public String ScaleNumbersFormat
        {
            get
            {
                return m_ScaleNumbersFormat;
            }
            set
            {
                if (m_ScaleNumbersFormat != value)
                {
                    m_ScaleNumbersFormat = value;
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The number of the scale line to start writing numbers next to.")]
        public Int32 ScaleNumbersStartScaleLine
        {
            get
            {
                return m_ScaleNumbersStartScaleLine;
            }
            set
            {
                if (m_ScaleNumbersStartScaleLine != value)
                {
                    m_ScaleNumbersStartScaleLine = Math.Max(value, 1);
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The number of scale line steps for writing numbers.")]
        public Int32 ScaleNumbersStepScaleLines
        {
            get
            {
                return m_ScaleNumbersStepScaleLines;
            }
            set
            {
                if (m_ScaleNumbersStepScaleLines != value)
                {
                    m_ScaleNumbersStepScaleLines = Math.Max(value, 1);
                    drawGaugeBackground = true;
                }
            }
        }

        [System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Gauge"),
        System.ComponentModel.Description("The angle relative to the tangent of the base arc at a scale line that is used to rotate numbers. set to 0 for no rotation or e.g. set to 90.")]
        public Int32 ScaleNumbersRotation
        {
            get
            {
                return m_ScaleNumbersRotation;
            }
            set
            {
                if (m_ScaleNumbersRotation != value)
                {
                    m_ScaleNumbersRotation = value;
                    drawGaugeBackground = true;
                }
            }
        }
        #endregion

        #region helper
        private void FindFontBounds()
        {
            //find upper and lower bounds for numeric characters
            Int32 c1;
            Int32 c2;
            Boolean boundfound;
            Bitmap b;
            Graphics g;
            SolidBrush backBrush = new SolidBrush(Color.White);
            SolidBrush foreBrush = new SolidBrush(Color.Black);
            SizeF boundingBox;

            b = new Bitmap(5, 5);
            g = Graphics.FromImage(b);
            Font fnt = new Font("Verdana", 8);
            boundingBox = g.MeasureString("0123456789", fnt, -1, StringFormat.GenericTypographic);
            b = new Bitmap((Int32)(boundingBox.Width), (Int32)(boundingBox.Height));
            g = Graphics.FromImage(b);
            g.FillRectangle(backBrush, 0.0F, 0.0F, boundingBox.Width, boundingBox.Height);
            g.DrawString("0123456789", fnt, foreBrush, 0.0F, 0.0F, StringFormat.GenericTypographic);

            fontBoundY1 = 0;
            fontBoundY2 = 0;
            c1 = 0;
            boundfound = false;
            while ((c1 < b.Height) && (!boundfound))
            {
                c2 = 0;
                while ((c2 < b.Width) && (!boundfound))
                {
                    if (b.GetPixel(c2, c1) != backBrush.Color)
                    {
                        fontBoundY1 = c1;
                        boundfound = true;
                    }
                    c2++;
                }
                c1++;
            }

            c1 = b.Height - 1;
            boundfound = false;
            while ((0 < c1) && (!boundfound))
            {
                c2 = 0;
                while ((c2 < b.Width) && (!boundfound))
                {
                    if (b.GetPixel(c2, c1) != backBrush.Color)
                    {
                        fontBoundY2 = c1;
                        boundfound = true;
                    }
                    c2++;
                }
                c1--;
            }
        }
        #endregion

        #region base member overrides

        protected override void RenderContents(HtmlTextWriter output)
        {
            if ((_width < 10) || (_height < 10))
            {
                return;
            }

            Rectangle rectangle = new Rectangle(0, 0, _width, _height);
            Bitmap gaugeBitmap = new Bitmap(Width, Height);
            Graphics ggr = Graphics.FromImage(gaugeBitmap);

            #region Draw Background
            if (drawGaugeBackground)
            {
                drawGaugeBackground = false;

                FindFontBounds();

                ggr.FillRectangle(new SolidBrush(BackColor), rectangle);

                ggr.SmoothingMode = SmoothingMode.AntiAlias;
                ggr.PixelOffsetMode = PixelOffsetMode.HighQuality;

                GraphicsPath gp = new GraphicsPath();
                Single rangeStartAngle;
                Single rangeSweepAngle;
                for (Int32 counter = 0; counter < _ranges.Length; counter++)
                {
                    if (_ranges[counter].RangeEndValue > _ranges[counter].RangeStartValue
                    && _ranges[counter].RangeEnabled)
                    {
                        rangeStartAngle = m_BaseArcStart + (_ranges[counter].RangeStartValue - m_MinValue) * m_BaseArcSweep / (m_MaxValue - m_MinValue);
                        rangeSweepAngle = (_ranges[counter].RangeEndValue - _ranges[counter].RangeStartValue) * m_BaseArcSweep / (m_MaxValue - m_MinValue);
                        gp.Reset();
                        gp.AddPie(new Rectangle(m_Center.X - _ranges[counter].RangeOuterRadius, m_Center.Y - _ranges[counter].RangeOuterRadius, 2 * _ranges[counter].RangeOuterRadius, 2 * _ranges[counter].RangeOuterRadius), rangeStartAngle, rangeSweepAngle);
                        gp.Reverse();
                        gp.AddPie(new Rectangle(m_Center.X - _ranges[counter].RangeInnerRadius, m_Center.Y - _ranges[counter].RangeInnerRadius, 2 * _ranges[counter].RangeInnerRadius, 2 * _ranges[counter].RangeInnerRadius), rangeStartAngle, rangeSweepAngle);
                        gp.Reverse();
                        ggr.SetClip(gp);
                        ggr.FillPie(new SolidBrush(_ranges[counter].Color), new Rectangle(m_Center.X - _ranges[counter].RangeOuterRadius, m_Center.Y - _ranges[counter].RangeOuterRadius, 2 * _ranges[counter].RangeOuterRadius, 2 * _ranges[counter].RangeOuterRadius), rangeStartAngle, rangeSweepAngle);
                    }
                }

                ggr.SetClip(rectangle);
                if (m_BaseArcRadius > 0)
                {
                    ggr.DrawArc(new Pen(m_BaseArcColor, m_BaseArcWidth), new Rectangle(m_Center.X - m_BaseArcRadius, m_Center.Y - m_BaseArcRadius, 2 * m_BaseArcRadius, 2 * m_BaseArcRadius), m_BaseArcStart, m_BaseArcSweep);
                }

                String valueText = "";
                SizeF boundingBox;
                Single countValue = 0;
                Int32 counter1 = 0;

                Font fnt = new Font("Verdana", 8);
                Font scaleNumbersFont = new Font("Verdana", 7, FontStyle.Bold);
                while (countValue <= (m_MaxValue - m_MinValue))
                {
                    valueText = (m_MinValue + countValue).ToString(m_ScaleNumbersFormat);
                    ggr.ResetTransform();
                    boundingBox = ggr.MeasureString(valueText, fnt, -1, StringFormat.GenericTypographic);

                    gp.Reset();
                    gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesMajorOuterRadius, m_Center.Y - m_ScaleLinesMajorOuterRadius, 2 * m_ScaleLinesMajorOuterRadius, 2 * m_ScaleLinesMajorOuterRadius));
                    gp.Reverse();
                    gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesMajorInnerRadius, m_Center.Y - m_ScaleLinesMajorInnerRadius, 2 * m_ScaleLinesMajorInnerRadius, 2 * m_ScaleLinesMajorInnerRadius));
                    gp.Reverse();
                    ggr.SetClip(gp);

                    ggr.DrawLine(new Pen(m_ScaleLinesMajorColor, m_ScaleLinesMajorWidth),
                    (Single)(Center.X),
                    (Single)(Center.Y),
                    (Single)(Center.X + 2 * m_ScaleLinesMajorOuterRadius * Math.Cos((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue)) * Math.PI / 180.0)),
                    (Single)(Center.Y + 2 * m_ScaleLinesMajorOuterRadius * Math.Sin((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue)) * Math.PI / 180.0)));

                    gp.Reset();
                    gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesMinorOuterRadius, m_Center.Y - m_ScaleLinesMinorOuterRadius, 2 * m_ScaleLinesMinorOuterRadius, 2 * m_ScaleLinesMinorOuterRadius));
                    gp.Reverse();
                    gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesMinorInnerRadius, m_Center.Y - m_ScaleLinesMinorInnerRadius, 2 * m_ScaleLinesMinorInnerRadius, 2 * m_ScaleLinesMinorInnerRadius));
                    gp.Reverse();
                    ggr.SetClip(gp);

                    if (countValue < (m_MaxValue - m_MinValue))
                    {
                        for (Int32 counter2 = 1; counter2 <= m_ScaleLinesMinorNumOf; counter2++)
                        {
                            if (((m_ScaleLinesMinorNumOf % 2) == 1) && ((Int32)(m_ScaleLinesMinorNumOf / 2) + 1 == counter2))
                            {
                                gp.Reset();
                                gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesInterOuterRadius, m_Center.Y - m_ScaleLinesInterOuterRadius, 2 * m_ScaleLinesInterOuterRadius, 2 * m_ScaleLinesInterOuterRadius));
                                gp.Reverse();
                                gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesInterInnerRadius, m_Center.Y - m_ScaleLinesInterInnerRadius, 2 * m_ScaleLinesInterInnerRadius, 2 * m_ScaleLinesInterInnerRadius));
                                gp.Reverse();
                                ggr.SetClip(gp);

                                ggr.DrawLine(new Pen(m_ScaleLinesInterColor, m_ScaleLinesInterWidth),
                                (Single)(Center.X),
                                (Single)(Center.Y),
                                (Single)(Center.X + 2 * m_ScaleLinesInterOuterRadius * Math.Cos((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue) + counter2 * m_BaseArcSweep / (((Single)((m_MaxValue - m_MinValue) / m_ScaleLinesMajorStepValue)) * (m_ScaleLinesMinorNumOf + 1))) * Math.PI / 180.0)),
                                (Single)(Center.Y + 2 * m_ScaleLinesInterOuterRadius * Math.Sin((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue) + counter2 * m_BaseArcSweep / (((Single)((m_MaxValue - m_MinValue) / m_ScaleLinesMajorStepValue)) * (m_ScaleLinesMinorNumOf + 1))) * Math.PI / 180.0)));

                                gp.Reset();
                                gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesMinorOuterRadius, m_Center.Y - m_ScaleLinesMinorOuterRadius, 2 * m_ScaleLinesMinorOuterRadius, 2 * m_ScaleLinesMinorOuterRadius));
                                gp.Reverse();
                                gp.AddEllipse(new Rectangle(m_Center.X - m_ScaleLinesMinorInnerRadius, m_Center.Y - m_ScaleLinesMinorInnerRadius, 2 * m_ScaleLinesMinorInnerRadius, 2 * m_ScaleLinesMinorInnerRadius));
                                gp.Reverse();
                                ggr.SetClip(gp);
                            }
                            else
                            {
                                ggr.DrawLine(new Pen(m_ScaleLinesMinorColor, m_ScaleLinesMinorWidth),
                                (Single)(Center.X),
                                (Single)(Center.Y),
                                (Single)(Center.X + 2 * m_ScaleLinesMinorOuterRadius * Math.Cos((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue) + counter2 * m_BaseArcSweep / (((Single)((m_MaxValue - m_MinValue) / m_ScaleLinesMajorStepValue)) * (m_ScaleLinesMinorNumOf + 1))) * Math.PI / 180.0)),
                                (Single)(Center.Y + 2 * m_ScaleLinesMinorOuterRadius * Math.Sin((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue) + counter2 * m_BaseArcSweep / (((Single)((m_MaxValue - m_MinValue) / m_ScaleLinesMajorStepValue)) * (m_ScaleLinesMinorNumOf + 1))) * Math.PI / 180.0)));
                            }
                        }
                    }

                    ggr.SetClip(rectangle);

                    if (m_ScaleNumbersRotation != 0)
                    {
                        ggr.TextRenderingHint = System.Drawing.Text.TextRenderingHint.AntiAlias;
                        ggr.RotateTransform(90.0F + m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue));
                    }

                    ggr.TranslateTransform((Single)(Center.X + m_ScaleNumbersRadius * Math.Cos((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue)) * Math.PI / 180.0f)),
                                           (Single)(Center.Y + m_ScaleNumbersRadius * Math.Sin((m_BaseArcStart + countValue * m_BaseArcSweep / (m_MaxValue - m_MinValue)) * Math.PI / 180.0f)),
                                           System.Drawing.Drawing2D.MatrixOrder.Append);


                    if (counter1 >= ScaleNumbersStartScaleLine - 1)
                    {
                        ggr.DrawString(valueText, scaleNumbersFont, new SolidBrush(m_ScaleNumbersColor), -boundingBox.Width / 2, -fontBoundY1 - (fontBoundY2 - fontBoundY1 + 1) / 2, StringFormat.GenericTypographic);
                    }

                    countValue += m_ScaleLinesMajorStepValue;
                    counter1++;
                }

                ggr.ResetTransform();
                ggr.SetClip(rectangle);

                if (m_ScaleNumbersRotation != 0)
                {
                    ggr.TextRenderingHint = System.Drawing.Text.TextRenderingHint.SystemDefault;
                }

                for (Int32 counter = 0; counter < _captions.Length; counter++)
                {
                    if (_captions[counter].CapText != "")
                    {
                        ggr.DrawString(_captions[counter].CapText, fnt, new SolidBrush(_captions[counter].CapColor), _captions[counter].CapPosition.X, _captions[counter].CapPosition.Y, StringFormat.GenericTypographic);
                    }
                }
            }
            #endregion Draw Background

            ggr.DrawImageUnscaled(gaugeBitmap, 0, 0);
            ggr.SmoothingMode = SmoothingMode.AntiAlias;
            ggr.PixelOffsetMode = PixelOffsetMode.HighQuality;
            Int32 _needleYPos = 20;
            #region Draw Needles
            for (Int32 i = 0;    i < _needles.Length; i++)
            {
                Single brushAngle = (Int32)(m_BaseArcStart + (_needles[i].Value - m_MinValue) * m_BaseArcSweep / (m_MaxValue - m_MinValue)) % 360;
                Double needleAngle = brushAngle * Math.PI / 180;

                Color needleColor = Color.Black;
                #region Needle Type
                switch (_needles[i].NeedleType)
                {
                    
                    case Needle.NeedleTypeEnum.Pointer:
                        PointF[] points = new PointF[3];
                        Brush brush1 = Brushes.White;
                        Brush brush2 = Brushes.White;
                        Brush brush3 = Brushes.White;
                        Brush brush4 = Brushes.White;

                        Brush brushBucket = Brushes.White;
                        Int32 subcol = (Int32)(((brushAngle + 225) % 180) * 100 / 180);
                        Int32 subcol2 = (Int32)(((brushAngle + 135) % 180) * 100 / 180);

                        switch (_needles[i].NeedleColor)
                        {
                            case NeedleColorEnum.Gray:
                                ggr.FillEllipse(new SolidBrush(Color.Gray), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Gray;
                                brush1 = new SolidBrush(Color.FromArgb(80 + subcol, 80 + subcol, 80 + subcol));
                                brush2 = new SolidBrush(Color.FromArgb(180 - subcol, 180 - subcol, 180 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(80 + subcol2, 80 + subcol2, 80 + subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(180 - subcol2, 180 - subcol2, 180 - subcol2));
                                ggr.DrawEllipse(Pens.Gray, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                            case NeedleColorEnum.Red:
                                ggr.FillEllipse(new SolidBrush(Color.Red), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Red;
                                brush1 = new SolidBrush(Color.FromArgb(145 + subcol, subcol, subcol));
                                brush2 = new SolidBrush(Color.FromArgb(245 - subcol, 100 - subcol, 100 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(145 + subcol2, subcol2, subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(245 - subcol2, 100 - subcol2, 100 - subcol2));
                                ggr.DrawEllipse(Pens.Red, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                            case NeedleColorEnum.Green:
                                ggr.FillEllipse(new SolidBrush(Color.Green), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Green;
                                brush1 = new SolidBrush(Color.FromArgb(subcol, 145 + subcol, subcol));
                                brush2 = new SolidBrush(Color.FromArgb(100 - subcol, 245 - subcol, 100 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(subcol2, 145 + subcol2, subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(100 - subcol2, 245 - subcol2, 100 - subcol2));
                                ggr.DrawEllipse(Pens.Green, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                            case NeedleColorEnum.Blue:
                                ggr.FillEllipse(new SolidBrush(Color.Blue), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Blue;
                                brush1 = new SolidBrush(Color.FromArgb(subcol, subcol, 145 + subcol));
                                brush2 = new SolidBrush(Color.FromArgb(100 - subcol, 100 - subcol, 245 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(subcol2, subcol2, 145 + subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(100 - subcol2, 100 - subcol2, 245 - subcol2));
                                ggr.DrawEllipse(Pens.Blue, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                            case NeedleColorEnum.Magenta:
                                ggr.FillEllipse(new SolidBrush(Color.Magenta), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Magenta;
                                brush1 = new SolidBrush(Color.FromArgb(subcol, 145 + subcol, 145 + subcol));
                                brush2 = new SolidBrush(Color.FromArgb(100 - subcol, 245 - subcol, 245 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(subcol2, 145 + subcol2, 145 + subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(100 - subcol2, 245 - subcol2, 245 - subcol2));
                                ggr.DrawEllipse(Pens.Magenta, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                            case NeedleColorEnum.Violet:
                                ggr.FillEllipse(new SolidBrush(Color.Gray), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Violet;
                                brush1 = new SolidBrush(Color.FromArgb(145 + subcol, subcol, 145 + subcol));
                                brush2 = new SolidBrush(Color.FromArgb(245 - subcol, 100 - subcol, 245 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(145 + subcol2, subcol2, 145 + subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(245 - subcol2, 100 - subcol2, 245 - subcol2));
                                ggr.DrawEllipse(Pens.Violet, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                            case NeedleColorEnum.Yellow:
                                ggr.FillEllipse(new SolidBrush(Color.Gray), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                needleColor = Color.Yellow;
                                brush1 = new SolidBrush(Color.FromArgb(145 + subcol, 145 + subcol, subcol));
                                brush2 = new SolidBrush(Color.FromArgb(245 - subcol, 245 - subcol, 100 - subcol));
                                brush3 = new SolidBrush(Color.FromArgb(145 + subcol2, 145 + subcol2, subcol2));
                                brush4 = new SolidBrush(Color.FromArgb(245 - subcol2, 245 - subcol2, 100 - subcol2));
                                ggr.DrawEllipse(Pens.Violet, Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);
                                break;
                        }

                        if (Math.Floor((Single)(((brushAngle + 225) % 360) / 180.0)) == 0)
                        {
                            brushBucket = brush1;
                            brush1 = brush2;
                            brush2 = brushBucket;
                        }

                        if (Math.Floor((Single)(((brushAngle + 135) % 360) / 180.0)) == 0)
                        {
                            brush4 = brush3;
                        }

                        points[0].X = (Single)(Center.X + _needles[i].NeedleRadius * Math.Cos(needleAngle));
                        points[0].Y = (Single)(Center.Y + _needles[i].NeedleRadius * Math.Sin(needleAngle));
                        points[1].X = (Single)(Center.X - _needles[i].NeedleRadius / 20 * Math.Cos(needleAngle));
                        points[1].Y = (Single)(Center.Y - _needles[i].NeedleRadius / 20 * Math.Sin(needleAngle));
                        points[2].X = (Single)(Center.X - _needles[i].NeedleRadius / 5 * Math.Cos(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Cos(needleAngle + Math.PI / 2));
                        points[2].Y = (Single)(Center.Y - _needles[i].NeedleRadius / 5 * Math.Sin(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Sin(needleAngle + Math.PI / 2));
                        ggr.FillPolygon(brush1, points);

                        points[2].X = (Single)(Center.X - _needles[i].NeedleRadius / 5 * Math.Cos(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Cos(needleAngle - Math.PI / 2));
                        points[2].Y = (Single)(Center.Y - _needles[i].NeedleRadius / 5 * Math.Sin(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Sin(needleAngle - Math.PI / 2));
                        ggr.FillPolygon(brush2, points);

                        points[0].X = (Single)(Center.X - (_needles[i].NeedleRadius / 20 - 1) * Math.Cos(needleAngle));
                        points[0].Y = (Single)(Center.Y - (_needles[i].NeedleRadius / 20 - 1) * Math.Sin(needleAngle));
                        points[1].X = (Single)(Center.X - _needles[i].NeedleRadius / 5 * Math.Cos(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Cos(needleAngle + Math.PI / 2));
                        points[1].Y = (Single)(Center.Y - _needles[i].NeedleRadius / 5 * Math.Sin(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Sin(needleAngle + Math.PI / 2));
                        points[2].X = (Single)(Center.X - _needles[i].NeedleRadius / 5 * Math.Cos(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Cos(needleAngle - Math.PI / 2));
                        points[2].Y = (Single)(Center.Y - _needles[i].NeedleRadius / 5 * Math.Sin(needleAngle) + _needles[i].NeedleWidth * 2 * Math.Sin(needleAngle - Math.PI / 2));
                        ggr.FillPolygon(brush4, points);

                        points[0].X = (Single)(Center.X - _needles[i].NeedleRadius / 20 * Math.Cos(needleAngle));
                        points[0].Y = (Single)(Center.Y - _needles[i].NeedleRadius / 20 * Math.Sin(needleAngle));
                        points[1].X = (Single)(Center.X + _needles[i].NeedleRadius * Math.Cos(needleAngle));
                        points[1].Y = (Single)(Center.Y + _needles[i].NeedleRadius * Math.Sin(needleAngle));

                        ggr.DrawLine(new Pen(needleColor), Center.X, Center.Y, points[0].X, points[0].Y);
                        ggr.DrawLine(new Pen(needleColor), Center.X, Center.Y, points[1].X, points[1].Y);
                        break;
                    case Needle.NeedleTypeEnum.Line:
                        Point startPoint = new Point((Int32)(Center.X - _needles[i].NeedleRadius / 8 * Math.Cos(needleAngle)),
                                                   (Int32)(Center.Y - _needles[i].NeedleRadius / 8 * Math.Sin(needleAngle)));
                        Point endPoint = new Point((Int32)(Center.X + _needles[i].NeedleRadius * Math.Cos(needleAngle)),
                                                 (Int32)(Center.Y + _needles[i].NeedleRadius * Math.Sin(needleAngle)));

                        ggr.FillEllipse(new SolidBrush(needleColor), Center.X - _needles[i].NeedleWidth * 3, Center.Y - _needles[i].NeedleWidth * 3, _needles[i].NeedleWidth * 6, _needles[i].NeedleWidth * 6);

                        switch (m_NeedleColor1)
                        {
                            case NeedleColorEnum.Gray:
                                ggr.DrawLine(new Pen(Color.DarkGray, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.DarkGray, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                            case NeedleColorEnum.Red:
                                ggr.DrawLine(new Pen(Color.Red, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.Red, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                            case NeedleColorEnum.Green:
                                ggr.DrawLine(new Pen(Color.Green, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.Green, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                            case NeedleColorEnum.Blue:
                                ggr.DrawLine(new Pen(Color.Blue, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.Blue, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                            case NeedleColorEnum.Magenta:
                                ggr.DrawLine(new Pen(Color.Magenta, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.Magenta, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                            case NeedleColorEnum.Violet:
                                ggr.DrawLine(new Pen(Color.Violet, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.Violet, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                            case NeedleColorEnum.Yellow:
                                ggr.DrawLine(new Pen(Color.Yellow, _needles[i].NeedleWidth), Center.X, Center.Y, endPoint.X, endPoint.Y);
                                ggr.DrawLine(new Pen(Color.Yellow, _needles[i].NeedleWidth), Center.X, Center.Y, startPoint.X, startPoint.Y);
                                break;
                        }
                        break;
                }
                ggr.FillEllipse(new SolidBrush(needleColor), new Rectangle(_colorGuideXPos, _needleYPos, 10, 10));
                ggr.DrawString(_needles[i].NeedleText + " = " + _needles[i].Value.ToString(System.Globalization.CultureInfo.InvariantCulture), new Font("Arial", 8.0f, FontStyle.Regular), new SolidBrush(m_ScaleNumbersColor), new PointF(_colorGuideXPos + 15, _needleYPos));
                _needleYPos += 20;
                #endregion Needle Type
            }

            #endregion Draw Needles
            
            string urldirectory = Assembly.GetExecutingAssembly().CodeBase;
            string realdirectory = urldirectory.Replace('/', '\\');
            realdirectory = realdirectory.Substring(8, realdirectory.Length - 8);
            realdirectory = realdirectory.Substring(0, realdirectory.LastIndexOf('\\') + 1);
            realdirectory = realdirectory.TrimEnd("bin\\".ToCharArray());
            realdirectory += "\\gauges\\";

            // Then check whether the flash subdirectory exists
            Guid imagename = Guid.NewGuid();
            if (!Directory.Exists(realdirectory))
                Directory.CreateDirectory(realdirectory);
            string file = realdirectory + imagename.ToString() + ".jpg";
            string imgFile = "./gauges/" + imagename.ToString() + ".jpg";
            gaugeBitmap.Save(file, ImageFormat.Jpeg);
            System.Web.UI.WebControls.Image imgControl = new System.Web.UI.WebControls.Image();
            String imgHtml = "<div id='" + imagename.ToString() + "Div'><center><img src='" + imgFile + "' ></center></div>";
            output.Write(imgHtml);
            Thread cleanupthread = new Thread(new ThreadStart(CleanFiles));
            cleanupthread.Start();
        }
        #endregion

        /// <summary>
        /// This cleans all files older than 10 minutes from the images directory
        /// </summary>
        private void CleanFiles()
        {
            // Grab our real directory for storing of gauge images
            string urldirectory = Assembly.GetExecutingAssembly().CodeBase;
            string realdirectory = urldirectory.Replace('/', '\\');
            realdirectory = realdirectory.Substring(8, realdirectory.Length - 8);
            realdirectory = realdirectory.Substring(0, realdirectory.LastIndexOf('\\') + 1);
            realdirectory = realdirectory.TrimEnd("bin\\".ToCharArray());
            realdirectory += "\\gauges\\";

            // Find files which are older than 10 minutes
            try
            {
                DirectoryInfo dirinfo = new DirectoryInfo(realdirectory);
                List<string> deletefiles = new List<string>();
                foreach (FileInfo thisfile in dirinfo.GetFiles().Where(testfile => testfile.CreationTime.AddMinutes(10) < DateTime.Now))
                    deletefiles.Add(thisfile.FullName);
                foreach (string thisfile in deletefiles)
                    File.Delete(thisfile);
            }
            catch
            {
            }
        }

        public Int32 Width
        {
            get { return _width; }
            set { _width = value; }
        }

        public Int32 Height
        {
            get { return _height; }
            set { _height = value; }
        }

        private Range[] _ranges = { new Range() };
        public Range[] Ranges
        {
            get { return _ranges; }
            set 
            {
                _ranges = value;
                drawGaugeBackground = true;
            }
        }

        private Caption[] _captions = { new Caption() };
        public Caption[] Captions
        {
            get { return _captions; }
            set
            {
                _captions = value;
            }
        }

        private Needle[] _needles = { new Needle() };
        public Needle[] Needles
        {
            get { return _needles; }
            set 
            { 
                _needles = value;
            }
        }
        

        public class Range
        {
            private Color _rangeColor = Color.Green;
            [System.ComponentModel.Browsable(true),
             System.ComponentModel.Category("Range"),
             System.ComponentModel.Description("The color of the range.")]
            public Color Color
            {
                get
                {
                    return _rangeColor;
                }
                set
                {
                    if (_rangeColor != value)
                    {
                        _rangeColor = value;
                    }
                }
            }

            private Boolean _rangeEnabled = true;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Range"),
            System.ComponentModel.Description("Enables or disables the range selected by Range_Idx.")]
            public Boolean RangeEnabled
            {
                get
                {
                    return _rangeEnabled;
                }
                set
                {
                    if (_rangeEnabled != value)
                    {
                        _rangeEnabled = value;
                    }
                }
            }

            public Single _rangeEndValue = 100;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Range"),
            System.ComponentModel.Description("The end value of the range. Must be greater than RangeStartValue.")]
            public Single RangeEndValue
            {
                get
                {
                    return _rangeEndValue;
                }
                set
                {
                    if (_rangeEndValue != value)
                    {
                        _rangeEndValue = value;
                    }
                }
            }

            public Single _rangeStartValue = 0;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Range"),
            System.ComponentModel.Description("The start value of the range. Must be lower than RangeEndValue.")]
            public Single RangeStartValue
            {
                get
                {
                    return _rangeStartValue;
                }
                set
                {
                    if (_rangeStartValue != value)
                    {
                        _rangeStartValue = value;
                    }
                }
            }

            public Int32 _rangeInnerRadius=70;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Range"),
            System.ComponentModel.Description("The inner radius of the range.")]
            public Int32 RangeInnerRadius
            {
                get
                {
                    return _rangeInnerRadius;
                }
                set
                {
                    if (_rangeInnerRadius != value)
                    {
                        _rangeInnerRadius = value;
                    }
                }
            }

            public Int32 _rangeOuterRadius=80;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Range"),
            System.ComponentModel.Description("The outer radius of the range.")]
            public Int32 RangeOuterRadius
            {
                get
                {
                    return _rangeOuterRadius;
                }
                set
                {
                    if (_rangeOuterRadius != value)
                    {
                        _rangeOuterRadius = value;
                    }
                }
            }

            private Boolean _valueIsInRange = true;
            public Boolean ValueIsInRange
            {
                get { return _valueIsInRange; }
                set { _valueIsInRange = value; }
            }
        }

        public class Caption
        {
            private Color _capColor;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Caption"),
            System.ComponentModel.Description("The color of the caption text.")]
            public Color CapColor
            {
                get
                {
                    return _capColor;
                }
                set
                {
                    if (_capColor != value)
                    {
                        _capColor = value;
                    }
                }
            }

            private String _capText;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Caption"),
            System.ComponentModel.Description("The text of the caption.")]
            public String CapText
            {
                get
                {
                    return _capText;
                }
                set
                {
                    if (_capText != value)
                    {
                        _capText = value;
                    }
                }
            }

            private Point _capPosition;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Caption"),
            System.ComponentModel.Description("The position of the caption.")]
            public Point CapPosition
            {
                get
                {
                    return _capPosition;
                }
                set
                {
                    if (_capPosition != value)
                    {
                        _capPosition = value;
                    }
                }
            }

        }

        public class Needle
        {
            public delegate void ValueChangedDelegate(Object sender, ValueInRangeChangedEventArgs e);
            public event ValueChangedDelegate ValueChanged;

            private NeedleTypeEnum _needleType = 0;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Needle"),
            System.ComponentModel.Description("The type of the needle, currently only type 0 and 1 are supported. Type 0 looks nicers but if you experience performance problems you might consider using type 1.")]
            public NeedleTypeEnum NeedleType
            {
                get
                {
                    return _needleType;
                }
                set
                {
                    if (_needleType != value)
                    {
                        _needleType = value;
                    }
                }
            }

            private String _needleText = "";
            public String NeedleText
            {
                get { return _needleText; }
                set
                {
                    if (_needleText != value)
                    {
                        _needleText = value;
                    }
                }
            }

            private Int32 _needleRadius = 80;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Needle"),
            System.ComponentModel.Description("The radius of the needle.")]
            public Int32 NeedleRadius
            {
                get
                {
                    return _needleRadius;
                }
                set
                {
                    if (_needleRadius != value)
                    {
                        _needleRadius = value;
                    }
                }
            }

            private NeedleColorEnum _needleColor = NeedleColorEnum.Green;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Needle"),
            System.ComponentModel.Description("The color of the needle.")]
            public NeedleColorEnum NeedleColor
            {
                get
                {
                    return _needleColor;
                }
                set
                {
                    if (_needleColor != value)
                    {
                        _needleColor = value;
                    }
                }
            }

            private Int32 _needleWidth = 5;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Needle"),
            System.ComponentModel.Description("The width of the needle.")]
            public Int32 NeedleWidth
            {
                get
                {
                    return _needleWidth;
                }
                set
                {
                    if (_needleWidth != value)
                    {
                        _needleWidth = value;
                    }
                }
            }

            private Single _value;
            [System.ComponentModel.Browsable(true),
            System.ComponentModel.Category("Needle"),
            System.ComponentModel.Description("The value.")]
            public Single Value
            {
                get
                {
                    return _value;
                }
                set
                {
                    if (_value != value)
                    {
                        //for (Int32 counter = 0; counter < _ranges.Length; counter++)
                        //{
                        //    if ((_ranges[counter].RangeStartValue <= m_value)
                        //    && (m_value <= _ranges[counter].RangeEndValue)
                        //    && (_ranges[counter].RangeEnabled))
                        //    {
                        //        if (_ranges[counter].ValueIsInRange)
                        //        {
                        //            if (ValueInRangeChanged != null)
                        //            {
                        //                ValueInRangeChanged(this, new ValueInRangeChangedEventArgs(counter));
                        //            }
                        //        }
                        //    }
                        //    else
                        //    {
                        //        _ranges[counter].ValueIsInRange = false;
                        //    }
                        //}
                        _value = value;
                        if (ValueChanged != null)
                        {
                            ValueChanged(this, new ValueInRangeChangedEventArgs((int)_value));
                        }
                    }
                }
            }

            public enum NeedleTypeEnum : int
            {
                Pointer,
                Line
            }
        }
    }
}
