﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Threading;
using Bamboo.Cubed.UI.WebControls.ProgressBarGaugeObjects;
using Bamboo.Cubed.UI.WebControls.GaugesCommonObjects;

namespace Bamboo.Cubed.UI.WebControls
{
    public class ProgessBarGauge : WebControl
    {
        private Font _valuesFont = new Font("Verdana", 8, FontStyle.Bold);

        public ProgessBarGauge()
        {
        }

        #region Properties

        private MajorTicks _majorTicks = new MajorTicks();
        /// <summary>
        /// Gets or sets the major ticks settings.
        /// </summary>
        public MajorTicks MajorTicks
        {
            get { return _majorTicks; }
            set 
            {
                if (_majorTicks != value)
                {
                    _majorTicks = value;
                }
            }
        }

        private Double _minValue = 0;
        /// <summary>
        /// Gets or sets the minimum value to display on the Gauge.
        /// </summary>
        public Double MinValue
        {
            get { return _minValue; }
            set
            {
                if (_minValue != value)
                {
                    _minValue = value;
                }
            }
        }

        private Double _maxValue = 100;
        /// <summary>
        /// Gets or sets the maximum value to display on the Gauge.
        /// </summary>
        public Double MaxValue
        {
            get { return _maxValue; }
            set
            {
                if (_maxValue != value)
                {
                    _maxValue = value;
                }
            }
        }

        private Int32 _margin = 0;
        /// <summary>
        /// Gets or sets the Margins of the Control.
        /// </summary>
        public Int32 Margin
        {
            get { return _margin; }
            set
            {
                if (_margin != value)
                {
                    _margin = value;
                }
            }
        }

        private Int32 _borderWidth = 3;
        /// <summary>
        /// Gets or sets the Gaugeds Border Width
        /// </summary>
        public Int32 BorderWidth
        {
            get { return _borderWidth; }
            set
            {
                if (_borderWidth != value)
                {
                    _borderWidth = value;
                }
            }
        }

        private ProgressBarGaugeSectionCollection _sections;
        /// <summary>
        /// Gets or sets the Sections used to display the data on the Gauge.
        /// </summary>
        public ProgressBarGaugeSectionCollection Sections
        {
            get { return _sections; }
            set
            {
                if (_sections != value)
                {
                    _sections = value;
                }
            }
        }



        #endregion Properties

        #region Drawing Methods

        private void DrawBackground(Graphics pGraphics)
        {
            //Get the current Width and height. 
            //Both value must be set as pixels on the control use.
            Int32 width = (Int32)Width.Value;
            Int32 height = (Int32)Height.Value;

            //Get the Rectangle that forms the whole control.
            Rectangle bgRectangle = new Rectangle(0, 0, (Int32)Width.Value, (Int32)Height.Value);
            //Create the background brush.
            using (Brush bgBrush = new SolidBrush(BackColor))
            {
                //Fill the whole control rectangle with the Brush.
                pGraphics.FillRectangle(bgBrush, bgRectangle);

                //Create the Rectangle that contains the gauge.
                Rectangle gaugeRectangle = new Rectangle(_margin, _margin, width - _margin, height - _margin);

                using (Pen borderPen = new Pen(Color.Black, _borderWidth))
                {
                    pGraphics.DrawRectangle(borderPen, gaugeRectangle);
                }

                DrawMajorTicks(pGraphics, bgRectangle);
            }
        }

        private void DrawMajorTicks(Graphics pGraphics, Rectangle pBgRectangle)
        {
            String valueText = "";
            Double countValue = 0;
            //Get the current Width and height. 
            //Both value must be set as pixels on the control use.
            Int32 width = (Int32)Width.Value;
            Int32 height = (Int32)Height.Value;

            while (countValue <= (_maxValue - _minValue))
            {
                valueText = (_minValue + countValue).ToString();
                pGraphics.ResetTransform();
                
                //Create the Rectangle that contains the gauge.
                Rectangle majorTicksRectangle = new Rectangle(_margin, _margin-4, width - _margin, height - _margin + 4);
                pGraphics.SetClip(majorTicksRectangle);

                using (Pen majorTickPen = new Pen(Color.Black, _majorTicks.Width))
                {
                }
            }
        }

        #endregion Drawing Methods
    }
}
