/*
 * This source file (MDMFunctionParameter.cs) is part of the Cubed(tm) analysis server. It is made available under the 
 * licensing terms below, and you are only permitted to use, compile, change, re-distribute or review this source file
 * if you agree to the terms of the license.
 * 
 * Open Source License - Cubed
 * Trademarked & Copyright  2006-2008 Brendan Hayes, Bamboo Software and contributors, All Rights Reserved
 * Portions Copyright  their respective authors, All Rights Reserved
 * Cubed is released under a dual licensing model. This model is applicable in two different situations:
 * 1) In conditions where you would like to use Cubed for commercial purposes. Under these conditions, you must 
 *    contact the copyright owner of this program and obtain a commercial license. Under these circumstances it 
 *    is illegal to use the software in any way without first obtaining a license. To do so, contact Bamboo Software 
 *    using the details at the bottom of this document. Commercial purposes include, but may not be limited to, 
 *    the following:
 *      a. Selling or licensing the software for a fee to a third party; 
 *      b. Using any portion of the source code of this software in another application which is then sold 
 *      or licensed for a fee;
 *      c. Using the software (in whole or in part) in a commercial environment, where the use of the software 
 *         in any way adds value to a business or assists in earning money.
 * 2) In other conditions, where the software is not being used in a commercial environment, and is being given away 
 *    free of charge (or the source code is being used for other non-profit purposes), the software is licensed under 
 *    the use of the GNU Public License (GPL). Please see the attached licensing document for full details. 
 */
using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using System.Xml.Serialization;

namespace Bamboo.Cubed.Structures.Query
{
    /// <summary>
    /// An MDMFucntionParameter is a Parameter passed somewhere in a query that generates a result.
    /// </summary>
    public class MDMFunctionParameter : IXmlSerializable
    {
        /// <summary>
        /// Initialises a default instanace of the MDMFunctionParameter
        /// </summary>
        public MDMFunctionParameter()
        {
            paramtype = default(Type);
            param = default(object);
        }
        
        /// <summary>
        /// Initialices an instance of an MDMFunctionParameter.
        /// </summary>
        /// <param name="paramType">The Type of the parameter</param>
        /// <param name="paramObject">The object passed as a parameter. Usually a MDMObject but could be another thing</param>
        public MDMFunctionParameter(Type paramType, Object paramObject)
        {
            this.paramtype = paramType;
            this.param = paramObject;
        }        

        private Type paramtype;
        /// <summary>
        /// The type of the parameter
        /// </summary>
        [XmlIgnore()]
        public Type ParamType
        {
            get
            {
                return this.paramtype;
            }

            set
            {
                this.paramtype = value;
            }
        }

        private Object param;
        /// <summary>
        /// The parameter it self
        /// </summary>
        [XmlIgnore]
        public Object Param
        {
            get
            {
                return param;
            }
            set
            {
                this.param = value;
            }
        }

        /// <summary>
        /// Returns the XML Schema of the Function Parameter object
        /// </summary>
        /// <returns></returns>
        public System.Xml.Schema.XmlSchema GetSchema()
        {
            return null;
        }

        /// <summary>
        /// Reads an XML version of the Function Parameter object
        /// </summary>
        /// <param name="reader">The XML reader to read from</param>
        public void ReadXml(XmlReader reader)
        {
            Type type = Type.GetType(reader.GetAttribute("type"));
            reader.ReadStartElement();
            this.param = new XmlSerializer(type).Deserialize(reader);
            this.paramtype = type;
            reader.ReadEndElement();
        }

        /// <summary>
        /// Writes an XML version of the Function Parameter object
        /// </summary>
        /// <param name="writer">The XML writer to write to</param>
        public void WriteXml(XmlWriter writer)
        {
            writer.WriteAttributeString("type", param.GetType().ToString());
            new XmlSerializer(param.GetType()).Serialize(writer, param);
        }
    }
}